/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service;

import com.ibm.staf.*;


// STAFServiceInterfaceLevel2 - This class defines the Level 2 inteface for
//                              STAF Java services.
//
// This interface defines the following methods
//
//   init - This method is called once when STAF is in startup mode.  This
//          method allows the service to perform its initialization.
//
//   acceptRequest - This is the method called whenever the service needs to
//                   perform a request.  This is primary method by which
//                   programs/users interact with the service.
//
//   term - This method is called once when STAF is in shutdown mode.  This
//          method allows the service to perform any cleanup.

public interface STAFServiceInterfaceLevel2
{

    static public class InitInfo
    {
        public String name;
        public String parms;

        public InitInfo(String name, String parms)
        {
            this.name = name;
            this.parms = parms;
        }
    }

    static public class RequestInfo
    {
        public String localMachine;
        public String machine;
        public String effectiveMachine;
        public String processName;
        public int handle;
        public int trustLevel;
        public String request;

        public RequestInfo (String localMachine, String machine,
                            String effectiveMachine, String processName,
                            int handle, int trustLevel, String request)
        {
            this.machine = machine;
            this.localMachine = localMachine;
            this.effectiveMachine = effectiveMachine;
            this.processName = processName;
            this.handle = handle;
            this.trustLevel = trustLevel;
            this.request = request;
        }
    }

    int init(InitInfo initInfo);
    STAFResult acceptRequest(RequestInfo reqInfo);
    int term();
}
