#!/usr/bin/python
#############################################################################
# Software Testing Automation Framework (STAF)                              #
# (C) Copyright IBM Corp. 2001                                              #
#                                                                           #
# This software is licensed under the Common Public License (CPL) V1.0.     #
#############################################################################

import PYSTAF

# Utility functions

def STAFWrapData(message):
    return ":%d:%s" % (len(message), message)

# STAFException class

class STAFException:

    def __init__(self, rc = 0, result = ""):
        self.rc = rc
        self.result = result

# STAFResult class

class STAFResult:

    Ok                          = 0
    InvalidAPI                  = 1
    UnknownService              = 2
    InvalidHandle               = 3
    HandleAlreadyExists         = 4
    HandleDoesNotExist          = 5
    UnknownError                = 6
    InvalidRequestString        = 7
    InvalidServiceResult        = 8
    REXXError                   = 9
    BaseOSError                 = 10
    ProcessAlreadyComplete      = 11
    ProcessNotComplete          = 12
    VariableDoesNotExist        = 13
    UnResolvableString          = 14
    InvalidResolveString        = 15
    NoPathToMachine             = 16
    FileOpenError               = 17
    FileReadError               = 18
    FileWriteError              = 19
    FileDeleteError             = 20
    STAFNotRunning              = 21
    CommunicationError          = 22
    TrusteeDoesNotExist         = 23
    InvalidTrustLevel           = 24
    AccessDenied                = 25
    STAFRegistrationError       = 26
    ServiceConfigurationError   = 27
    QueueFull                   = 28
    NoQueueElement              = 29
    NotifieeDoesNotExist        = 30
    InvalidAPILevel             = 31
    ServiceNotUnregisterable    = 32
    ServiceNotAvailable         = 33
    SemaphoreDoesNotExist       = 34
    NotSemaphoreOwner           = 35
    SemaphoreHasPendingRequests = 36
    Timeout                     = 37
    JavaError                   = 38
    ConverterError              = 39
    ServiceAlreadyExists        = 40
    InvalidObject               = 41
    InvalidParm                 = 42
    RequestNumberNotFound       = 43
    InvalidAsynchOption         = 44
    RequestNotComplete          = 45
    ProcessAuthenticationDenied = 46
    InvalidValue                = 47
    DoesNotExist                = 48
    AlreadyExists               = 49
    DirectoryNotEmpty           = 50
    DirectoryCopyError          = 51
    DiagnosticsNotEnabled       = 52

    def __init__(self, rc = 0, result = ""):
        self.rc = rc
        self.result = result

# STAFHandle class

class STAFHandle:

    # STAFHandle types

    Standard = 0
    Static   = 1

    # Modes for submit call

    Synchronous   = 0
    FireAndForget = 1
    Queue         = 2
    Retain        = 3
    QueueRetain   = 4

    # Note: I would have liked to have used STAFHandle.Standard in place of "0"
    #       in the methods below, but the class hasn't been defined yet.

    def __init__(self, handleNameOrNumber, handleType = 0):

        self.handleType = handleType

        if (handleType == STAFHandle.Standard):
            rc, self.handle = PYSTAF.STAFRegister(handleNameOrNumber)
            if (rc != 0):
                raise STAFException(rc)
        else:
            self.handle = handleNameOrNumber

    def submit(self, location, service, request, mode = Synchronous):
        rc, result = PYSTAF.STAFSubmit(self.handle, mode, location, service,
                                       request)
        return STAFResult(rc, result)

    def unregister(self):
        if (self.handleType == 0):
            rc = PYSTAF.STAFUnregister(self.handle)
            if (rc != 0):
                raise STAFException(rc)
            self.handle = 0;

        return 0

# Used if called as script

if (__name__ == "__main__"):

    import sys

    if len(sys.argv) < 4:
        print "Usage: %s location service request" % sys.argv[0]
        sys.exit(1)

    location = sys.argv[1]
    service = sys.argv[2]
    request = sys.argv[3]

    for requestPart in sys.argv[4:]:
        request = request + " " + requestPart

    try:
        handle = STAFHandle("STAF/Client/Python")
    except STAFException, e:
        print "Error registering with STAF, RC: %d" % e.rc
        sys.exit(e.rc)

    result = handle.submit(location, service, request)

    if (result.rc != 0):
        print "Error submitting request, RC: %d" % result.rc

        if (len(result.result) != 0):
            print "Additional info: %s" % result.result

    else:
        print "Response"
        print "--------"
        print result.result

    rc = handle.unregister()

    sys.exit(rc)
