/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import com.ibm.staf.*;

public class STAFDemoController extends JFrame
{
    public static void main(String []argv)
    {
        STAFDemoController app = new STAFDemoController();

        // Dimension dim = app.getPreferredSize();
        // app.setSize(dim.width, dim.height);
	app.setSize(300, 200);
        app.show();
    }

    static STAFHandle handle;
    static String stafMachineName;
    static String stafEffectiveMachineName;
    static int numResources = 2;    
    
    JPanel stafStartPanel;
    String targetJavacmd;
    String targetClasspath;

    public STAFDemoController()
    {
        super("STAF Demo Controller");

        try
	{
            handle = new STAFHandle("STAFDemo_Controller");
	}
        catch(STAFException e)
	{
            System.out.println("Error registering with STAF, RC: " + e.rc);
            System.exit(e.rc);
	}

        STAFResult stafResult = handle.submit2("local", "var", "resolve " +
            "{STAF/Config/Machine}");

        if (stafResult.rc != 0)
            System.out.println("Error getting local machine name RC: "+
                stafResult.rc + " Result: " + stafResult.result);

        stafMachineName = stafResult.result;
        
        stafResult = handle.submit2("local", "var", "resolve " +
            "{STAF/Config/EffectiveMachine}");

        if (stafResult.rc != 0)
            System.out.println("Error getting local effective machine name RC:"
             + " " + stafResult.rc + " Result: " + stafResult.result);

        stafEffectiveMachineName = stafResult.result;

        if (stafResult.rc != 0)
            System.out.println("Error getting local effective machine name RC:"
                + " " + stafResult.rc + " Result: " + stafResult.result);

        stafResult = handle.submit2("local", "respool", "delete pool " +
                     "STAFDemo confirm force");
                     
        if ((stafResult.rc != 0) && (stafResult.rc != STAFResult.DoesNotExist))
            System.out.println("Error deleting STAFDemo resource pool RC: "+
                stafResult.rc + " Result: " + stafResult.result);

        stafResult = handle.submit2("local", "respool", "create pool " +
                     "STAFDemo description \"" +
                     "STAF Demo Resource Pool\"");
                     
        if (stafResult.rc != 0)
            System.out.println("Error creating STAFDemo resource pool RC: " +
                stafResult.rc + " Result: " + stafResult.result);

        for (int i = 1; i <= numResources; ++i)
        {
            stafResult = handle.submit2("local", "respool", "add pool " +
                         "STAFDemo entry Resource" + String.valueOf(i));
            if (stafResult.rc != 0)
                System.out.println("Error adding resource to  STAFDemo RC: "+
                    stafResult.rc + " Result: " + stafResult.result);
        }

        ProcessEndThread procThread = new ProcessEndThread();
        procThread.start();

        stafStartPanel = new STAFStartPanel();

        getContentPane().add(stafStartPanel);

        addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent event)
	    {
                try
	        {
                    handle.submit2("local", "queue", "queue handle " +
                                   handle.getHandle() +
                                   " message STAFDemo/Stop");
                    handle.unRegister();
		}
                catch(STAFException e)
	        {
                    System.out.println("Error unregistering with STAF, RC: " +
                                        e.rc);
		}

                dispose();
                System.exit(0);
	    }
	});
    }

    class ProcessEndThread extends Thread
    {
        public void run()
        {
            for(;;)
	    {
                STAFResult stafResult = STAFDemoController.handle.submit2(
                                        "local", "queue", "get wait");

                int index = 0;

                if ((index = stafResult.result.indexOf(
                     "STAF/PROCESS/END")) != -1)
		{
                    String message = stafResult.result.substring(index +
                                     new String("STAF/PROCESS/END ").length());

                    String theHandle = message.substring(0,
                                       message.indexOf(';'));

                     
                    // Skip past priority and timestamp
                    index = stafResult.result.indexOf(';');
                    index = stafResult.result.indexOf(';', index + 1);

                    String theMachine = stafResult.result.substring(index + 1,
                                        stafResult.result.indexOf(';',
                                        index + 1));

                    STAFDemoController.handle.submit2(theMachine, "process",
                        "free handle " + theHandle);
		}
                else if ((index = stafResult.result.indexOf(
                          "STAFDemo/Stop")) != -1)
	        {
                    return;
		}
	    }
	}
    }

    public class STAFStartPanel extends JPanel
    {
        JTextField fMachine = new JTextField(25);
        JTextField fHandle = new JTextField(25);
        JButton fStart = new JButton("Start");
        JButton fConnect = new JButton("Connect");
        JLabel fNumResourcesLabel = new JLabel();
        JButton fAddResource = new JButton("+");
        JButton fDeleteResource = new JButton("-");

        public STAFStartPanel()
        {
	    // Setup display
            GridBagLayout gbl = new GridBagLayout();
            GridBagConstraints gbc = new GridBagConstraints();

            setLayout(gbl);

            gbc.anchor = GridBagConstraints.WEST;
            gbc.fill   = GridBagConstraints.HORIZONTAL;
            gbc.gridwidth = GridBagConstraints.REMAINDER;

            // Setup Process portion of panel
            JPanel processPanel = new JPanel();
            processPanel.setBorder(new TitledBorder("Process"));
            processPanel.setLayout(new GridBagLayout());
            GridBagConstraints procGBC = new GridBagConstraints();

            procGBC.anchor = GridBagConstraints.WEST;
            procGBC.fill   = GridBagConstraints.HORIZONTAL;
            procGBC.gridwidth = 1;
            processPanel.add(new JLabel("Machine:"), procGBC);

            processPanel.add(Box.createHorizontalStrut(5));
            procGBC.gridwidth = GridBagConstraints.REMAINDER;
            fMachine.setText(stafEffectiveMachineName);
            processPanel.add(fMachine, procGBC);

            procGBC.gridwidth = 1;
            processPanel.add(fStart, procGBC);

            processPanel.add(Box.createHorizontalStrut(5));
            procGBC.gridwidth = GridBagConstraints.REMAINDER;
            processPanel.add(new JLabel("a new process"), procGBC);

            procGBC.gridwidth = 1;
            processPanel.add(fConnect, procGBC);

            processPanel.add(Box.createHorizontalStrut(5));
            processPanel.add(new JLabel("to handle:"));

            processPanel.add(Box.createHorizontalStrut(5));
            procGBC.gridwidth = GridBagConstraints.REMAINDER;
            processPanel.add(fHandle, procGBC);            

            add(processPanel, gbc);

            // Setup ResPool panel
            JPanel respoolPanel = new JPanel();
            respoolPanel.setBorder(new TitledBorder("Resource Pool"));

            respoolPanel.add(new JLabel("Number of resource:"));
            respoolPanel.add(Box.createHorizontalStrut(5));
            fNumResourcesLabel.setText(
                String.valueOf(STAFDemoController.numResources));
            respoolPanel.add(fNumResourcesLabel);
            respoolPanel.add(Box.createHorizontalStrut(10));
            respoolPanel.add(fAddResource);
            respoolPanel.add(fDeleteResource);

            add(respoolPanel, gbc);

            // Setup button actions
            fStart.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event)
		{
                    String machineName = fMachine.getText();

                    STAFResult stafResult = 
                        handle.submit2(machineName, "ping", "ping");
                        
                    if (stafResult.rc != 0)
                    {
                        String errMsg = 
                            "\nThis machine could not communicate with "
                            + "machine " + machineName + "\nRC: " + 
                            stafResult.rc + " Result: + " + stafResult.result;
                            
                        JOptionPane.showMessageDialog(stafStartPanel,
                            errMsg, "Error starting process", 
                            JOptionPane.ERROR_MESSAGE);
                            
                        return;
                    }

                    stafResult = 
                        handle.submit2(machineName, "var", "resolve " +
                        "{STAF/Config/STAFRoot}");
                        
                    String stafRoot = stafResult.result;
                    
                    stafResult = 
                        handle.submit2(machineName, "var", "resolve " +
                        "{STAF/Config/Sep/Path}");
                        
                    String sepPath = stafResult.result;
                    
                    stafResult = 
                        handle.submit2(machineName, "var", "resolve " +
                        "{STAF/Config/Sep/File}");
                        
                    String sepFile = stafResult.result;
                    
                    stafResult = 
                        handle.submit2(machineName, "var", "resolve " +
                        "{STAF/Env/CLASSPATH}");
    
                    targetClasspath = stafResult.result;

                    stafResult = 
                        handle.submit2(machineName, "var", "resolve " +
                        "{STAF/Config/OS/Name}");
            
                    String jstafDir;
                    
                    if (stafResult.result.startsWith("Win"))
                    {
                        targetJavacmd = "javaw";
                        jstafDir = "bin";
                    }
                    else
                    {
                        targetJavacmd = "java";
                        jstafDir = "lib";
                    }
                    
                    targetClasspath += sepPath + stafRoot + sepFile + "samples"
                        + sepFile + "demo" + sepFile + "STAFDemo.jar" + 
                        sepPath + stafRoot + sepFile + jstafDir + sepFile + 
                        "JSTAF.zip";

                    stafResult = 
                        handle.submit2(machineName, "fs", "query entry " +
                        stafRoot + sepFile + "samples"
                        + sepFile + "demo" + sepFile + "STAFDemo.jar");
                        
                    if (stafResult.rc == STAFResult.DoesNotExist)
                    {
                        // copy the STAFDemo.jar file
                        
                        stafResult = 
                            handle.submit2("local", "var", "resolve " +
                            "{STAF/Config/STAFRoot}");
                            
                        String localStafRoot = stafResult.result;
                            
                        stafResult = 
                            handle.submit2("local", "var", "resolve " +
                            "{STAF/Config/Sep/File}");
                        
                        String localSepFile = stafResult.result;
                        
                        stafResult = 
                            handle.submit2("local", "fs", "copy file " +
                            localStafRoot + localSepFile + "samples"
                            + localSepFile + "demo" + localSepFile 
                            + "STAFDemo.jar " + 
                            "tofile " + stafRoot + sepFile + "samples"
                            + sepFile + "demo" + sepFile + "STAFDemo.jar "
                            + "tomachine " + machineName);
                            
                        if (stafResult.rc != 0) 
                        {
                            System.out.println("Copy file RC: " + stafResult.rc +
                                " Result: " + stafResult.result);
                        }
                    }

                    stafResult =
                        handle.submit2(machineName, "PROCESS",
                        "start command " + targetJavacmd + " " + 
                        "parms STAFProcess " +
                        "var STAFDemo/ResourcePoolMachine=" + stafMachineName +
                        " var STAF/Service/Log/Mask=Error" +
                        " env CLASSPATH=" + targetClasspath + 
                        "notify onend");

                    if (stafResult.rc != 0)
                    {
                        String errMsg = "Error starting process.  RC:" + 
                            stafResult.rc + " Result:" + stafResult.result;
                            
                        if (stafResult.rc == STAFResult.AccessDenied)
                        {
                            errMsg += "\nThis error code indicates that "
                                + "machine " + machineName + " has not given"
                                + " this machine a TRUST level of 5";
                        }
                        else if (stafResult.rc == STAFResult.NoPathToMachine)
                        {
                            errMsg += "\nThis error code indicates that this"
                                + " machine could not communication with "
                                + "machine " + machineName;
                        }

                        JOptionPane.showMessageDialog(stafStartPanel,
                            errMsg, "Error starting process", 
                            JOptionPane.ERROR_MESSAGE);
                            
                        return;
                    }

                    String theHandle = stafResult.result;

                    stafResult = handle.submit2(machineName, "LOG", "delete " +
                        "machine " + machineName + " handle " + theHandle +
                        " logname STAFDemo confirm");

                    stafResult = handle.submit2(machineName, "SEM",
                        "event STAFDemo/Handle/" + theHandle +
                        "/Continue post");

                    STAFProcessFrame frame =
                        new STAFProcessFrame(machineName, theHandle);

                    frame.setSize(425, 270);
                    frame.show();
		}
            });

            fConnect.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event)
		{
                    STAFProcessFrame frame =
                        new STAFProcessFrame(fMachine.getText(),
                                             fHandle.getText());
                    frame.setSize(425, 270);
                    frame.show();
		}
            });

            fAddResource.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event)
		{
                    STAFResult stafResult = handle.submit2("local", "respool",
                        "add pool STAFDemo entry Resource" +
                        String.valueOf(STAFDemoController.numResources + 1));

                    if (stafResult.rc != 0)
                        System.out.println("Error adding resource RC: " + 
                            stafResult.rc + " Result: " + stafResult.result);

                    fNumResourcesLabel.setText(
                        String.valueOf(++STAFDemoController.numResources));
		}
            });

            fDeleteResource.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event)
		{
                    STAFResult stafResult = handle.submit2("local", "respool",
                        "remove pool STAFDemo entry Resource" +
                        String.valueOf(STAFDemoController.numResources) +
                        " confirm force");

                    if (stafResult.rc != 0)
                        System.out.println("Error removing resource RC: " + 
                            stafResult.rc + " Result: " + stafResult.result);

                    fNumResourcesLabel.setText(
                        String.valueOf(--STAFDemoController.numResources));
		}
            });
	}
    }

    public class STAFProcessFrame extends JFrame
    {
        public STAFProcessFrame(String machine, String handle)
        {
            super("Machine: " + machine + " Handle: " + handle);

            getContentPane().add(new STAFProcessPanel(this, machine, handle));

            addWindowListener(new WindowAdapter()
            {
                public void windowClosing(WindowEvent event)
	        {
                    dispose();
	        }
	    });
        }
    }

    public class STAFProcessPanel extends JPanel
    {
        String fMachine;
        String fHandle;
        JButton fStop = new JButton("Stop");
        JButton fPause = new JButton("Pause");
        JButton fContinue = new JButton("Continue");
        JTextField fMonitorRefreshRate = new JTextField(25);
        JLabel fMonitorDate = new JLabel("Unknown");
        JLabel fMonitorStatus = new JLabel("Unknown");
        JButton fSetBackground = new JButton("Set");
        JComboBox fColorList = new JComboBox();
        MonitorThread fMonitorThread;
        STAFProcessFrame fProcessFrame;
        JCheckBox fErrorLog = new JCheckBox("Error");
        JLabel fErrorCount = new JLabel("0");
        JCheckBox fWarningLog = new JCheckBox("Warning");
        JLabel fWarningCount = new JLabel("0");
        JCheckBox fInfoLog = new JCheckBox("Info");
        JLabel fInfoCount = new JLabel("0");
        JCheckBox fDebugLog = new JCheckBox("Debug");
        JLabel fDebugCount = new JLabel("0");
        JList fLogList = new JList();
        JButton fSetLogMask = new JButton("Set Log Mask");
        JButton fRefreshLog = new JButton("Refresh Totals");
        JButton fClearLog = new JButton("Clear Log");
        JButton fDisplayLog = new JButton("Display Log");

        public STAFProcessPanel(STAFProcessFrame frame, String machine,
                                String handle)
        {
            fProcessFrame = frame;
            fMachine = machine;
            fHandle = handle;

            // Setup display
            GridBagLayout gbl = new GridBagLayout();
            GridBagConstraints gbc = new GridBagConstraints();

            setLayout(gbl);

            gbc.anchor = GridBagConstraints.WEST;
            // gbc.fill = GridBagConstraints.HORIZONTAL;
            gbc.fill = GridBagConstraints.BOTH;
            gbc.gridwidth = 1;
            gbc.gridx = 1;
            gbc.gridy = 1;

            // Setup Process/Semaphore panel
            JPanel processSemPanel = new JPanel();
            processSemPanel.setBorder(new TitledBorder("Queue / Semaphore"));
            processSemPanel.add(fStop);
            processSemPanel.add(fPause);
            processSemPanel.add(fContinue);
            add(processSemPanel, gbc);

            // Setup variable panel
            JPanel varPanel = new JPanel();
            varPanel.setBorder(new TitledBorder("Variable"));
            varPanel.add(fSetBackground);
            varPanel.add(new JLabel("background color to"));
            fColorList.addItem("Blue");
            fColorList.addItem("Green");
            fColorList.addItem("Red");
            fColorList.addItem("Yellow");
            varPanel.add(fColorList);

            // gbc.gridwidth = GridBagConstraints.REMAINDER;
            gbc.gridy = 2;
            add(varPanel, gbc);

            // Setup monitor panel
            JPanel monPanel = new JPanel();
            monPanel.setLayout(new GridBagLayout());
            GridBagConstraints monGBC = new GridBagConstraints();

            monGBC.anchor = GridBagConstraints.WEST;
            monGBC.fill = GridBagConstraints.HORIZONTAL;
            monGBC.gridwidth = 1;

            monPanel.setBorder(new TitledBorder("Monitor"));

            monGBC.fill = GridBagConstraints.NONE;
            monGBC.gridwidth = 3;
            monPanel.add(new JLabel("Refresh rate (sec.):"), monGBC);
            monPanel.add(Box.createHorizontalStrut(5));

            monGBC.fill = GridBagConstraints.HORIZONTAL;
            monGBC.gridwidth = 1;
            fMonitorRefreshRate.setColumns(3);
            fMonitorRefreshRate.setText("2");
            monPanel.add(fMonitorRefreshRate, monGBC);

            monPanel.add(Box.createGlue(), monGBC);
            monGBC.gridwidth = GridBagConstraints.REMAINDER;
            monPanel.add(Box.createHorizontalStrut(80), monGBC);

            monGBC.gridwidth = 1;
            monGBC.fill = GridBagConstraints.NONE;
            monPanel.add(new JLabel("Timestamp:"), monGBC);

            monGBC.fill = GridBagConstraints.HORIZONTAL;
            monPanel.add(Box.createHorizontalStrut(5));

            monGBC.gridwidth = GridBagConstraints.REMAINDER;;
            monPanel.add(fMonitorDate, monGBC);

            monGBC.gridwidth = 1;
            monGBC.fill = GridBagConstraints.NONE;
            monPanel.add(new JLabel("Status:"), monGBC);

            monGBC.fill = GridBagConstraints.HORIZONTAL;
            monPanel.add(Box.createHorizontalStrut(5));

            monGBC.gridwidth = GridBagConstraints.REMAINDER;;
            monPanel.add(fMonitorStatus, monGBC);

            gbc.gridy = 3;
            add(monPanel, gbc);

            // Setup log panel
            JPanel logPanel = new JPanel();
            logPanel.setLayout(new GridBagLayout());
            GridBagConstraints logGBC = new GridBagConstraints();

            logGBC.anchor = GridBagConstraints.WEST;
            logGBC.fill = GridBagConstraints.HORIZONTAL;
            logGBC.gridwidth = 1;

            logPanel.setBorder(new TitledBorder("Log / Variable"));

            fErrorLog.setSelected(true);
            logPanel.add(fErrorLog, logGBC);
            logGBC.gridwidth = GridBagConstraints.REMAINDER;
            logPanel.add(fErrorCount, logGBC);
            logGBC.gridwidth = 1;
            logPanel.add(fWarningLog, logGBC);
            logGBC.gridwidth = GridBagConstraints.REMAINDER;
            logPanel.add(fWarningCount, logGBC);
            logGBC.gridwidth = 1;
            logPanel.add(fInfoLog, logGBC);
            logGBC.gridwidth = GridBagConstraints.REMAINDER;
            logPanel.add(fInfoCount, logGBC);
            logGBC.gridwidth = 1;
            logPanel.add(fDebugLog, logGBC);
            logGBC.gridwidth = GridBagConstraints.REMAINDER;
            logPanel.add(fDebugCount, logGBC);
            logPanel.add(fSetLogMask, logGBC);
            logPanel.add(fRefreshLog, logGBC);
            logPanel.add(fClearLog, logGBC);
            logPanel.add(fDisplayLog, logGBC);

            // gbc.fill = GridBagConstraints.BOTH;
            gbc.gridx = 2;
            gbc.gridy = 1;
            gbc.gridheight = 3;
            add(logPanel, gbc);

            // Add button commands
            fStop.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event)
		{
                    STAFResult stafResult =
                        STAFDemoController.handle.submit2(fMachine, "QUEUE",
                        "queue handle " + fHandle + " message STAFDemo/Stop");

                    if (stafResult.rc != 0)
                        System.out.println("Error queueing message RC: " + 
                            stafResult.rc + " Result: " + stafResult.result);
		}
            });

            fPause.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event)
		{
                    STAFResult stafResult =
                        STAFDemoController.handle.submit2(fMachine, "SEM",
                        "event STAFDemo/Handle/" + fHandle +
                         "/Continue reset");

                    if (stafResult.rc != 0)
                        System.out.println("Error pausing process RC: " + 
                            stafResult.rc + " Result: " + stafResult.result);
		}
            });

            fContinue.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event)
		{
                    STAFResult stafResult =
                        STAFDemoController.handle.submit2(fMachine, "SEM",
                        "event STAFDemo/Handle/" + fHandle +
                         "/Continue post");

                    if (stafResult.rc != 0)
                        System.out.println("Error continuing process RC: " + 
                            stafResult.rc + " Result: " + stafResult.result);
		}
            });

            fSetBackground.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event)
		{
                    STAFResult stafResult =
                        STAFDemoController.handle.submit2(fMachine, "VAR",
                        "handle " + fHandle +
                        " set STAFDemo/BackgroundColor=" +
                        (String)fColorList.getSelectedItem());

                    if (stafResult.rc != 0)
                        System.out.println("Error setting background color RC: " + 
                            stafResult.rc + " Result: " + stafResult.result);
		}
            });

            fSetLogMask.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event)
		{
                    String logMask = new String();

                    if (fErrorLog.isSelected())
                        logMask += " Error";
                    if (fWarningLog.isSelected())
                        logMask += " Warning";
                    if (fInfoLog.isSelected())
                        logMask += " Info";
                    if (fDebugLog.isSelected())
                        logMask += " Debug";

                    STAFResult stafResult =
                        STAFDemoController.handle.submit2(fMachine, "VAR",
                        "handle " + fHandle +
                        " set \"STAF/Service/Log/Mask=" + logMask + "\"");

                    if (stafResult.rc != 0)
                        System.out.println("Error setting log mask RC: " + 
                            stafResult.rc + " Result: " + stafResult.result);
		}
            });

            fRefreshLog.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event)
		{
                    STAFResult stafResult =
                        STAFDemoController.handle.submit2(fMachine, "LOG",
                        "query machine " + fMachine + " handle " + fHandle +
                        " logname STAFDemo stats");

                    if ((stafResult.rc != 0) && (stafResult.rc != 17))
                        System.out.println("Error querying log RC: " + 
                            stafResult.rc + " Result: " + stafResult.result);

                    BufferedReader logFile = new BufferedReader(
                        new StringReader(stafResult.result));

                    try
		    {
                        int index = 0;

                        for(String theLogLine = logFile.readLine();
                            theLogLine != null;
                            theLogLine = logFile.readLine())
		        {
                            if (theLogLine.startsWith("Error="))
                                fErrorCount.setText(theLogLine.substring(6));
                            else if (theLogLine.startsWith("Warning="))
                                fWarningCount.setText(theLogLine.substring(8));
                            else if (theLogLine.startsWith("Info="))
                                fInfoCount.setText(theLogLine.substring(5));
                            else if (theLogLine.startsWith("Debug="))
                                fDebugCount.setText(theLogLine.substring(6));
		        }
                    }
                    catch(IOException e)
		    { /* Do Nothing */ }
		}
            });

            fClearLog.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event)
		{
                    STAFResult stafResult =
                        STAFDemoController.handle.submit2(fMachine, "LOG",
                        "delete machine " + fMachine + " handle " + fHandle +
                        " logname STAFDemo confirm");

                    if (stafResult.rc != 0)
                        System.out.println("Error deleting log RC: " + 
                            stafResult.rc + " Result: " + stafResult.result);

                    fErrorCount.setText("0");
                    fWarningCount.setText("0");
                    fInfoCount.setText("0");
                    fDebugCount.setText("0");
		}
            });

            fDisplayLog.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event)
		{
                    STAFResult stafResult =
                        STAFDemoController.handle.submit2(fMachine, "LOG",
                        "query machine " + fMachine + " handle " + fHandle +
                        " logname STAFDemo");

                    if ((stafResult.rc != 0) && (stafResult.rc != 17))
                        System.out.println("Error querying log RC: " + 
                            stafResult.rc + " Result: " + stafResult.result);

                    BufferedReader logFile = new BufferedReader(
                        new StringReader(stafResult.result));

                    Vector logLines = new Vector();

                    try
		    {
                        for(String theLogLine = logFile.readLine();
                            theLogLine != null;
                            theLogLine = logFile.readLine())
		        {
                            Vector thisLogData = new Vector();

                            int index = theLogLine.indexOf('|');
                            thisLogData.add(theLogLine.substring(0, index));

                            index = theLogLine.indexOf('|', index + 1);
                            index = theLogLine.indexOf('|', index + 1);
                            index = theLogLine.indexOf('|', index + 1);
                            int messageIndex = theLogLine.indexOf('|',
                                               index + 1);
                            thisLogData.add(theLogLine.substring(index + 1,
                                            messageIndex));
                            thisLogData.add(theLogLine.substring(
                                            messageIndex + 1));
                            logLines.add(thisLogData);
		        }
                    }
                    catch(IOException e)
		    { /* Do Nothing */ }

                    STAFLogFrame logFrame = new STAFLogFrame(fMachine, fHandle,
                                                             logLines);
                    logFrame.setSize(500, 300);
                    logFrame.show();
		}
            });

            // Launch monitor thread
            fMonitorThread = new MonitorThread(fMachine, fHandle,
                                               fMonitorRefreshRate,
                                               fMonitorDate, fMonitorStatus);
            fMonitorThread.start();

            // Add listener so that we can stop the monitor thread
            fProcessFrame.addWindowListener(new WindowAdapter()
            {
                public void windowClosing(WindowEvent event)
	        {
                    fMonitorThread.fContinue = false;
	        }
	    });
	}
    }

    public class STAFLogFrame extends JFrame
    {
        public STAFLogFrame(String machine, String handle, Vector logLines)
        {
            super("Log for Machine: " + machine + " Handle: " + handle);

            Vector columnNames = new Vector();
            columnNames.add(new String("Timestamp"));
            columnNames.add(new String("Level"));
            columnNames.add(new String("Message"));

            JTable logTable = new JTable(logLines, columnNames);
            logTable.setRowSelectionAllowed(false);
            logTable.setColumnSelectionAllowed(false);
            JScrollPane logScroll = new JScrollPane(logTable);
            getContentPane().add(logScroll);

            addWindowListener(new WindowAdapter()
            {
                public void windowClosing(WindowEvent event)
	        {
                    dispose();
	        }
	    });
        }
    }

    public class MonitorThread extends Thread
    {
        public MonitorThread(String machine, String handle, JTextField sleep,
                             JLabel date, JLabel status)
        {
            fMachine = machine;
            fHandle = handle;
            fSleepField = sleep;
            fDateField = date;
            fStatusField = status;
	}

        String fMachine;
        String fHandle;
        JTextField fSleepField;
        JLabel fDateField;
        JLabel fStatusField;
        public boolean fContinue = true;

        public void run()
        {
            for(;;)
            {
                if (!fContinue) return;

                int sleep = 2;

                try
		{
                    sleep = Integer.parseInt(fSleepField.getText());
                    Thread.sleep(sleep * 1000);
                }
                catch(NumberFormatException e)
		{ /* Do Nothing */ }
                catch(InterruptedException e)
		{ /* Do Nothing */ }

                STAFResult stafResult =
                    STAFDemoController.handle.submit2(fMachine, "MONITOR",
                    "query machine " + fMachine + " handle " + fHandle);

                if (stafResult.rc != 0)
		{
                    fDateField.setText("Unknown");
                    fStatusField.setText("Unknown");
		}
                else
		{
                    int firstSpace = stafResult.result.indexOf(' ');
                    String dateText =
                        stafResult.result.substring(0, firstSpace);
                    String messageText =
                        stafResult.result.substring(firstSpace + 1);
                    fDateField.setText(dateText);
                    fStatusField.setText(messageText);
		}
	    }
	}
    }
}
