/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

import com.ibm.staf.*;
import com.ibm.staf.service.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;

class STAFLogViewer
{
    String helpText = "\nSTAFLogViewer Help\n\n" + 
        "-machine <machineName> -handle <logHandle> " + 
        "-name <logName>\n" + "-help\n" + "-version";
    String kVersion = "1.0.0";

    public static void main(String argv[])
    {
        new STAFLogViewer(argv);
    }
    
    public STAFLogViewer(String argv[])
    {
        if ((argv.length < 1) || (argv.length > 6))
        {
            System.out.println("Invalid parameters");
            System.out.println(helpText);
            System.exit(0);
        }
        else
        {
            if (argv[0].equalsIgnoreCase("-HELP"))
            {
                System.out.println(helpText);
                System.exit(0);
            }
            else if (argv[0].equalsIgnoreCase("-VERSION"))
            {
                System.out.println(kVersion);
                System.exit(0);
            }
            else
            {
                for (int i = 0; i < argv.length; i++)
                {
                    if (argv[i].equals("-machine"))
                    {
                        fMachine = argv[i+1];
                    }
                    else if (argv[i].equals("-handle"))
                    {
                        fLogHandle = argv[i+1];
                    }
                    else if (argv[i].equals("-name"))
                    {
                        fLogName = argv[i+1];
                    }
                }
            }
        }
                
        try
        {
            fHandle = new STAFHandle("STAFLogViewer");
            
            if (fMachine.equalsIgnoreCase("local"))
            {
                fMachine = (fHandle.submit2("local", "var", 
                    "resolve {STAF/Config/EffectiveMachine}")).result;
            }
            
            STAFResult stafResult = fHandle.submit2(fMachine, "LOG",
                "query machine " + fMachine + " handle " + fLogHandle +
                " logname " + fLogName);

            if ((stafResult.rc != 0) && (stafResult.rc != 17))
            {
                System.out.println("Error querying log.  RC=" + stafResult.rc);
                return;
            }
            
            BufferedReader logFile = new BufferedReader(
                new StringReader(stafResult.result));

            Vector logLines = new Vector();
             
            for(String theLogLine = logFile.readLine();
                theLogLine != null;
                theLogLine = logFile.readLine())
            {
                Vector thisLogData = new Vector();

                int index = theLogLine.indexOf('|');
                thisLogData.add(theLogLine.substring(0, index));

                index = theLogLine.indexOf('|', index + 1);
                index = theLogLine.indexOf('|', index + 1);
                index = theLogLine.indexOf('|', index + 1);
                int messageIndex = theLogLine.indexOf('|', index + 1);
                thisLogData.add(theLogLine.substring(index + 1, messageIndex));
                thisLogData.add(theLogLine.substring(messageIndex + 1));
                logLines.add(thisLogData);
            }
            
            STAFLogFrame logFrame = 
                new STAFLogFrame(fMachine, fLogHandle, logLines);
            logFrame.setSize(500, 300);
            logFrame.show();
            
            String osName = System.getProperties().getProperty("os.name");

            if (osName.equals("Windows 2000")) 
            {
                logFrame.setState(JFrame.ICONIFIED);
                logFrame.setState(JFrame.NORMAL);
            }
            else 
            {
                logFrame.toFront();
            }
        }
        catch(STAFException e)
        {
            System.out.println("Error registering with STAF, RC: " + e.rc);
            System.exit(e.rc);
        }
        catch(IOException e)
        {
            e.printStackTrace();
        }
    }
    
    public class STAFLogFrame extends JFrame
    {
        public STAFLogFrame(String machine, String handle, Vector logLines)
        {
            super("Log for Machine: " + machine + " Handle: " + handle);

            Vector columnNames = new Vector();
            columnNames.add(new String("Timestamp"));
            columnNames.add(new String("Level"));
            columnNames.add(new String("Message"));

            JTable logTable = new JTable(logLines, columnNames);
            logTable.setRowSelectionAllowed(false);
            logTable.setColumnSelectionAllowed(false);
            JScrollPane logScroll = new JScrollPane(logTable);
            getContentPane().add(logScroll);

            addWindowListener(new WindowAdapter()
            {
                public void windowClosing(WindowEvent event)
	        {
                    dispose();
                    System.exit(0);
	        }
	    });
        }
    }
        
    private STAFHandle fHandle;
    private String fMachine;
    private String fLogHandle;
    private String fLogName;
}
