/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service.event;

import java.util.*;
import java.io.*;

public class Client
implements Serializable
{
    static final long serialVersionUID = 1;
    transient double version = 0.90;
    transient double fileVersion;
    transient Who fWho;
    transient How fHow;

    public class Who
    {
        transient String machineName = "";
        transient String processName = "";
        transient int handle = 0;

        public Who()
        { /* Default */ }

        public Who(String machineName, String processName)
        {
            this.machineName = machineName;
            this.processName = processName;
            this.handle = 0;
        }

        public Who(String machineName, int handle)
        {
            this.machineName = machineName;
            this.processName = "";
            this.handle = handle;
        }
    }

    public class How
    {
        transient int maxAttempts = 1;
        transient long timeout = 60000;
        transient int priority = 5;
        transient int priorityDelta = 1;

        public How()
        { /* Default */ }

        public How(int maxAttempts, long timeout, int priority, int delta)
        {
            this.maxAttempts = maxAttempts;
            this.timeout = timeout;
            this.priority = priority;
            this.priorityDelta = delta;
        }
    }

    public Client(Client c)
    { 
        fWho = c.fWho;
        fHow = new How(c.fHow.maxAttempts, c.fHow.timeout, c.fHow.priority, 
                       c.fHow.priorityDelta);
    }

    public Client(String machineName, String processName)
    {
        fWho = new Who(machineName, processName);
        fHow = new How();
    }

    public Client(String machineName, int handle)
    {
        fWho = new Who(machineName, handle);
        fHow = new How();
    }

    public Client(String machineName, String processName, int maxAttempts,
                  long timeout, int priority, int delta)
    {
        fWho = new Who(machineName, processName);
        fHow = new How(maxAttempts, timeout, priority, delta);
    }

    public Client(String machineName, int handle, int maxAttempts,
                  long timeout, int priority, int delta)
    {
        fWho = new Who(machineName, handle);
        fHow = new How(maxAttempts, timeout, priority, delta);
    }

    private void writeObject(ObjectOutputStream stream) throws IOException
    {
        stream.writeDouble(version);
        stream.writeObject(fWho.machineName);
        stream.writeObject(fWho.processName);
        stream.writeInt(fWho.handle);

        stream.writeInt(fHow.maxAttempts);
        stream.writeLong(fHow.timeout);
        stream.writeInt(fHow.priority);
        stream.writeInt(fHow.priorityDelta);
    }

    private void readObject(ObjectInputStream stream)
    throws IOException, ClassNotFoundException
    {   
        fWho = new Who();
        fHow = new How();
        
        version = 0.90;
        fileVersion = stream.readDouble();
        fWho.machineName = (String)stream.readObject();
        fWho.processName = (String)stream.readObject();
        fWho.handle = stream.readInt();

        fHow.maxAttempts = stream.readInt();
        fHow.timeout = stream.readLong();
        fHow.priority = stream.readInt();
        fHow.priorityDelta = stream.readInt();
    }      
	
    public long getTimeout() { return fHow.timeout; }
    public String getMachineName() { return fWho.machineName; }
    public int getHandle() { return fWho.handle; }
    public String getProcessName() { return fWho.processName; }
    public int getMaxAttempts() { return fHow.maxAttempts; }
    public int getPriority() { return fHow.priority; }

    public String toString()
    {
        StringBuffer result = new StringBuffer();
        
        if (fWho.handle == 0)
        {
            result.append(fWho.machineName + ";" + fWho.processName + ";" + 
                          fHow.maxAttempts + ";" + fHow.timeout + ";" +
                          fHow.priority + ";" + fHow.priorityDelta);
        }
        else
        {
            result.append(fWho.machineName + ";" + fWho.handle + ";" + 
                          fHow.maxAttempts + ";" + fHow.timeout + ";" +
                          fHow.priority + ";" + fHow.priorityDelta);
        }

        return result.toString();
    }

    public String longToString()
    {
        StringBuffer result = new StringBuffer();
     
        if (fWho.handle == 0)
        {  
            result.append("Machine:            " + fWho.machineName +
                          "\nProcess:            " + fWho.processName +
                          "\nMax Attempts:       " + fHow.maxAttempts +
                          "\nTimeout:            " + fHow.timeout +
                          "\nPriority:           " + fHow.priority +
                          "\nPriority Delta:     " + fHow.priorityDelta);
        }
        else
        {
            result.append("Machine:            " + fWho.machineName +
                          "\nHandle:             " + fWho.handle +
                          "\nMax Attempts:       " + fHow.maxAttempts +
                          "\nTimeout:            " + fHow.timeout +
                          "\nPriority:           " + fHow.priority +
                          "\nPriority Delta:     " + fHow.priorityDelta);
        }

        return result.toString(); 
    }                

    public boolean equals(Object o)
    {
        Client c = (Client)o;

        // if c is registered by name (i.e. handle = 0, processName != null)
        // then only compare processName, otherwise only compare handle, but
        // always ignore how data since this can be different for a client

        if (c == this) return true;
        if (!(c instanceof Client)) return false;
        if (!(c.fWho.machineName.equals(this.fWho.machineName))) return false;

        if ((fWho.handle == 0) && (fWho.processName.equals(c.fWho.processName)))
        {
            return true;
        }

        if ((fWho.handle != 0) && (fWho.handle == c.fWho.handle)) return true;

        return false;
    }
}
