/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2002                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service.stax;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.lang.Math.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import java.lang.reflect.*;
import com.ibm.staf.*;
import com.ibm.staf.service.*;
import java.text.SimpleDateFormat;
import java.util.jar.*;

public class STAXMonitorFrame extends JFrame implements ActionListener,
                                                        Runnable, 
                                                        KeyListener,
                                                        MouseListener,
                                                        TreeSelectionListener,
                                                        ChangeListener
{
    public static final int EXTENSION_ACTIVE = 1;
    public static final int EXTENSION_STATUS = 2;
    public static final int EXTENSION_INFO = 3;
    public static final int EXTENSION_ACTIVE_JOB_ELEMENTS = 4;
    public static final int EXTENSION_ACTIVE_JOB_ELEMENTS_AND_ACTIVE = 5;
    public static final int EXTENSION_ACTIVE_JOB_ELEMENTS_AND_STATUS = 6;
    public static final int EXTENSION_ACTIVE_JOB_ELEMENTS_AND_INFO = 7;
    
    public static final int AUTOMONITOR_ALWAYS = 1;
    public static final int AUTOMONITOR_RECOMMENDED = 2;
    public static final int AUTOMONITOR_NEVER = 3;
    
    public static final int DISABLED = 0;
    public static final int ENABLED = 1;
    public static final int DEFAULT = 2;

    // STAX Service Error Codes
    static final int BlockNotHeld = 4002;
    static final int BlockAlreadyHeld = 4003;

    String fLocalMachineName;
    String fStaxMachine;
    String fStaxServiceName;
    String fEventMachine;
    String fEventServiceName;
    String fJobNumber;
    String fXmlFileName;
    String fXmlFileMachine;
    String fFunction;
    String fArgs;
    String fJobName;
    Vector fScripts;
    Vector fScriptFilesVector;
    String fScriptFilesMachineName;
    static String kMonitorFrameTitle = "STAX Job Monitor";
    JLabel fCurrentSelection;
    STAXMonitorTreeNode fCurrentSelectedNode;
    JTable fDetailsTable;
    STAXMonitorTableModel fDetailsTableModel;    
    STAXMonitorTableSorter fDetailsModelSorter;
    JTree fMonitorTree;
    DefaultTreeModel fMonitorTreeModel;    
    STAFHandle fHandle;
    private Thread fQueueThread;    
    JTable fMessageTable;
    JScrollPane fMessageScrollPane;
    STAXMonitorTableModel fMessageTableModel;    
    STAXMonitorTableSorter fMessageModelSorter;
    Hashtable fMonitorTreeBlocks = new Hashtable(); // key is block name
    Hashtable fMonitorTreeBlockNodes = new Hashtable(); // key is block node    
    Hashtable fCmdIDHashtable = new Hashtable();
    Hashtable fSubjobIDHashtable = new Hashtable();
    Hashtable fProcIDHashtable = new Hashtable();
    Hashtable fPluginNodeHashtable = new Hashtable();
    Hashtable fPluginNodeToIDHashtable = new Hashtable();
    Hashtable fPluginNodeToNodeTextHashtable = new Hashtable();
    Hashtable fMonitorTreeProcessNodes = new Hashtable();
    Hashtable fMonitorTreeCommandNodes = new Hashtable();
    Hashtable fMonitorTreeSubjobNodes = new Hashtable();
    Hashtable fMonitorTreeProcessStartTimes = new Hashtable();
    Hashtable fMonitorTreeCommandStartTimes = new Hashtable();
    Hashtable fMonitorTreeSubjobStartTimes = new Hashtable();
    boolean fMonitorStopped = false;
    JPanel fSelectionDetailsPanel;
    Hashtable fDataHashtable = new Hashtable();
    Vector fDataColumns;
    Vector fMessageColumns;
    STAFProcessMonitor fProcessMonitor;
    MonitorElapsedTime fElapsedTime;
    boolean fContinueProcessMonitor = true;
    boolean fContinueElapsedTime = true;
    JPopupMenu fBlockPopupMenu = new JPopupMenu();
    JMenuItem fBlockHoldMenuItem = new JMenuItem("Hold");
    JMenuItem fBlockReleaseMenuItem = new JMenuItem("Release");
    JMenuItem fBlockTerminateMenuItem = new JMenuItem("Terminate");
    JPopupMenu fSubjobPopupMenu = new JPopupMenu();
    JMenuItem fSubjobStartMonitoringMenuItem = new 
        JMenuItem("Start Monitoring");
    JMenuItem fSubjobDisplayJobLogMenuItem = new 
        JMenuItem("Display Job Log");
    JMenuItem fSubjobDisplayJobUserLogMenuItem = new 
        JMenuItem("Display Job User Log");
    JMenuItem fSubjobTerminateMenuItem = new JMenuItem("Terminate Job");
    STAXMonitor fParentMonitor;
    private String fLocalLineSep; // line separator on the STAX Monitor machine
    private String fSTAXLineSep; // line separator used on the STAX Service machine
    TreePath previousTreePath;
    JMenu fFileMenu;
    JMenu fDisplayMenu;
    JMenu fViewMenu;
    JMenuItem fFileResubmit;
    JMenuItem fFileExitAndResubmit;
    JMenuItem fFileExit;
    JMenuItem fDisplayJobLog = new JMenuItem("Display Job Log");
    JMenuItem fDisplayJobUserLog = new JMenuItem("Display Job User Log");
    boolean fShowNoSTAXMonitorInformation = false;
    int fClearLogs = STAXMonitorFrame.DEFAULT;
    int fLogTCElapsedTime = STAXMonitorFrame.DEFAULT;
    int fLogTCNumStarts = STAXMonitorFrame.DEFAULT;
    int fLogTCStartStop = STAXMonitorFrame.DEFAULT;
    int fAutoMonitorSubjobs = STAXMonitorFrame.AUTOMONITOR_NEVER;
    String fLimitMessages = "";
    Hashtable fRegisteredPlugins = new Hashtable();
    JCheckBoxMenuItem fProcessMenu;
    JCheckBoxMenuItem fSTAFCmdMenu;
    JCheckBoxMenuItem fSubjobMenu;
    JCheckBoxMenuItem fTestcaseInfoMenu;
    JCheckBoxMenuItem fTestcaseStatusMenu;
    JCheckBoxMenuItem fActiveJobElementsMenu;
    JCheckBoxMenuItem fCurrentSelectionMenu;
    JCheckBoxMenuItem fMessagesMenu;
    Hashtable fViewablePlugins = new Hashtable();
    Hashtable fViewableComponents = new Hashtable();
    JTabbedPane fActiveElementsPane = new JTabbedPane();
    JTabbedPane fStatusPane = new JTabbedPane();
    JTabbedPane fInfoPane = new JTabbedPane();
    Vector fPluginJarsVector;
    Vector fExternalPluginNotificationTypes = new Vector();
    Hashtable fPluginNodeDetailsHashtable = new Hashtable();
    STAXMonitorProcessExtension processTablePlugin;
    STAXMonitorExtension stafcmdTablePlugin;
    STAXMonitorExtension subjobTablePlugin;
    STAXMonitorExtension testcaseTablePlugin;
    STAXMonitorExtension testcaseInfoTablePlugin;
   
    public STAXMonitorFrame()
    {
    }

    public STAXMonitorFrame(STAXMonitor parentMonitor, String staxMach, 
                            String staxService, String jobNum,
                            String eventMach, String eventService,
                            boolean showNoMonitorInfo, String limitMessages,
                            Vector pluginJarsVector, 
                            int autoMonitorSubjobs) throws STAFException
    {
        this(false, parentMonitor, staxMach, staxService, jobNum, 
             eventMach, eventService, "", "", "", "", "", null, 
             showNoMonitorInfo, limitMessages, pluginJarsVector, null, "",
             STAXMonitorFrame.DEFAULT, STAXMonitorFrame.DEFAULT,
             STAXMonitorFrame.DEFAULT, STAXMonitorFrame.DEFAULT,
             autoMonitorSubjobs);
    }
    
    public STAXMonitorFrame(STAXMonitor parentMonitor, String staxMachine, 
                            String staxServiceName, String jobNumber,
                            String eventMachine, String eventServiceName, 
                            String xmlFileName, String xmlFileMachine, 
                            String function, String args, String jobName, 
                            Vector scripts, boolean showNoMonitorInfo,
                            String limitMessages, Vector pluginJarsVector, 
                            Vector scriptFilesVector, 
                            String scriptFilesMachineName,
                            int clearLogs, int logTCElapsedTime,
                            int logTCNumStarts, int logTCStartStop,
                            int autoMonitorSubjobs) throws STAFException
    {
        this(true, parentMonitor, staxMachine, staxServiceName, jobNumber,
             eventMachine, eventServiceName, xmlFileName, xmlFileMachine,
             function, args, jobName, scripts, showNoMonitorInfo, 
             limitMessages, pluginJarsVector, scriptFilesVector, 
             scriptFilesMachineName, clearLogs, logTCElapsedTime,
             logTCNumStarts, logTCStartStop, autoMonitorSubjobs);
    }

    public STAXMonitorFrame(boolean newJob,
                            STAXMonitor parentMonitor, String staxMachine, 
                            String staxServiceName, String jobNumber,
                            String eventMachine, String eventServiceName, 
                            String xmlFileName, String xmlFileMachine, 
                            String function, String args, String jobName, 
                            Vector scripts, 
                            boolean showNoMonitorInfo,
                            String limitMessages,
                            Vector pluginJarsVector, Vector scriptFilesVector,
                            String scriptFilesMachineName,
                            int clearLogs,
                            int autoMonitorSubjobs) throws STAFException
    {
        this(newJob, parentMonitor, staxMachine, staxServiceName, jobNumber,
             eventMachine, eventServiceName, xmlFileName, xmlFileMachine,
             function, args, jobName, scripts, showNoMonitorInfo, 
             limitMessages, pluginJarsVector, scriptFilesVector, 
             scriptFilesMachineName, clearLogs, STAXMonitorFrame.DEFAULT,
             STAXMonitorFrame.DEFAULT, STAXMonitorFrame.DEFAULT, 
             autoMonitorSubjobs);
    }

    public STAXMonitorFrame(boolean newJob,
                            STAXMonitor parentMonitor, String staxMachine, 
                            String staxServiceName, String jobNumber,
                            String eventMachine, String eventServiceName, 
                            String xmlFileName, String xmlFileMachine, 
                            String function, String args, String jobName, 
                            Vector scripts, 
                            boolean fShowNoSTAXMonitorInformation,
                            String limitMessages,
                            Vector pluginJarsVector, Vector scriptFilesVector,
                            String scriptFilesMachineName,
                            int clearLogs, int logTCElapsedTime,
                            int logTCNumStarts, int logTCStartStop,
                            int autoMonitorSubjobs) throws STAFException
    {
        fPluginJarsVector = pluginJarsVector;
        fParentMonitor = parentMonitor;
        fStaxMachine = staxMachine;
        fStaxServiceName = staxServiceName;
        fJobNumber = jobNumber;
        fEventMachine = eventMachine;
        fLimitMessages = limitMessages;
        
        if (fEventMachine.equals(""))
        {
            fEventMachine = "local";
        }

        fEventServiceName = eventServiceName;
        fXmlFileName = xmlFileName;
        fXmlFileMachine = xmlFileMachine;
        fFunction = function;
        fArgs = args;
        fJobName = jobName;
        fScripts = scripts;
        fScriptFilesVector = scriptFilesVector;
        fScriptFilesMachineName = scriptFilesMachineName;
        fClearLogs = clearLogs;
        fLogTCElapsedTime = logTCElapsedTime;
        fLogTCNumStarts = logTCNumStarts;
        fLogTCStartStop = logTCStartStop;
        fAutoMonitorSubjobs = autoMonitorSubjobs;
    
        if (newJob)
        {
            String request = "EXECUTE HOLD file " + 
                             STAFUtil.wrapData(fXmlFileName);
                             
            if (!fXmlFileMachine.equals(""))
            {
                request += " MACHINE " + STAFUtil.wrapData(fXmlFileMachine);
            }
    
            if (!fFunction.equals(""))
            {
                request += " FUNCTION " + STAFUtil.wrapData(fFunction);
            }
            
            if (!fArgs.equals(""))
            {
                request += " ARGS " + STAFUtil.wrapData(fArgs);
            }
            
            if (!fJobName.equals(""))
            {
                request += " JOBNAME " + STAFUtil.wrapData(fJobName);
            }       
            
            if (fScripts != null)
            {
                if (!fScripts.isEmpty())
                {
                    for (int i=0; i < fScripts.size(); i++)
                    {
                        request += " SCRIPT " + STAFUtil.wrapData(
                                              (String)fScripts.elementAt(i));
                    }
                }
            }
            
            if (!fScriptFilesMachineName.equals(""))
            {
                request += " SCRIPTFILEMACHINE " + 
                    STAFUtil.wrapData(fScriptFilesMachineName);
            }
            
            if (fScriptFilesVector != null)
            {
                if (!fScriptFilesVector.isEmpty())
                {
                    for (int i=0; i < fScriptFilesVector.size(); i++)
                    {
                        request += " SCRIPTFILE " + STAFUtil.wrapData(
                            (String)fScriptFilesVector.elementAt(i));
                    }
                }
            }
            
            if (fClearLogs == STAXMonitorFrame.ENABLED)
                request += " CLEARLOGS Enabled";
            else if (fClearLogs == STAXMonitorFrame.DISABLED)
                request += " CLEARLOGS Disabled";
            
            if (fLogTCElapsedTime == STAXMonitorFrame.ENABLED)
                request += " LOGTCELAPSEDTIME Enabled";
            else if (fLogTCElapsedTime == STAXMonitorFrame.DISABLED)
                request += " LOGTCELAPSEDTIME Disabled";

            if (fLogTCNumStarts == STAXMonitorFrame.ENABLED)
                request += " LOGTCNUMSTARTS Enabled";
            else if (fLogTCNumStarts == STAXMonitorFrame.DISABLED)
                request += " LOGTCNUMSTARTS Disabled";

            if (fLogTCStartStop == STAXMonitorFrame.ENABLED)
                request += " LOGTCSTARTSTOP Enabled";
            else if (fLogTCStartStop == STAXMonitorFrame.DISABLED)
                request += " LOGTCSTARTSTOP Disabled";

            try
            {                
                // this handle will be used only to submit the EXECUTE request                
                fHandle = STAXMonitorUtil.getNewSTAFHandle("STAX/JobMonitor/Execute");
            }
            catch (STAFException e)
            {
                throw e;            
            }
    
            STAFResult executeResult =
                fHandle.submit2(fStaxMachine, fStaxServiceName, request);
    
            STAXMonitorUtil.freeHandle(fHandle.getHandle());

            if (executeResult.rc != 0)
            {
                throw new STAFException(executeResult.rc,                                     
                                        "Error starting Job Monitor \n" 
                                        + executeResult.result + "\n" + 
                                        request);
            }
            else
            {
                fJobNumber = executeResult.result;                
            }
        }
        
        try
        {            
            fHandle = STAXMonitorUtil.getNewSTAFHandle(
                "STAX/JobMonitor/" + fStaxMachine + "/" + fJobNumber);
                                     
            fLocalLineSep = (fHandle.submit2("local", "var", 
                            "resolve {STAF/Config/Sep/Line}")).result;
                           
            fSTAXLineSep = (fHandle.submit2(fStaxMachine, "var", 
                           "resolve {STAF/Config/Sep/Line}")).result;
        }
        catch (STAFException e)
        {
            throw e;            
        }

        boolean jobAlreadyHeld = false;
        
        if (!newJob)
        {
            String request = "HOLD JOB " + fJobNumber;
            STAFResult holdResult = fHandle.submit2(fStaxMachine, 
    	                         fStaxServiceName, request);
    
            if (holdResult.rc != 0)
            {
                if (holdResult.rc == BlockAlreadyHeld)
                {
                    jobAlreadyHeld = true;
                }
                else
                {
                    throw new STAFException(holdResult.rc,                                     
                                        "Error holding Job\n" 
                                        + holdResult.result);
                }
            }
        }        

        setTitle(kMonitorFrameTitle);
        addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                if (!fMonitorStopped)
                {
                    stopMonitor();
                }
                closeMonitor();
            }
        });

        fLocalMachineName = (fHandle.submit2("local", "var",
                           "resolve {STAF/Config/EffectiveMachine}")).result;
   
        fActiveElementsPane.addChangeListener(this);
        fStatusPane.addChangeListener(this);
        fInfoPane.addChangeListener(this);

        JMenuBar mainMenuBar = new JMenuBar();
        setJMenuBar(mainMenuBar);
        fFileMenu = new JMenu("File");
        mainMenuBar.add(fFileMenu);
        fDisplayMenu = new JMenu("Display");
        mainMenuBar.add(fDisplayMenu);
        fViewMenu = new JMenu("View");
        mainMenuBar.add(fViewMenu);
        fFileResubmit = new JMenuItem("Resubmit Job");
        fFileResubmit.addActionListener(this);
        fFileExitAndResubmit = new JMenuItem("Exit and Resubmit Job");
        fFileExitAndResubmit.addActionListener(this);
        fFileExit = new JMenuItem("Exit");
        fFileExit.addActionListener(this);
        fFileMenu.add(fFileResubmit);
        fFileMenu.add(fFileExitAndResubmit);
        fFileMenu.insertSeparator(2);
        fFileMenu.add(fFileExit);
        fDisplayMenu.add(fDisplayJobLog);
        fDisplayMenu.add(fDisplayJobUserLog);
        fDisplayJobLog.addActionListener(this);
        fDisplayJobUserLog.addActionListener(this);

        fDataColumns = new Vector();
        fDataColumns.addElement(" Name ");
        fDataColumns.addElement(" Value ");
        
        fMessageColumns = new Vector();
        fMessageColumns.addElement(" Timestamp ");
        fMessageColumns.addElement(" Message ");

        getContentPane().setLayout(new BorderLayout());
        getContentPane().setBackground(Color.white);       

        fSelectionDetailsPanel = new JPanel();
        fSelectionDetailsPanel.setLayout(new BorderLayout());
      
        JPanel selectionButtonPanel = new JPanel();        
        selectionButtonPanel.setLayout(new 
                                       FlowLayout(FlowLayout.LEFT, 0, 0));
                                             
        JPanel detailsTablePanel = new JPanel();
        detailsTablePanel.setLayout(new BorderLayout());

        fDetailsTableModel = new STAXMonitorTableModel(fDataColumns, 0);
        fDetailsModelSorter = new STAXMonitorTableSorter(fDetailsTableModel);
        fDetailsTable = new JTable(fDetailsModelSorter);
        fDetailsModelSorter.addMouseListenerToHeaderInTable(fDetailsTable, 1);
        fDetailsTable.setVisible(false);
        fDetailsTable.getTableHeader().setVisible(false);

        updateDetailsTableRenderers();
        
        JPanel messagesPanel = new JPanel();
        messagesPanel.setLayout(new BorderLayout());
        
        fMessageTableModel = new STAXMonitorTableModel(fMessageColumns, 0);
        fMessageModelSorter = 
            new STAXMonitorTableSorter(fMessageTableModel, 0);
        fMessageTable = new JTable(fMessageModelSorter);        
        fMessageModelSorter.addMouseListenerToHeaderInTable(fMessageTable, 1);

        fMessageTable.getColumnModel().getColumn(0).setCellRenderer(
            new STAXMonitorTableCellRenderer(Color.black));
    	  
        fMessageTable.getColumnModel().getColumn(0).
            setHeaderRenderer(new STAXMonitorTableCellRenderer(Color.black, 
                true, new Font("Dialog", Font.BOLD, 12)));
                                  
        fMessageTable.getColumnModel().getColumn(1).setCellRenderer(
            new STAXMonitorTableCellRenderer(Color.black));
    	  
        fMessageTable.getColumnModel().getColumn(1).
            setHeaderRenderer(new STAXMonitorTableCellRenderer(Color.black,
                true, new Font("Dialog", Font.BOLD, 12)));

        fMessageScrollPane = new JScrollPane(fMessageTable);        
        
        fMessageScrollPane.setVerticalScrollBarPolicy(
            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        fMessageScrollPane.setHorizontalScrollBarPolicy(
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        
        fMessageTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
                                  
        STAXMonitorUtil.sizeColumnsToFitText(fMessageTable);

        fInfoPane.add("Messages", fMessageScrollPane);
        fViewableComponents.put("Messages", fMessageScrollPane);

        JScrollPane detailsScrollPane = new JScrollPane(fDetailsTable);        
        
        detailsScrollPane.setVerticalScrollBarPolicy(
            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        detailsScrollPane.setHorizontalScrollBarPolicy(
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        
        fDetailsTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);       

        STAXMonitorUtil.sizeColumnsToFitText(fDetailsTable);         
        
        fCurrentSelection = new JLabel("");
        fCurrentSelection.setFont(new Font("Dialog", Font.BOLD, 14));
        fCurrentSelection.setForeground(new Color(23, 0, 122));
        
        fSelectionDetailsPanel.add(BorderLayout.NORTH, fCurrentSelection);
        
        JPanel selectionDetailsPanel1 = new JPanel();
        selectionDetailsPanel1.setLayout(new BorderLayout());
        fSelectionDetailsPanel.add(BorderLayout.CENTER, detailsScrollPane);
        
        int index = fInfoPane.getTabCount();
        
        fInfoPane.setBackgroundAt(index - 1, Color.white);
        fInfoPane.setForegroundAt(index - 1, Color.darkGray);
   
        fMonitorTree = new JTree();
        fMonitorTree.setCellRenderer(new 
            STAXMonitorTreeCellRenderer(fShowNoSTAXMonitorInformation));
        fMonitorTree.putClientProperty("JTree.lineStyle", "Angled");
        fMonitorTree.setShowsRootHandles(true);
        fMonitorTree.addTreeSelectionListener(this);
        fMonitorTreeModel = (DefaultTreeModel) fMonitorTree.getModel();
        fMonitorTreeModel.setRoot(new STAXMonitorTreeNode());
        fMonitorTree.addMouseListener(this);

        JPanel treeAndSelectionPanel = new JPanel();
        treeAndSelectionPanel.setLayout(new BorderLayout());

        fActiveJobElementsMenu = new JCheckBoxMenuItem(
            "Active Job Elements", true);
        fActiveJobElementsMenu.addActionListener(this);
        fViewMenu.add(fActiveJobElementsMenu);
        
        JScrollPane scrollPane = new JScrollPane(fMonitorTree);
        fActiveElementsPane.add("Active Job Elements", scrollPane);
        
        fActiveElementsPane.setForegroundAt(0, Color.black);
        
        fViewableComponents.put("Active Job Elements", scrollPane);
               
        JSplitPane horizPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
            fActiveElementsPane, fStatusPane);
        horizPane.setOneTouchExpandable(true);
              
        JSplitPane vertPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
            horizPane, fInfoPane);
        vertPane.setOneTouchExpandable(true);
    
        JPanel monitorPanel = new JPanel();
        monitorPanel.setLayout(new BorderLayout());
        monitorPanel.add(vertPane);
  
        fSelectionDetailsPanel.setVisible(true);
        
        // Handle extensions
        try 
        {
            processTablePlugin = new STAXMonitorProcessExtension();
            JComponent processTable = 
                processTablePlugin.init(this, newJob, fStaxMachine, 
                fStaxServiceName, fJobNumber);
            processTable.setVisible(true);
            
            Vector notificationTypes = new Vector();
                
            StringTokenizer types = new 
                StringTokenizer(processTablePlugin.getNotificationEventTypes(),
                " ");
                    
            while (types.hasMoreElements())
            {
                String type = types.nextToken().toLowerCase();
                notificationTypes.add(type);
            }
                
            fRegisteredPlugins.put(processTablePlugin, notificationTypes);
                
            fProcessMenu = new JCheckBoxMenuItem(
                processTablePlugin.getTitle(), true);
            fProcessMenu.addActionListener(this);
            fViewMenu.add(fProcessMenu);
            
            fViewablePlugins.put(fProcessMenu, processTablePlugin);
            
            JScrollPane procScrollPane = 
                new JScrollPane(processTablePlugin.getComponent());
            procScrollPane.setVerticalScrollBarPolicy(
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
            procScrollPane.setHorizontalScrollBarPolicy(
                JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
            
            fActiveElementsPane.addTab(processTablePlugin.getTitle(),
                procScrollPane);
                
            fViewableComponents.put(processTablePlugin.getTitle(), 
                procScrollPane);
                      
            int i = fActiveElementsPane.getTabCount();
            fActiveElementsPane.setBackgroundAt(i - 1, Color.white);
            fActiveElementsPane.setForegroundAt(i - 1, Color.darkGray);
        }
        catch (STAFException ex)
        {
            System.out.println("STAFException thrown during " + 
                "STAXMonitorProcessTablePlugin initialization.  RC: " +
                ex.rc);
        }
            
        try
        {
            stafcmdTablePlugin = new STAXMonitorSTAFCmdExtension();
            JComponent stafcmdTable = 
                stafcmdTablePlugin.init(this, newJob, fStaxMachine,
                fStaxServiceName, fJobNumber);
            stafcmdTable.setVisible(true);
            
            Vector notificationTypes = new Vector();
                
            StringTokenizer types = new 
                StringTokenizer(stafcmdTablePlugin.getNotificationEventTypes(),
                " ");
                    
            while (types.hasMoreElements())
            {
                String type = types.nextToken().toLowerCase();
                notificationTypes.add(type);
            }
            
            fRegisteredPlugins.put(stafcmdTablePlugin, notificationTypes);
                
            fSTAFCmdMenu = new JCheckBoxMenuItem(
                stafcmdTablePlugin.getTitle(), true);
            fSTAFCmdMenu.addActionListener(this);
            fViewMenu.add(fSTAFCmdMenu);
            
            JScrollPane stafcmdScrollPane = 
                new JScrollPane(stafcmdTablePlugin.getComponent());
            stafcmdScrollPane.setVerticalScrollBarPolicy(
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
            stafcmdScrollPane.setHorizontalScrollBarPolicy(
                JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
            
            fActiveElementsPane.addTab(stafcmdTablePlugin.getTitle(),
                stafcmdScrollPane);
                
            fViewableComponents.put(stafcmdTablePlugin.getTitle(),
                stafcmdScrollPane);

            fViewablePlugins.put(fSTAFCmdMenu, stafcmdTablePlugin);
            
            int i = fActiveElementsPane.getTabCount();
            fActiveElementsPane.setBackgroundAt(i - 1, Color.white);
            fActiveElementsPane.setForegroundAt(i - 1, Color.darkGray);
        }
        catch (STAFException ex)
        {
            System.out.println("STAFException thrown during " + 
                "STAXMonitorSTAFCmdTablePlugin initialization.  RC: " +
                ex.rc);
        }
        
        try
        {
            subjobTablePlugin = new STAXMonitorSubjobExtension();
            JComponent subjobTable = 
                subjobTablePlugin.init(this, newJob, fStaxMachine,
                    fStaxServiceName, fJobNumber);
            subjobTable.setVisible(true);
            
            Vector notificationTypes = new Vector();
                
            StringTokenizer types = new 
                StringTokenizer(subjobTablePlugin.getNotificationEventTypes(),
                " ");
                    
            while (types.hasMoreElements())
            {
                String type = types.nextToken().toLowerCase();
                notificationTypes.add(type);
            }
            
            fRegisteredPlugins.put(subjobTablePlugin, notificationTypes);
                
            fSubjobMenu = new JCheckBoxMenuItem(
                subjobTablePlugin.getTitle(), true);
            fSubjobMenu.addActionListener(this);
            fViewMenu.add(fSubjobMenu);
            
            JScrollPane subjobScrollPane = 
                new JScrollPane(subjobTablePlugin.getComponent());
            subjobScrollPane.setVerticalScrollBarPolicy(
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
            subjobScrollPane.setHorizontalScrollBarPolicy(
                JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
            
            fActiveElementsPane.addTab(subjobTablePlugin.getTitle(),
                subjobScrollPane);
                
            fViewableComponents.put(subjobTablePlugin.getTitle(),
                subjobScrollPane);
            
            fViewablePlugins.put(fSubjobMenu, subjobTablePlugin);
            
            int i = fActiveElementsPane.getTabCount();
            fActiveElementsPane.setBackgroundAt(i - 1, Color.white);
            fActiveElementsPane.setForegroundAt(i - 1, Color.darkGray);
        }
        catch (STAFException ex)
        {
            System.out.println("STAFException thrown during " + 
                "STAXMonitorSubjobTablePlugin initialization.  RC: " +
                ex.rc);
        }
        
        fViewMenu.addSeparator();
        
        // Testcase Status
        try
        {
            testcaseTablePlugin = new STAXMonitorTestcaseExtension();
            JComponent testcaseTable = 
                testcaseTablePlugin.init(this, newJob, fStaxMachine,
                fStaxServiceName, fJobNumber);
            testcaseTable.setVisible(true);
            
            Vector notificationTypes = new Vector();
                
            StringTokenizer types = new 
                StringTokenizer(testcaseTablePlugin.getNotificationEventTypes(),
                " ");
                    
            while (types.hasMoreElements())
            {
                String type = types.nextToken().toLowerCase();
                notificationTypes.add(type);
            }
            
            fRegisteredPlugins.put(testcaseTablePlugin, notificationTypes);
                
            fTestcaseStatusMenu = new JCheckBoxMenuItem(
                testcaseTablePlugin.getTitle(), true);
            fTestcaseStatusMenu.addActionListener(this);
            
            JScrollPane tcScrollPane = 
                new JScrollPane(testcaseTablePlugin.getComponent());
            tcScrollPane.setVerticalScrollBarPolicy(
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
            tcScrollPane.setHorizontalScrollBarPolicy(
                JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
                
            fViewMenu.add(fTestcaseStatusMenu);
                
            fViewablePlugins.put(fTestcaseStatusMenu, testcaseTablePlugin);
            
            fStatusPane.addTab(testcaseTablePlugin.getTitle(), 
                tcScrollPane);
                
            fViewableComponents.put(testcaseTablePlugin.getTitle(),
                tcScrollPane);
                      
            int i = fStatusPane.getTabCount();
            fStatusPane.setBackgroundAt(i - 1, Color.white);
            fInfoPane.setForegroundAt(i - 1, Color.darkGray);
        }
        catch (STAFException ex)
        {
            System.out.println("STAFException thrown during " + 
                "STAXMonitorSTAFCmdTablePlugin initialization.  RC: " +
                ex.rc);
        }
        
        // Testcase Info
        try
        {
            testcaseInfoTablePlugin = new STAXMonitorTestcaseExtension();
            JComponent testcaseTable = 
                testcaseInfoTablePlugin.init(this, newJob, fStaxMachine,
                fStaxServiceName, fJobNumber);
            testcaseTable.setVisible(true);
            
            Vector notificationTypes = new Vector();
                
            StringTokenizer types = new 
                StringTokenizer(testcaseInfoTablePlugin.
                getNotificationEventTypes(), " ");
                    
            while (types.hasMoreElements())
            {
                String type = types.nextToken().toLowerCase();
                notificationTypes.add(type);
            }
            
            fRegisteredPlugins.put(testcaseInfoTablePlugin, notificationTypes);
                 
            fTestcaseInfoMenu = new JCheckBoxMenuItem(
                testcaseInfoTablePlugin.getTitle() + " ", false);
            fTestcaseInfoMenu.addActionListener(this);
            
            JScrollPane tcScrollPane = 
                new JScrollPane(testcaseInfoTablePlugin.getComponent());
            tcScrollPane.setVerticalScrollBarPolicy(
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
            tcScrollPane.setHorizontalScrollBarPolicy(
                JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
                
            fViewablePlugins.put(fTestcaseInfoMenu, testcaseInfoTablePlugin);
        }
        catch (STAFException ex)
        {
            System.out.println("STAFException thrown during " + 
                "STAXMonitorSTAFCmdTablePlugin initialization.  RC: " +
                ex.rc);
        }
       
        fViewMenu.addSeparator();

        fMessagesMenu = 
            new JCheckBoxMenuItem("Messages", true);
        fMessagesMenu.addActionListener(this);
                
        fViewMenu.add(fMessagesMenu);
        
        fCurrentSelectionMenu = 
            new JCheckBoxMenuItem("Current Selection", true);
        fCurrentSelectionMenu.addActionListener(this);
                        
        fViewMenu.add(fCurrentSelectionMenu);
        
        // XXX fix this later....need to use the type to determine where 
        // the plugin should be placed in the menu
        fViewMenu.add(fTestcaseInfoMenu);
        
        fBlockPopupMenu.add(fBlockHoldMenuItem);
        fBlockHoldMenuItem.addActionListener(this);
        fBlockPopupMenu.add(fBlockReleaseMenuItem);
        fBlockReleaseMenuItem.addActionListener(this);
        fBlockPopupMenu.add(fBlockTerminateMenuItem);
        fBlockTerminateMenuItem.addActionListener(this);
        
        fSubjobPopupMenu.add(fSubjobStartMonitoringMenuItem);
        fSubjobStartMonitoringMenuItem.addActionListener(this);
        fSubjobPopupMenu.addSeparator();
        fSubjobPopupMenu.add(fSubjobDisplayJobLogMenuItem);
        fSubjobDisplayJobLogMenuItem.addActionListener(this);
        fSubjobPopupMenu.add(fSubjobDisplayJobUserLogMenuItem);
        fSubjobDisplayJobUserLogMenuItem.addActionListener(this);
        fSubjobPopupMenu.addSeparator();
        fSubjobPopupMenu.add(fSubjobTerminateMenuItem);
        fSubjobTerminateMenuItem.addActionListener(this);
        
        fProcessMonitor = new STAFProcessMonitor();
        fProcessMonitor.start();
        
        fElapsedTime = new MonitorElapsedTime();
        fElapsedTime.start();

        if (!newJob)
        {      
            String request = "LIST JOB " + fJobNumber + " BLOCKS";            
            STAFResult listResult = fHandle.submit2(fStaxMachine, 
    	                         fStaxServiceName, request);    	      
            
            if (listResult.rc != 0)
            {
                throw new STAFException(listResult.rc,                                     
                                        "Error listing Job Blocks\n" 
                                        + listResult.result);
            }
            
            BufferedReader reader = 
                new BufferedReader(new StringReader(listResult.result));
            String line;
            Vector heldBlocks = new Vector();

            try
            {                
                while ((line = reader.readLine()) != null)
                {
                    String block = line.substring(0, line.indexOf(";"));
                    String status = line.substring(line.indexOf(";") + 1);
                    heldBlocks.addAll(handleBlock(block, status));
                }
                
                for (int i = 0; i < heldBlocks.size(); i++)
                {
                    ((STAXMonitorTreeNode)heldBlocks.elementAt(i)).
                        setBlockStatus(STAXMonitorTreeNode.blockHeld);
                }
            }
            catch(IOException ex)
            {
                ex.printStackTrace();
            }
            finally
            {
                try
                {
                    reader.close();
                }
                catch (IOException ex)
                {
                    ex.printStackTrace();
                }
            }
            
            request = "LIST JOB " + fJobNumber + " PROCESSES";
            listResult = fHandle.submit2(fStaxMachine, fStaxServiceName, 
                                         request);    	      
            
            if (listResult.rc != 0)
            {
                throw new STAFException(listResult.rc,                                     
                                        "Error listing Job PROCESSES\n" 
                                        + listResult.result);
            }
            
            reader = new BufferedReader(new StringReader(listResult.result));

            try
            {
                while ((line = reader.readLine()) != null)
                {
                    int locationPos = line.indexOf(";") + 1;
                    int handlePos = line.indexOf(";", locationPos + 1) + 1;
                    int nextPos = line.indexOf(";", handlePos + 1);                   
                    String location = line.substring(locationPos,
                                           handlePos - 1);
                    String handleNumber = line.substring(handlePos, nextPos);              
                    
                    request = "QUERY JOB " + fJobNumber + " PROCESS " + 
                              location + ":" + handleNumber;
                    STAFResult queryResult = fHandle.submit2(fStaxMachine,
                                                     fStaxServiceName,
                                                     request);
                    
                    if (location.equals("local"))
                    {
                        location = fStaxMachine;
                    }

                    BufferedReader queryReader = new 
                        BufferedReader(new StringReader(queryResult.result));
                    String queryLine;
                    String block = "";
                    String processName = "";
                    String command = "";
                    String commandMode = "";
                    String commandShell = "";
                    String processTitle = "";
                    String parms = "";
                    String workdir = "";
                    String kBlock = 
                        "Block Name      ";
                    String kProcessName = 
                        "Name            ";
                    String kCommand = 
                        "Command         ";
                    String kCommandMode = 
                        "Command Mode    ";
                    String kCommandShell = 
                        "Command Shell   ";
                    String kTitle = 
                        "Title           ";
                    String kParms = 
                        "Parms           ";
                    String kWorkdir = 
                        "WorkDir         ";
                
                    while ((queryLine = queryReader.readLine()) != null)
                    {
                        // Format for most lines is Description: value
                        // Get the value from lines formatted as above.

                        String value = "";

                        int colonIndex = queryLine.indexOf(":");

                        if (colonIndex > -1 && queryLine.length() > 2)
                            value = queryLine.substring(colonIndex + 2).trim();
                        else continue;

                        if (queryLine.startsWith(kBlock))
                            block = value;                            
                        else if (queryLine.startsWith(kProcessName))
                            processName = value;
                        else if (queryLine.startsWith(kCommand))
                            command = value;
                        else if (queryLine.startsWith(kCommandMode))
                            commandMode = value;
                        else if (queryLine.startsWith(kCommandShell))
                            commandShell = value;
                        else if (queryLine.startsWith(kTitle))
                            processTitle = value;
                        else if (queryLine.startsWith(kParms))
                            parms = value;
                        else if (queryLine.startsWith(kWorkdir))
                            workdir = value;
                    }

                    queryReader.close();
                    
                    handleProcess("", block, processName, location, 
                                  handleNumber, command,
                                  commandMode, commandShell,
                                  processTitle, parms, workdir);
                }           
            }
            catch(IOException ex)
            {
                ex.printStackTrace();
            }
            finally
            {
                try
                {
                    reader.close();
                }
                catch (IOException ex)
                {
                    ex.printStackTrace();
                }
            }
            
            request = "LIST JOB " + fJobNumber + " STAFCMDS";
            listResult = fHandle.submit2(fStaxMachine, fStaxServiceName, 
                                         request);    	      
            
            if (listResult.rc != 0)
            {
                throw new STAFException(listResult.rc,                                     
                                        "Error listing Job STAFCMDS\n" 
                                        + listResult.result);
            }

            reader = new BufferedReader(new StringReader(listResult.result));

            try
            {                
                while ((line = reader.readLine()) != null)
                {
                    int locationPos = line.indexOf(";") + 1;
                    int reqNumPos = line.indexOf(";", locationPos + 1) + 1;
                    int servicePos = line.indexOf(";", reqNumPos + 1) + 1;
                    int requestPos = line.indexOf(";", servicePos + 1) + 1;
                    
                    String commandName = line.substring(0, locationPos - 1);
                    String location = line.substring(locationPos,
                                           reqNumPos - 1);
                    String requestNumber = line.substring(reqNumPos,
                                           servicePos - 1);
                    String service = line.substring(servicePos,
                                           requestPos - 1);
                    String cmdRequest = line.substring(requestPos);
                    
                    request = "QUERY JOB " + fJobNumber + " STAFCMD " + 
                              requestNumber;
                    STAFResult queryResult = fHandle.submit2(fStaxMachine,
                                                     fStaxServiceName,
                                                     request);                                                 
                    
                    if (location.equals("local"))
                    {
                        location = fStaxMachine;
                    }

                    BufferedReader queryReader = new 
                        BufferedReader(new StringReader(queryResult.result));
                    String queryLine;
                    String block = "";                                              
                
                    while ((queryLine = queryReader.readLine()) != null)
                    {
                        String kBlock = "Block Name: ";                        
                        if (queryLine.startsWith(kBlock))
                        {
                            block = queryLine.substring(kBlock.length());                            
                        }
                    }

                    queryReader.close();

                    handleCommand(commandName, block, location, requestNumber,
                                  service, cmdRequest);
                }        
                
            }
            catch(IOException ex)
            {
                ex.printStackTrace();
            }
            finally
            {
                try
                {
                    reader.close();
                }
                catch (IOException ex)
                {
                    ex.printStackTrace();
                }
            }
     
            request = "LIST JOB " + fJobNumber + " SUBJOBS";
            listResult = fHandle.submit2(fStaxMachine, fStaxServiceName, 
                                         request);    	      
            
            if (listResult.rc != 0)
            {
                throw new STAFException(listResult.rc,                                     
                                        "Error listing Job SUBJOBS\n" 
                                        + listResult.result);
            }

            reader = new BufferedReader(new StringReader(listResult.result));

            try
            {                
                while ((line = reader.readLine()) != null)
                {
                    int jobIDIndex = line.indexOf(";");
                    String jobID = line.substring(0, jobIDIndex);
                    int jobNameIndex = line.indexOf(";", jobIDIndex+1);
                    jobName = line.substring(jobIDIndex+1, jobNameIndex);
                    int jobBlockIndex = line.indexOf(";", jobNameIndex+1);
                    String jobBlock = line.substring(jobNameIndex+1, 
                        jobBlockIndex);
                        
                    request = "QUERY JOB " + jobID;
                    STAFResult queryResult = fHandle.submit2(fStaxMachine,
                        fStaxServiceName, request);
                
                    String jobFile           = "";
                    String kXMLFile          = "XML File Name ";
                    String jobFileMachine    = "";
                    String kFileMachine      = "File Machine  ";
                    String subjobFunction    = "";
                    String kFunction         = "Function      ";
                    String functionArgs      = "";
                    String kArgs             = "Arguments     ";
                    scripts                  = new Vector();
                    String kScript           = "Script #";
                    Vector scriptFiles       = new Vector();
                    String kScriptFile       = "Script File #";
                    String scriptFileMachine = "";
                    String kScriptMachine    = "Script Machine";
                    String startDate         = "";
                    String kStartDate        = "Start Date    ";
                    String startTime         = "";
                    String kStartTime        = "Start Time    ";
                    String clearLogsString   = "Disabled";
                    String kClearLogs        = "Clear Logs    ";
                    String logTCElapsedTimeString = "Disabled";
                    String kLogTCElapsedTime = "Log TC Elapsed Time";
                    String logTCNumStartsString = "Disabled";
                    String kLogTCNumStarts   = "Log TC Num Starts  ";
                    String logTCStartStopString = "Disabled";
                    String kLogTCStartStop   = "Log TC Start/Stop  ";

                    BufferedReader queryReader = new BufferedReader(
                        new StringReader(queryResult.result));
                    int scriptIndex = -1;

                    while ((line = queryReader.readLine()) != null)
                    {
                        // Format for most lines is Description: value
                        // Get the value from lines formatted as above.

                        String value = "";

                        int colonIndex = line.indexOf(":");

                        if (colonIndex > -1 && line.length() > 2)
                            value = line.substring(colonIndex + 2).trim();

                        // Check if line contains value needed to restart a job

                        if (line.startsWith(kXMLFile))
                        {
                            jobFile = value;
                        }
                        else if (line.startsWith(kFileMachine))
                        {
                            jobFileMachine = value;
                        }
                        else if (line.startsWith(kFunction))
                        {
                            subjobFunction = value;
                        }
                        else if (line.startsWith(kArgs))
                        {
                            functionArgs = value;
                        }
                        else if (line.startsWith(kScript))
                        {
                            scripts.add(value);

                            // Set the index in the SCRIPT vector in case it
                            // the SCRIPT value is multiple lines
                            scriptIndex++; 
                        }
                        else if (line.startsWith(kScriptFile))
                        {
                            scriptIndex = -1;
                            scriptFiles.add(value);
                        }
                        else if (line.startsWith(kScriptMachine))
                        {
                            scriptIndex = -1;
                            scriptFileMachine = value;
                        }
                        else if (line.startsWith(kStartDate))
                        {
                            scriptIndex = -1;
                            startDate = value;
                        }
                        else if (line.startsWith(kStartTime))
                        {
                            startTime = value;
                        }
                        else if (line.startsWith(kClearLogs))
                        {
                            clearLogsString = value;
                        }
                        else if (line.startsWith(kLogTCElapsedTime))
                        {
                            logTCElapsedTimeString = value;
                        }
                        else if (line.startsWith(kLogTCNumStarts))
                        {
                            logTCNumStartsString = value;
                        }
                        else if (line.startsWith(kLogTCStartStop))
                        {
                            logTCStartStopString = value;
                        }
                        else if (scriptIndex >= 0)
                        {
                            // Add continuation line to current SCRIPT value
                            scripts.setElementAt(
                                (String)scripts.get(scriptIndex) + "\n" + line,
                                scriptIndex);
                        }
                    }           
                    
                    queryReader.close();

                    String subjobText = "Job " + jobID + " - " + jobName;
                    
                    STAXMonitorTreeNode newNode = 
                        new STAXMonitorTreeNode(subjobText, 
                        STAXMonitorTreeNode.subjobNodeType);
            
                    STAXMonitorTreeNode parentNode = (STAXMonitorTreeNode) 
                        fMonitorTreeBlocks.get(jobBlock);
                        
                    // parentNode can be null if the block has ended                
                    if (parentNode != null)
                    {
                        fMonitorTreeModel.insertNodeInto(newNode, parentNode,
                            parentNode.getChildCount());
                        TreeNode[] parentNodes = 
                            fMonitorTreeModel.getPathToRoot(parentNode);        
                    
                        fMonitorTree.expandPath(new TreePath(parentNodes));            
                            
                        fSubjobIDHashtable.put(jobID, newNode);
                        fMonitorTreeSubjobNodes.put(newNode, jobID);
        
                        fMonitorTreeSubjobStartTimes.put(newNode, 
                            STAXMonitorUtil.getCalendar2(startDate, 
                            startTime));
                    }
                        
                    Vector subjobDataVector = new Vector();
                    
                    addRow(subjobDataVector, "Job ID", jobID);
                    addRow(subjobDataVector, "Job Name", jobName);
                    addRow(subjobDataVector, "Clear Logs", clearLogsString);
                    addRow(subjobDataVector, "Log TC Elapsed Time",
                           logTCElapsedTimeString);
                    addRow(subjobDataVector, "Log TC Num Starts",
                           logTCNumStartsString);
                    addRow(subjobDataVector, "Log TC Start/Stop",
                           logTCStartStopString);
                    addRow(subjobDataVector, "Job File", jobFile);

                    if (!jobFileMachine.equals(""))
                    {
                        addRow(subjobDataVector,
                               "Job File Machine", jobFileMachine);
                    }

                    addRow(subjobDataVector, "Function", subjobFunction);
                    addRow(subjobDataVector, "Function Args", functionArgs);

                    for (int i = 0; i < scriptFiles.size(); ++i)
                    {
                        if (i == 0)
                        {
                            addRow(subjobDataVector, "Script Files Machine",
                                   scriptFileMachine);
                        }

                        addRow(subjobDataVector, "Script File #" + (i + 1),
                               (String)scriptFiles.elementAt(i));
                    }

                    for (int i = 0; i < scripts.size(); ++i)
                    {                    
                        addRow(subjobDataVector, "Script #" + (i + 1),
                               (String)scripts.elementAt(i));
                    }

                    addRow(subjobDataVector, "Start Date", startDate);
                    addRow(subjobDataVector, "Start Time", startTime);
                    addRow(subjobDataVector, "Block", jobBlock);
                    
                    synchronized(fDataHashtable)
                    {
                        fDataHashtable.put(subjobText, subjobDataVector);
                    }
                }
            }
            catch(IOException ex)
            {
                ex.printStackTrace();
            }
            finally
            {
                try
                {
                    reader.close();
                }
                catch (IOException ex)
                {
                    ex.printStackTrace();
                }
            }
        }
        
        try
        {
            Thread.sleep(500);
        }
        catch (InterruptedException ex)
        {
        }
        
        fInfoPane.add("Current Selection", fSelectionDetailsPanel);
        fViewableComponents.put("Current Selection", fSelectionDetailsPanel);
        
        index = fInfoPane.getTabCount();
        
        fInfoPane.setBackgroundAt(index - 1, Color.white);
        fInfoPane.setForegroundAt(index - 1, Color.darkGray);
       
        // Get monitor extension (plugin) classes
        Vector pluginClasses = parentMonitor.getPluginClasses();

        // Handle external extensions (plugins)        
        for (int p = 0; p < pluginClasses.size(); p++)
        {
            try
            {
                Class pluginClass = (Class)pluginClasses.elementAt(p);
                Object pluginObj = null;

                // Try using a constructor that accepts a STAX object

                Class [] parameterTypes = new Class[1];
                parameterTypes[0] = this.getClass();

                try
                {
                    Constructor construct = 
                        pluginClass.getConstructor(parameterTypes);
                    Object [] initArgs = new Object[1];
                    initArgs[0] = this;
                    pluginObj = construct.newInstance(initArgs);
                }
                catch (NoSuchMethodException e)
                {
                    // Extension does not have a constructor that accepts
                    // a STAX object, so use constructor without parameters
                    pluginObj = pluginClass.newInstance();
                }

                STAXMonitorExtension plugin = (STAXMonitorExtension)pluginObj;

                JComponent pluginComponent = 
                    plugin.init(this, newJob, fStaxMachine,
                    fStaxServiceName, fJobNumber);
                    
                JScrollPane pluginScrollPane = null;
                
                pluginComponent.setVisible(true);
                
                Vector notificationTypes = new Vector();
                
                StringTokenizer types = new 
                    StringTokenizer(plugin.getNotificationEventTypes(), " ");
                    
                while (types.hasMoreElements())
                {
                    String type = types.nextToken().toLowerCase();
                    notificationTypes.add(type);
                    fExternalPluginNotificationTypes.add(type);
                }
                
                fRegisteredPlugins.put(plugin, notificationTypes);
                
                pluginScrollPane = 
                    new JScrollPane(plugin.getComponent());
                pluginScrollPane.setVerticalScrollBarPolicy(
                    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
                pluginScrollPane.setHorizontalScrollBarPolicy(
                    JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
                
                fViewableComponents.put(plugin.getTitle(), pluginScrollPane);
                
                pluginComponent.setVisible(true);
                
                int pluginType = plugin.getExtensionType();
                
                if (pluginType == STAXMonitorFrame.EXTENSION_ACTIVE)
                {
                    fActiveElementsPane.addTab(plugin.getTitle(), 
                        pluginScrollPane);
                          
                    int i = fActiveElementsPane.getTabCount();
                    fActiveElementsPane.setBackgroundAt(i - 1, Color.white);
                    fActiveElementsPane.setForegroundAt(i - 1, Color.darkGray);
                }
                else if (pluginType == STAXMonitorFrame.EXTENSION_STATUS)
                {
                    fStatusPane.addTab(plugin.getTitle(), pluginScrollPane);
                          
                    int i = fStatusPane.getTabCount();
                    fStatusPane.setBackgroundAt(i - 1, Color.white);
                    fStatusPane.setForegroundAt(i - 1, Color.darkGray);
                }
                else if (pluginType == STAXMonitorFrame.EXTENSION_INFO)
                {
                    fInfoPane.addTab(plugin.getTitle(), pluginScrollPane);
                          
                    int i = fInfoPane.getTabCount();
                    fInfoPane.setBackgroundAt(i - 1, Color.white);
                    fInfoPane.setForegroundAt(i - 1, Color.darkGray);
                }
            }
            catch (InvocationTargetException e)
            {
                System.out.println("InvocationTargetException thrown during " +
                                   "STAXMonitor plugin initialization\n" +
                                   e.getMessage());

                e.printStackTrace();
            }
            catch (InstantiationException e)
            {
                System.out.println("InstantiationException thrown during " +
                                   "STAXMonitor plugin initialization\n" +
                                   e.getMessage());

                e.printStackTrace();
            }
            catch (IllegalAccessException e)
            {
                System.out.println("IllegalAccessException thrown during " +
                                   "STAXMonitor plugin initialization\n" +
                                   e.getMessage());

                e.printStackTrace();
            }
            catch (STAFException ex)
            {
                System.out.println("STAFException thrown during " + 
                                   "STAXMonitor plugin initialization.  RC: " +
                                   ex.rc);
            }
        }
        
        getContentPane().add(BorderLayout.CENTER, monitorPanel);

        pack();
        setSize(new Dimension(950, 600));
        setLocation(50, 30);
        show();

            // the calls to invalidate and validate are needed so that
            // the dividers will be located correctly on Linux systems        
        horizPane.setDividerLocation(.55);
        invalidate();
        validate();
        vertPane.setDividerLocation(.68);        
        invalidate();
        validate();
        
        startMonitor();
        
        if (!jobAlreadyHeld)
        {
            STAFResult releaseResult = fHandle.submit2(fStaxMachine, 
	                         fStaxServiceName, "RELEASE JOB " + fJobNumber);
            
            if ((releaseResult.rc == STAFResult.DoesNotExist) ||
                (releaseResult.rc == BlockNotHeld))
            {
                // need to wait for slower machines
                for (int i = 0; i < 15; i++)
                {
                    try 
                    {
                        Thread.sleep(2000);
                                   
                        releaseResult = fHandle.submit2(fStaxMachine, 
                            fStaxServiceName, "RELEASE JOB " + fJobNumber);
                            
                        if ((releaseResult.rc != STAFResult.DoesNotExist) &&
                            (releaseResult.rc != BlockNotHeld))
                        {
                            break;
                        }
                    }
                    catch (InterruptedException ex)
                    {
                    }
                }                
            }
            
            if (releaseResult.rc != 0)
            {
                JOptionPane.showMessageDialog(this,
                                    "An error was encountered while "
                                    + "attempting to release the job, rc="
                                    + releaseResult.rc + " result="
                                    + releaseResult.result,
                                    "Error releasing job",
                                    JOptionPane.ERROR_MESSAGE);
            }                      
        }
        
        setTitle(kMonitorFrameTitle + " " + "Machine:" +
                     fStaxMachine + " JobID:" + fJobNumber + 
                     " " + fJobName + " <Running>");
        
        fQueueThread = new Thread(this);
        fQueueThread.start(); // this calls the run() method
    }
    
    public STAXMonitor getParentMonitor() 
    {
        return fParentMonitor;
    }

    public void actionPerformed(ActionEvent e)
    {
        String action = "";
        if (e.getSource() == fBlockHoldMenuItem)
        {
            action = "hold";            
        }
        else if (e.getSource() == fBlockReleaseMenuItem)
        {
            action = "release";
        }
        else if (e.getSource() == fBlockTerminateMenuItem)
        {
            action = "terminate";
        }
        else if (e.getSource() == fSubjobStartMonitoringMenuItem)
        {
            String subjobID = (String)
                fMonitorTreeSubjobNodes.get(fCurrentSelectedNode);
            fParentMonitor.monitorExistingJob(subjobID, -1);
            
            return;
        }
        else if (e.getSource() == fSubjobDisplayJobLogMenuItem)
        {
            String subjobID = (String)
                fMonitorTreeSubjobNodes.get(fCurrentSelectedNode);
                
            String[] showLogParms = new String[4];
            showLogParms[0] = "-machine";
            showLogParms[1] = fStaxMachine;
            showLogParms[2] = "-name";
            showLogParms[3] = fStaxServiceName.toUpperCase() +
                "_Job_" + subjobID;
            
            STAXMonitorLogViewer logViewer = 
                new STAXMonitorLogViewer(this, fHandle, showLogParms);
                        
            return;
        }
        else if (e.getSource() == fSubjobDisplayJobUserLogMenuItem)
        {
            String subjobID = (String)
                fMonitorTreeSubjobNodes.get(fCurrentSelectedNode);
            
            String[] showLogParms = new String[4];
            showLogParms[0] = "-machine";
            showLogParms[1] = fStaxMachine;
            showLogParms[2] = "-name";
            showLogParms[3] = fStaxServiceName.toUpperCase() +
                "_Job_" + subjobID + "_User";
            
            STAXMonitorLogViewer logViewer = 
                new STAXMonitorLogViewer(this, fHandle, showLogParms);
            
            return;
        }
        else if (e.getSource() == fSubjobTerminateMenuItem)
        {
            String subjobID = (String)
                fMonitorTreeSubjobNodes.get(fCurrentSelectedNode);
            
            int confirmation = JOptionPane.showConfirmDialog(this,
                "Are you certain that you want\n" +
                "to terminate Job # " + subjobID + " ?", 
                "Confirm Job Termination",
                JOptionPane.YES_NO_OPTION,
                JOptionPane.QUESTION_MESSAGE);
                         
            if (!(confirmation == JOptionPane.YES_OPTION))
            {
                return;
            }                                                  
       
            String request = "TERMINATE JOB " + subjobID;            
        
            STAFResult result =
                fHandle.submit2(fStaxMachine, 
                fStaxServiceName, request);
                
            if (result.rc != 0)
            {
                fParentMonitor.
                    showErrorDialog(this, "An error was encountered while "
                    + "attempting to terminate jobID " 
                    + subjobID + " rc=" + result.rc
                    + " result=" + result.result);
            }
            return;
        }
        else if (e.getSource() == fDisplayJobLog)
        {
            fBlockPopupMenu.setVisible(false);
            
            String[] showLogParms = new String[4];
            showLogParms[0] = "-machine";
            showLogParms[1] = fStaxMachine;
            showLogParms[2] = "-name";
            showLogParms[3] = fStaxServiceName.toUpperCase() +
                "_Job_" + fJobNumber;
            
            STAXMonitorLogViewer logViewer = 
                new STAXMonitorLogViewer(this, fHandle, showLogParms);
            
            fBlockPopupMenu.setVisible(false);
            
            return;
        }
        else if (e.getSource() == fDisplayJobUserLog)
        {
            fBlockPopupMenu.setVisible(false);
           
            String[] showLogParms = new String[4];
            showLogParms[0] = "-machine";
            showLogParms[1] = fStaxMachine;
            showLogParms[2] = "-name";
            showLogParms[3] = fStaxServiceName.toUpperCase() +
                "_Job_" + fJobNumber + "_User";
            
            STAXMonitorLogViewer logViewer = 
                new STAXMonitorLogViewer(this, fHandle, showLogParms);
            
            fBlockPopupMenu.setVisible(false);
            
            return;
        }
        else if (e.getSource() == fFileExit)
        {
            if (!fMonitorStopped)
            {
                stopMonitor();
            }
            
            closeMonitor();
            return;
        }
        else if ((e.getSource() == fFileResubmit) ||
                 (e.getSource() == fFileExitAndResubmit))
        {
            try
            {
                fParentMonitor.addMonitoredJob(new STAXMonitorFrame(
                    true, fParentMonitor, fStaxMachine, 
                    fStaxServiceName, fJobNumber, fEventMachine, 
                    fEventServiceName, fXmlFileName, fXmlFileMachine, 
                    fFunction, fArgs, fJobName, fScripts, 
                    fShowNoSTAXMonitorInformation, fLimitMessages,
                    fPluginJarsVector, fScriptFilesVector, 
                    fScriptFilesMachineName, fClearLogs,
                    fLogTCElapsedTime, fLogTCNumStarts, fLogTCStartStop,
                    fAutoMonitorSubjobs));
            }
            catch (STAFException ex)
            {
                showErrorDialog(this, "RC:" + ex.rc + "\n" 
                                + ex.getMessage());
                return;
            }
            
            if (e.getSource() == fFileExitAndResubmit)
            {
                if (!fMonitorStopped)
                {
                    stopMonitor();
                }
                
                closeMonitor();
            }

            return;
        }
        else if (e.getSource() == fProcessMenu)
        {
            STAXMonitorExtension plugin = (STAXMonitorExtension)
                fViewablePlugins.get(e.getSource());
            
            if (((JCheckBoxMenuItem)e.getSource()).isSelected())
            {
                JScrollPane scrollPane = new JScrollPane(plugin.getComponent());
                scrollPane.setVerticalScrollBarPolicy(
                    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
                scrollPane.setHorizontalScrollBarPolicy(
                    JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
                    
                int pluginType = plugin.getExtensionType();
                
                if (pluginType == STAXMonitorFrame.EXTENSION_ACTIVE)
                {
                    fActiveElementsPane.addTab(plugin.getTitle(), 
                        scrollPane);
                        
                    int index = fActiveElementsPane.getTabCount();
                    fActiveElementsPane.setBackgroundAt(index - 1, 
                        Color.white);
                    fActiveElementsPane.setForegroundAt(index - 1, 
                        Color.darkGray);
                    
                }
                
                fViewableComponents.put(plugin.getTitle(), scrollPane);
            }
            else
            {
                int pluginType = plugin.getExtensionType();
                
                if (pluginType == STAXMonitorFrame.EXTENSION_ACTIVE)
                {
                    fActiveElementsPane.remove((JComponent)
                        fViewableComponents.get(plugin.getTitle()));
                }
                       
                fViewableComponents.remove(plugin.getTitle());
            }

            return;
        }
        else if (e.getSource() == fSTAFCmdMenu)
        {
            STAXMonitorExtension plugin = (STAXMonitorExtension)
                fViewablePlugins.get(e.getSource());
            
            if (((JCheckBoxMenuItem)e.getSource()).isSelected())
            {
                JScrollPane scrollPane = new JScrollPane(plugin.getComponent());
                scrollPane.setVerticalScrollBarPolicy(
                    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
                scrollPane.setHorizontalScrollBarPolicy(
                    JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
            
                int pluginType = plugin.getExtensionType();
                
                if (pluginType == STAXMonitorFrame.EXTENSION_ACTIVE)
                {
                    fActiveElementsPane.addTab(plugin.getTitle(), 
                        scrollPane);
                        
                    int index = fActiveElementsPane.getTabCount();
                    fActiveElementsPane.setBackgroundAt(index - 1, 
                        Color.white);
                    fActiveElementsPane.setForegroundAt(index - 1, 
                        Color.darkGray);
                }
                
                fViewableComponents.put(plugin.getTitle(), scrollPane);
            }
            else
            {
                int pluginType = plugin.getExtensionType();
                
                if (pluginType == STAXMonitorFrame.EXTENSION_ACTIVE)
                {
                    fActiveElementsPane.remove((JComponent)
                        fViewableComponents.get(plugin.getTitle()));
                }
                
                fViewableComponents.remove(plugin.getTitle());
            }
            
            return;
        }
        else if (e.getSource() == fSubjobMenu)
        {
            STAXMonitorExtension plugin = (STAXMonitorExtension)
                fViewablePlugins.get(e.getSource());
            
            if (((JCheckBoxMenuItem)e.getSource()).isSelected())
            {
                JScrollPane scrollPane = new JScrollPane(plugin.getComponent());
                scrollPane.setVerticalScrollBarPolicy(
                    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
                scrollPane.setHorizontalScrollBarPolicy(
                    JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
            
                int pluginType = plugin.getExtensionType();
                
                if (pluginType == STAXMonitorFrame.EXTENSION_ACTIVE)
                {
                    fActiveElementsPane.addTab(plugin.getTitle(), 
                        scrollPane);
                        
                    int index = fActiveElementsPane.getTabCount();
                    fActiveElementsPane.setBackgroundAt(index - 1, 
                        Color.white);
                    fActiveElementsPane.setForegroundAt(index - 1, 
                        Color.darkGray);
                }
                
                fViewableComponents.put(plugin.getTitle(), scrollPane);
            }
            else
            {
                int pluginType = plugin.getExtensionType();
                
                if (pluginType == STAXMonitorFrame.EXTENSION_ACTIVE)
                {
                    fActiveElementsPane.remove((JComponent)
                        fViewableComponents.get(plugin.getTitle()));
                }
                
                fViewableComponents.remove(plugin.getTitle());
            }
            
            return;
        }
        else if (e.getSource() == fActiveJobElementsMenu)
        {
            if (((JCheckBoxMenuItem)e.getSource()).isSelected())
            {
                JScrollPane scrollPane = new JScrollPane(fMonitorTree);
                scrollPane.setVerticalScrollBarPolicy(
                    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
                scrollPane.setHorizontalScrollBarPolicy(
                    JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
                    
                fActiveElementsPane.add("Active Job Elements", scrollPane);
                fViewableComponents.put("Active Job Elements", scrollPane);
            }
            else
            {
                fActiveElementsPane.remove((JComponent)
                    fViewableComponents.get("Active Job Elements"));
                fViewableComponents.remove("Active Job Elements");
            }

            return;
        }
        else if (e.getSource() == fTestcaseInfoMenu)
        {
            STAXMonitorExtension plugin = (STAXMonitorExtension)
                fViewablePlugins.get(e.getSource());
               
            if (((JCheckBoxMenuItem)e.getSource()).isSelected())
            {
                JScrollPane scrollPane = new JScrollPane(plugin.getComponent());
                scrollPane.setVerticalScrollBarPolicy(
                    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
                scrollPane.setHorizontalScrollBarPolicy(
                    JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
                    
                int pluginType = plugin.getExtensionType();
                    
                if (pluginType == STAXMonitorFrame.EXTENSION_INFO)
                {
                    fInfoPane.addTab(plugin.getTitle() + " ", scrollPane);
                        
                    int index = fInfoPane.getTabCount();
                    fInfoPane.setBackgroundAt(index - 1, Color.white);
                    fInfoPane.setForegroundAt(index - 1, Color.darkGray);
                }
                
                fViewableComponents.put(plugin.getTitle() + " ", scrollPane);
            }
            else
            {
                int pluginType = plugin.getExtensionType();
                
                if (pluginType == STAXMonitorFrame.EXTENSION_INFO)
                {
                    fInfoPane.remove((JComponent)
                        fViewableComponents.get(plugin.getTitle() + " "));
                }
                
                fViewableComponents.remove(plugin.getTitle() + " ");
            }

            return;
        }
        else if (e.getSource() == fTestcaseStatusMenu)
        {
            STAXMonitorExtension plugin = (STAXMonitorExtension)
                fViewablePlugins.get(e.getSource());
               
            if (((JCheckBoxMenuItem)e.getSource()).isSelected())
            {
                JScrollPane scrollPane = new JScrollPane(plugin.getComponent());
                scrollPane.setVerticalScrollBarPolicy(
                    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
                scrollPane.setHorizontalScrollBarPolicy(
                    JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
                    
                int pluginType = plugin.getExtensionType();
                    
                if (pluginType == STAXMonitorFrame.EXTENSION_INFO)
                {
                    fStatusPane.addTab(plugin.getTitle(), scrollPane);
                        
                    int index = fStatusPane.getTabCount();
                    fStatusPane.setBackgroundAt(index - 1, Color.white);
                    fStatusPane.setForegroundAt(index - 1, Color.darkGray);
                }
                
                fViewableComponents.put(plugin.getTitle(), scrollPane);
            
            }
            else
            {
                int pluginType = plugin.getExtensionType();
                
                if (pluginType == STAXMonitorFrame.EXTENSION_INFO)
                {
                    fStatusPane.remove((JComponent)
                        fViewableComponents.get(plugin.getTitle()));
                }
                
                fViewableComponents.remove(plugin.getTitle());
            }

            return;
        }
        else if (e.getSource() == fCurrentSelectionMenu)
        {
            if (((JCheckBoxMenuItem)e.getSource()).isSelected())
            {
                JScrollPane scrollPane = 
                    new JScrollPane(fSelectionDetailsPanel);
                scrollPane.setVerticalScrollBarPolicy(
                    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
                scrollPane.setHorizontalScrollBarPolicy(
                    JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
                fInfoPane.add("Current Selection", scrollPane);
                fViewableComponents.put("Current Selection", scrollPane);
                
                int index = fInfoPane.getTabCount();
                fInfoPane.setBackgroundAt(index - 1, 
                    Color.white);
                fInfoPane.setForegroundAt(index - 1, 
                    Color.darkGray);
            }
            else
            {
                fInfoPane.remove((JComponent)
                    fViewableComponents.get("Current Selection"));
                fViewableComponents.remove("Current Selection");
            }

            return;
        }
        else if (e.getSource() == fMessagesMenu)
        {
            if (((JCheckBoxMenuItem)e.getSource()).isSelected())
            {
                JScrollPane scrollPane = 
                    new JScrollPane(fMessageTable);
                scrollPane.setVerticalScrollBarPolicy(
                    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
                scrollPane.setHorizontalScrollBarPolicy(
                    JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
                    
                fInfoPane.add("Messages", scrollPane);
                fViewableComponents.put("Messages", scrollPane);
            }
            else
            {
                fInfoPane.remove((JComponent)
                    fViewableComponents.get("Messages"));
                fViewableComponents.remove("Messages");
            }

            return;
        }
        else
        {
            return;
        }               
        
        String blockNode = 
            (String)fMonitorTreeBlockNodes.get(fCurrentSelectedNode);        
        
        String request = action + " JOB " + fJobNumber + " BLOCK " + 
                         STAFUtil.wrapData(blockNode);
        
        STAFResult result =
                fHandle.submit2(fStaxMachine, fStaxServiceName, request);
                
        if (result.rc != 0)
        {
            showErrorDialog("An error was encountered while "
                            + "attempting to " + action + " rc=" + result.rc
                            + " result=" + result.result);
        }
    }
    
    public void stateChanged(ChangeEvent e)
    {
        JTabbedPane pane = (JTabbedPane)e.getSource();
        
        for (int i = 0; i < pane.getTabCount(); i++)
        {
            pane.setBackgroundAt(i, Color.white);
            pane.setForegroundAt(i, Color.darkGray);
        }
        
        int selected = ((JTabbedPane)e.getSource()).getSelectedIndex();
     
        if (selected != -1)
        {   
            pane.setForegroundAt(selected, Color.black);
        }
    }
    
    public void showErrorDialog(Component parent, String message)
    {
        JTextPane messagePane = new JTextPane();                                                
        messagePane.setFont(new Font("Dialog", Font.BOLD, 12));
        messagePane.setEditable(false);                        
        messagePane.setText(message);
        messagePane.select(0,0);
        JScrollPane scrollPane = new JScrollPane(messagePane);
        scrollPane.setPreferredSize(new Dimension(400, 100));
        JOptionPane.showMessageDialog(parent, scrollPane,
                                      "STAX Error",
                                      JOptionPane.ERROR_MESSAGE);
    }

    void handleJobEvent(String message)
    {
        Map msgMap = STAXMonitorUtil.getMap(message);    

        int blockIndex = message.indexOf(";block=");
        int firstColon = message.indexOf(":", blockIndex);
        int secondColon = message.indexOf(":", firstColon + 1);
        
        String blockAndRemainder = message.substring(secondColon + 1);
        String block = blockAndRemainder.substring(0, 
    	                                        blockAndRemainder.indexOf(";"));
                                      
        if ((message.indexOf("status=:3:end") == -1) &&
            (message.indexOf("status=:4:stop") == -1))
        {
            handleBlock(block, "Running");
        }

        // at this point all of the blocks have been added      
        if (message.indexOf(";type=:5:block;") > -1)
        {
            if ((message.indexOf(";status=:3:end;") > -1) ||
                (message.indexOf(";status=:9:terminate;") > -1))
            {
                if (!block.equals("main"))
                {
                    synchronized(fMonitorTreeModel)
                    {
                        synchronized(fMonitorTreeBlocks)
                        {
                            fMonitorTreeModel.removeNodeFromParent(
                                (STAXMonitorTreeNode)
                                fMonitorTreeBlocks.get(block));
                            fMonitorTreeBlockNodes.remove(
                                fMonitorTreeBlocks.get(block));
                            fMonitorTreeBlocks.remove(block);
                        }
                        
                        fCurrentSelection.setText("");
                    }
                }
                else
                {
                    synchronized(fMonitorTreeModel)
                    {
                        synchronized(fMonitorTreeBlocks)
                        {
                            fMonitorTreeModel.removeNodeFromParent(
                                (STAXMonitorTreeNode)
                                fMonitorTreeBlocks.get(block));
                            fMonitorTreeBlockNodes.remove(
                                fMonitorTreeBlocks.get(block));
                            fMonitorTreeBlocks.remove(block);
                        }
                        
                        fCurrentSelection.setText("");
                    }
    	          }
            }
            else if (message.indexOf(";status=:4:hold;") > -1)
            {
                ((STAXMonitorTreeNode)fMonitorTreeBlocks.get(block)).
                    setBlockStatus(STAXMonitorTreeNode.blockHeld);                    
                
                fMonitorTree.updateUI();                   
            }
            else if (message.indexOf(";status=:7:release;") > -1)
            {
                ((STAXMonitorTreeNode)fMonitorTreeBlocks.get(block)).
                    setBlockStatus(STAXMonitorTreeNode.blockRunning);                    
                
                fMonitorTree.updateUI();                    
            }
        }
        else if (message.indexOf(";type=:7:command;") > -1)
        {   
            String commandName = STAXMonitorUtil.getValue(message, "name");
            
            String location = STAXMonitorUtil.getValue(message, "location");

            if (location.equals("local"))
            {                
                location = fStaxMachine;
            }            
            
            String requestNumber = 
                STAXMonitorUtil.getValue(message, "requestNumber");            
            
            String commandID = commandName + ";" + location + ";" 
                               + requestNumber;

            String commandTreeText = commandID;            

            if (message.indexOf(";status=:5:start;") > -1)
            {
                String service = STAXMonitorUtil.getValue(message, "service");
                String request = STAXMonitorUtil.getValue(message, "request");
                
                handleCommand(commandName, block, location, requestNumber,
                              service, request);

            } else if (message.indexOf(";status=:4:stop;") > -1)
            {    
                if (fCmdIDHashtable.containsKey(commandID))
                {
                    fMonitorTreeModel.
                        removeNodeFromParent((STAXMonitorTreeNode)
                            fCmdIDHashtable.get(commandID));
                    fMonitorTreeCommandNodes.remove
                            (fCmdIDHashtable.get(commandID));
                    fMonitorTreeCommandStartTimes.remove
                            (fCmdIDHashtable.get(commandID));
                    fCmdIDHashtable.remove(commandID);
                    fDataHashtable.remove(commandID);
                }
            }
        }
        else if (message.indexOf(";type=:7:process;") > -1)
        {
            String nodeText;
            String processName = STAXMonitorUtil.getValue(message, "name");
            nodeText = processName + " ";
            String location = STAXMonitorUtil.getValue(message, "location");
            
            if (location.equals("local"))
            {                
                location = fStaxMachine;
            }

            nodeText += location;
            String handleNumber = STAXMonitorUtil.getValue(message, "handle");
            nodeText += " " + handleNumber;
            
            String processID = processName + ";" + location + ";" 
                               + handleNumber;
            
            if (message.indexOf(";status=:5:start;") > -1)
            {
                String command = STAXMonitorUtil.getValue(message, "command");
                
                String commandMode = STAXMonitorUtil.getValue(message, "mode");
                
                String commandShell = STAXMonitorUtil.getValue(message, "shell");
                                          
                String processTitle = 
                    STAXMonitorUtil.getValue(message, "title");

                String parms = STAXMonitorUtil.getValue(message, "parms");

                String workdir = STAXMonitorUtil.getValue(message, "workdir");

                handleProcess(message, block, processName, location, 
                              handleNumber, command, 
                              commandMode, commandShell,
                              processTitle, parms, workdir);
            } 
            else if (message.indexOf(";status=:4:stop;") > -1)
            {
                if (fProcIDHashtable.containsKey(processID))
                {
                    synchronized(fMonitorTreeModel)
                    {                    
                        STAXMonitorTreeNode processNode = 
                            (STAXMonitorTreeNode)
                            fProcIDHashtable.get(processID);
                            
                        if (processNode == null)
                        {
                            System.out.println("DMB: processNode " + 
                                               processID + " is null");
                            System.out.println(fMonitorTreeModel);
                            System.exit(99);
                        }
                        
                        try
                        {
                            fMonitorTreeModel.removeNodeFromParent(processNode);
                        }
                        catch(NullPointerException e)
                        {
                            e.printStackTrace();                            
                        }
                        
                        fMonitorTreeProcessNodes.remove
                            (fProcIDHashtable.get(processID));
                        fMonitorTreeProcessStartTimes.remove
                            (fProcIDHashtable.get(processID));
                        fProcIDHashtable.remove(processID);                    
                        fDataHashtable.remove(processID);
                    }
                }
                else
                {
                    // XXX got a stop before the start?   
                }
            }
            else
            {
                // unknown status property
            }
        }
        else if (message.indexOf(";type=:6:subjob;") > -1)
        {   
            String jobID = STAXMonitorUtil.getValue(message, "jobID");
           
            if (message.indexOf(";status=:5:start;") > -1)
            {
                handleSubjob(jobID, block, message);

            } else if (message.indexOf(";status=:4:stop;") > -1)
            {    
                if (fSubjobIDHashtable.containsKey(jobID))
                {
                    fMonitorTreeModel.
                        removeNodeFromParent((STAXMonitorTreeNode)
                            fSubjobIDHashtable.get(jobID));
                    fMonitorTreeSubjobNodes.remove
                            (fSubjobIDHashtable.get(jobID));
                    fMonitorTreeSubjobStartTimes.remove
                            (fSubjobIDHashtable.get(jobID));
                    fCmdIDHashtable.remove(jobID);
                    fDataHashtable.remove(jobID);
                }
            }
        }
        else if (message.indexOf(";type=:3:job;") > -1)
        {
            if (message.indexOf(";status=:3:end;") > -1)
            {
                String result = STAXMonitorUtil.getValue(message, "result");

                fMonitorTreeModel.setRoot(new STAXMonitorTreeNode(null));                                                
                setTitle(kMonitorFrameTitle + " " +  "Machine:" + 
                         fStaxMachine + " JobID:" + fJobNumber + " " + 
                         fJobName + " <Completed>");
                fMonitorTree.setVisible(false);    	              
                stopMonitor();
            }
        }      
    }
    
    void handleCommand(String commandName, String block, String location, 
                       String requestNumber, String service, String request)
    {
        String commandID = commandName + ";" + location + ";" 
                           + requestNumber;    

        STAXMonitorTreeNode newNode = 
            new STAXMonitorTreeNode(commandName, 
            STAXMonitorTreeNode.commandNodeType);
            
        STAXMonitorTreeNode parentNode = (STAXMonitorTreeNode) 
                                          fMonitorTreeBlocks.get(block);
        fMonitorTreeModel.insertNodeInto(newNode, parentNode,
                                         parentNode.getChildCount());
        TreeNode[] parentNodes = 
            fMonitorTreeModel.getPathToRoot(parentNode);        
        
        fMonitorTree.expandPath(new TreePath(parentNodes));            
                
        fCmdIDHashtable.put(commandID, newNode);
        fMonitorTreeCommandNodes.put(newNode, commandID);
        
        Vector cmdDataVector = new Vector();
                
        addRow(cmdDataVector, "Location", location);                
        addRow(cmdDataVector, "Request Number", requestNumber);                
        addRow(cmdDataVector, "Service", service);                       
        addRow(cmdDataVector, "Request", request);
        
        STAFResult queryResult =
            fHandle.submit2(fStaxMachine, fStaxServiceName,
                            "query job " + fJobNumber + " stafcmd " +
                            requestNumber);
        
        if (queryResult.rc == 0)
        {
            fMonitorTreeCommandStartTimes.put(newNode, 
                STAXMonitorUtil.getCalendar(queryResult.result));
        
            String result = queryResult.result;
        
            int index = result.indexOf("Start Date");
            index = result.indexOf(":", index); 
            String startDate = result.substring(index + 2, index + 10);
        
            index = result.indexOf("Start Time");
            index = result.indexOf(":", index);         
            String startTime = result.substring(index + 2, index + 10);
        
            addRow(cmdDataVector, "Started", startDate + "-" + startTime);
        
            fDataHashtable.put(commandID, cmdDataVector);
        }
    }
    
    void handleSubjob(String jobID, String block, String message)
    {
        String subjobText = "Job " + jobID;
        
        String jobName = STAXMonitorUtil.getValue(message, "jobName");
            
        String function = STAXMonitorUtil.getValue(message, "function");
        
        String jobFile = STAXMonitorUtil.getValue(message, "jobfile");

        String jobFileMachine = STAXMonitorUtil.getValue(message,
                                                         "jobfilemachine");
        
        String functionArgs = STAXMonitorUtil.getValue(message, 
            "functionargs");
        
        String clearLogs = STAXMonitorUtil.getValue(message, "clearlogs");
        
        String autoMonitor = STAXMonitorUtil.getValue(message, "monitor");

        String logTCElapsedTime = STAXMonitorUtil.getValue(
            message, "logtcelapsedtime");

        String logTCNumStarts = STAXMonitorUtil.getValue(
            message, "logtcnumstarts");

        String logTCStartStop = STAXMonitorUtil.getValue(
            message, "logtcstartstop");
        
        String scriptFilesMachine = STAXMonitorUtil.getValue(message, 
            "scriptfilesmachine");

        String startDate = STAXMonitorUtil.getValue(message, "startdate");

        String startTime = STAXMonitorUtil.getValue(message, "starttime");
            
        if (!(jobName.equals("")))
        {
            subjobText += " - " + jobName;
        }

        STAXMonitorTreeNode newNode = 
            new STAXMonitorTreeNode(subjobText, 
            STAXMonitorTreeNode.subjobNodeType);
            
        STAXMonitorTreeNode parentNode = (STAXMonitorTreeNode) 
                                          fMonitorTreeBlocks.get(block);
        fMonitorTreeModel.insertNodeInto(newNode, parentNode,
                                         parentNode.getChildCount());
        TreeNode[] parentNodes = 
            fMonitorTreeModel.getPathToRoot(parentNode);        
        
        fMonitorTree.expandPath(new TreePath(parentNodes));            
                
        fSubjobIDHashtable.put(jobID, newNode);
        fMonitorTreeSubjobNodes.put(newNode, jobID);
        
        fMonitorTreeSubjobStartTimes.put(newNode, 
            STAXMonitorUtil.getCalendar2(startDate, startTime));
            
        Vector subjobDataVector = new Vector();
                
        addRow(subjobDataVector, "Job ID", jobID);
        addRow(subjobDataVector, "Job Name", jobName);
        addRow(subjobDataVector, "Clear Logs", clearLogs);
        addRow(subjobDataVector, "Log TC Elapsed Time", logTCElapsedTime);
        addRow(subjobDataVector, "Log TC Num Starts", logTCNumStarts);
        addRow(subjobDataVector, "Log TC Start/Stop", logTCStartStop);
        addRow(subjobDataVector, "Job File", jobFile);

        if (!jobFileMachine.equals(""))
        {
            addRow(subjobDataVector, "Job File Machine", jobFileMachine);
        }

        addRow(subjobDataVector, "Function", function);
        addRow(subjobDataVector, "Function Args", functionArgs);

        boolean endOfScripts = false;
        int scriptIndex = 0;
        int scriptNum = 0;
        
        while (!endOfScripts)
        {                    
            String scriptfile = STAXMonitorUtil.getValue(
                message, "scriptfile" + scriptIndex++);

            if (!scriptfile.equals(""))
            {
                if (scriptIndex == 1 && !scriptFilesMachine.equals(""))
                {
                    addRow(subjobDataVector, "Script Files Machine",
                           scriptFilesMachine);
                }

                addRow(subjobDataVector, "Script File #" + ++scriptNum,
                       scriptfile);
            }
            else
            {
               endOfScripts = true;
            }                    
        }
        
        endOfScripts = false;
        scriptIndex = 0;
        scriptNum = 0;

        while (!endOfScripts)
        {                    
            String script = STAXMonitorUtil.getValue(message, 
                "script" + scriptIndex++);
                                        
            if (!script.equals(""))
            {
                addRow(subjobDataVector, "Script #" + ++scriptNum, script);
            }
            else
            {
               endOfScripts = true;
            }                    
        }
        
        addRow(subjobDataVector, "Started", startDate + "-" + startTime);
        addRow(subjobDataVector, "Block", block);
        
        fDataHashtable.put(subjobText, subjobDataVector);
        
        if ((fAutoMonitorSubjobs == STAXMonitorFrame.AUTOMONITOR_ALWAYS) ||
            (fAutoMonitorSubjobs == STAXMonitorFrame.AUTOMONITOR_RECOMMENDED
            && autoMonitor.equals("true")))
        {
            fParentMonitor.monitorExistingJob(jobID, -1);
        }
     
        requestFocus();
    }
    
    void handleProcess(String message, String block, String processName, 
                       String location, String handleNumber, String command,
                       String commandMode, String commandShell, 
                       String processTitle, String parms, String workdir)
    {
        STAFResult monitorQueryResult = 
            fHandle.submit2(location, "monitor",
                            "query machine " +
                            location + " handle " + handleNumber);                    
         
        String monitorMessage = "";
        if (monitorQueryResult.rc == 0)
        {
            monitorMessage = monitorQueryResult.result;
            monitorMessage = 
                monitorMessage.substring(monitorMessage.indexOf(" "));
        }

        String processID = processName + ";" + location + ";" 
                               + handleNumber;
                                       
        STAXMonitorTreeNode newNode = new STAXMonitorTreeNode(processName,
            STAXMonitorTreeNode.processNodeType);
            
        synchronized(fMonitorTreeBlocks)
        {
            STAXMonitorTreeNode parentNode = (STAXMonitorTreeNode) 
                                              fMonitorTreeBlocks.get(block);           
        
            synchronized(fMonitorTreeModel)
            {
                fMonitorTreeModel.insertNodeInto(newNode, parentNode,
                                                 parentNode.getChildCount());
                TreeNode[] parentNodes = 
                    fMonitorTreeModel.getPathToRoot(parentNode);
                    
                fMonitorTree.expandPath(new TreePath(parentNodes));
            }
        }       
        
        fProcIDHashtable.put(processID, newNode);
        fMonitorTreeProcessNodes.put(newNode, processID);
        Vector procDataVector = new Vector();
                
        addRow(procDataVector, "Location", location);
                
        addRow(procDataVector, "Handle", handleNumber);
        
        addRow(procDataVector, "Command", command);
        
        addRow(procDataVector, "Command Mode", commandMode);
        
        if (!commandShell.equals(""))
        {
            addRow(procDataVector, "Command Shell", commandShell);
        }
        
        String workload = STAXMonitorUtil.getValue(message, "workload");
        if (!workload.equals(""))
        { 
            addRow(procDataVector, "Workload", workload);
        }
        
        if (!processTitle.equals(""))
        { 
            addRow(procDataVector, "Title", processTitle);
        }
        
        if (!parms.equals(""))
        { 
            addRow(procDataVector, "Parms", parms);
        }
        
        boolean endOfVars = false;
        int varIndex = 0;
        while (!endOfVars)
        {                    
            String var = STAXMonitorUtil.getValue(message, "var" + varIndex++);
                                        
            if (!var.equals(""))
            {
                addRow(procDataVector, "Var", var);
            }
            else
            {
               endOfVars = true;
            }                    
        }
        
        boolean endOfEnvs = false;
        int envIndex = 0;
        while (!endOfEnvs)
        {                    
            String env = STAXMonitorUtil.getValue(message, "env" + envIndex++);
                                        
            if (!env.equals(""))
            {
                addRow(procDataVector, "Env", env);
            }
            else
            {
                endOfEnvs = true;
            }                    
        }
        
        if (message.indexOf(";useprocessvars;") > -1)
        {
            addRow(procDataVector, "UseProcessVars", "");
        }
                
        String stopusing = STAXMonitorUtil.getValue(message, "stopusing");
        if (!stopusing.equals(""))
        { 
            addRow(procDataVector, "StopUsing", stopusing);
        }
                
        if (message.indexOf(";sameconsole;") > -1)
        {
            addRow(procDataVector, "SameConsole", "");
        }
                
        if (message.indexOf(";newconsole;") > -1)
        {
            addRow(procDataVector, "NewConsole", "");
        }
                
        String username = STAXMonitorUtil.getValue(message, "username");
        if (!username.equals(""))
        { 
            addRow(procDataVector, "Username", username);
        }
                
        String password = STAXMonitorUtil.getValue(message, "password");
        if (!password.equals(""))
        { 
            addRow(procDataVector, "Password", password);
        }
                
        if (message.indexOf(";disabledauthiserror;") > -1)
        {
            addRow(procDataVector, "DisabledAuthIsError", "");
        }
                
        if (message.indexOf(";ignoredisabledauth;") > -1)
        {
            addRow(procDataVector, "IgnoreDisabledAuth", "");
        }
                
        String stdin = STAXMonitorUtil.getValue(message, "stdin");
        if (!stdin.equals(""))
        { 
            addRow(procDataVector, "Stdin", stdin);
        }
                
        String stdout = STAXMonitorUtil.getValue(message, "stdout");
        if (!stdout.equals(""))
        { 
            addRow(procDataVector, "Stdout", stdout);
        }
                
        String stdoutappend = STAXMonitorUtil.getValue(message, "stdoutappend");
        if (!stdoutappend.equals(""))
        { 
            addRow(procDataVector, "StdoutAppend", stdoutappend);
        }
                
        if (message.indexOf(";stderrtostdout;") > -1)
        {
            addRow(procDataVector, "StderrToStdout", "");
        }
        else
        {
            String stderr = STAXMonitorUtil.getValue(message, "stderr");
            if (!stderr.equals(""))
            { 
                addRow(procDataVector, "Stderr", stderr);
            }
                
            String stderrappend = 
                STAXMonitorUtil.getValue(message, "stderrappend");
            if (!stderrappend.equals(""))
            { 
                addRow(procDataVector, "StderrAppend", stderrappend);
            }
        }

        if (message.indexOf(";returnstdout;") > -1)
        {
            addRow(procDataVector, "ReturnStdout", "");
        }
        
        if (message.indexOf(";returnstderr;") > -1)
        {
            addRow(procDataVector, "ReturnStderr", "");
        }
        
        boolean endOfReturnFiles = false;
        int returnFilesIndex = 0;
        while (!endOfReturnFiles)
        {                    
            String returnFile = STAXMonitorUtil.getValue(message, 
                "returnfile" + returnFilesIndex++);
                                        
            if (!(returnFile.equals("")))
            {
                addRow(procDataVector, "ReturnFile", returnFile);
            }
            else
            {
                endOfReturnFiles = true;
            }                    
        }
                
        String other = STAXMonitorUtil.getValue(message, "other");
        if (!other.equals(""))
        { 
            addRow(procDataVector, "Other", other);
        }
        
        STAFResult queryResult =
            fHandle.submit2(fStaxMachine, fStaxServiceName,
                            "query job " + fJobNumber + " process " +
                            location + ":" + handleNumber);
        
        if (queryResult.rc == 0)
        {           
            fMonitorTreeProcessStartTimes.put(newNode, 
                STAXMonitorUtil.getCalendar(queryResult.result));
        
            String result = queryResult.result;
        
            int index = result.indexOf("Start Date");
            index = result.indexOf(":", index); 
            String startDate = result.substring(index + 2, index + 10);
        
            index = result.indexOf("Start Time");
            index = result.indexOf(":", index);         
            String startTime = result.substring(index + 2, index + 10);
 
            addRow(procDataVector, "Started", startDate + "-" + startTime);
        
            fDataHashtable.put(processID, procDataVector);
        }
    }
       
    Vector handleBlock(String block, String status)
    {
        Vector heldBlocks = new Vector();
        
        if (block.indexOf(".") == -1)
        {
            // will occur only for main block
            if (!fMonitorTreeBlocks.containsKey(block))
            {
                // starting main					    
                STAXMonitorTreeNode newNode = 
                    new STAXMonitorTreeNode(block, 
                    STAXMonitorTreeNode.blockNodeType);

                if (status.startsWith("Held"))
                {                    
                    heldBlocks.add(newNode);
                }                    

                fMonitorTreeBlocks.put(block, newNode);
                fMonitorTreeBlockNodes.put(newNode, block);                
                
                fMonitorTreeModel.setRoot(newNode);
            }
        }
        else
        {
            String firstBlock = block.substring(0, block.indexOf("."));
            if (!fMonitorTreeBlocks.containsKey(firstBlock))
            {    
                 // this can occur if we are starting to monitor
                 // after the jobs has already started.  need to
                 // seed the hashmap with the first block (main)

                 STAXMonitorTreeNode newNode = new 
                     STAXMonitorTreeNode(firstBlock, 
                     STAXMonitorTreeNode.blockNodeType);            

                 fMonitorTreeBlocks.put(firstBlock, newNode);
                 fMonitorTreeBlockNodes.put(newNode, firstBlock);

                 fMonitorTreeModel.insertNodeInto(newNode,
                     (STAXMonitorTreeNode)fMonitorTreeModel.getRoot(), 0);                    
                
                 fMonitorTree.expandPath(new TreePath(
                     (STAXMonitorTreeNode)fMonitorTreeModel.getRoot()));
            }

            String blockSection;
            int lastIndex = 0;
            lastIndex = block.indexOf(".");
            boolean lastBlock = false;

            // we've already processed the first segment, main
            do 
            {
                int nextIndex = block.indexOf(".", lastIndex + 1);
    
                if (nextIndex == -1)
                {
                    // this means we've reach the final block segment
                    blockSection = block;                    
                    lastBlock = true;
                }
                else
                {
                    blockSection = block.substring(0, nextIndex);
                }

                if (!fMonitorTreeBlocks.containsKey(blockSection))
                {
                    String lastBlockSection = blockSection.substring(
                                           blockSection.lastIndexOf(".")+1);
                    STAXMonitorTreeNode newNode = new 
                        STAXMonitorTreeNode(lastBlockSection, 
                        STAXMonitorTreeNode.blockNodeType);                    
                    
                    if (lastBlock && status.startsWith("Held"))
                    {                        
                        heldBlocks.add(newNode);
                    }
                    
                    STAXMonitorTreeNode parentNode = (STAXMonitorTreeNode) 
                              (fMonitorTreeBlocks.get(blockSection.substring(0,
    	                                       blockSection.lastIndexOf("."))));
                    
                    fMonitorTreeModel.insertNodeInto(newNode, parentNode,
                                                 (parentNode.getChildCount()));                    
                    fMonitorTreeBlocks.put(blockSection, newNode);
                    fMonitorTreeBlockNodes.put(newNode, blockSection);

                    TreeNode[] parentNodes = fMonitorTreeModel.
                                                     getPathToRoot(parentNode);                                                     
                    
                    fMonitorTree.expandPath(new TreePath(parentNodes));                        
                }
                lastIndex = nextIndex;
            }
            while (!blockSection.equals(block));
        }
        
        return heldBlocks;
    }

    void handleJobMessage(String msg)
    {
        String message = STAXMonitorUtil.getValue(msg, "messagetext");
        int timestampIndex = message.indexOf(" ");
        
        Object rowData[] = new Object[2];
        rowData[0] = message.substring(0, timestampIndex);
        
        if (fMessageTableModel.getRowCount() == 
            (new Integer(fLimitMessages)).intValue())      
        {
            fMessageTableModel.removeRow(0);
        }

        if (fMessageTableModel.getRowCount() == 0)
        {
            int firstMsgLen = (message.substring(timestampIndex + 1)).length();
            
            String padding = "";
            
            for (int i = 0; i < 200 - firstMsgLen; i++)
            {
                padding += " ";
            }
            
            rowData[1] = message.substring(timestampIndex + 1) + padding;
        }
        else
        {
            rowData[1] = message.substring(timestampIndex + 1);
        }
        
        fMessageTableModel.addRow(rowData);
        
        STAXMonitorUtil.updateRowHeights(fMessageTable, 1);
        STAXMonitorUtil.sizeColumnsToFitText(fMessageTable);
        
        Rectangle rect = fMessageTable.getVisibleRect();
        Point point = rect.getLocation();        
        point.setLocation(rect.getX() + rect.getWidth(),
                          rect.getY() + rect.getHeight());
        int lastRow = fMessageTable.rowAtPoint(point);
        
        if (lastRow > (fMessageTableModel.getRowCount() - 5))
        {
            fMessageTable.scrollRectToVisible(new 
                Rectangle(0,fMessageTable.getPreferredSize().height,0,0));

            fMessageTable.scrollRectToVisible(new 
                Rectangle(0,fMessageTable.getPreferredSize().height,0,0));
        }
    }
    
    public void addActiveJobElementsNode(String type, String id, String block,
        ImageIcon image, JComponent component, Vector detailsData)
    {
        // pass the ID as the NodeText
        addActiveJobElementsNode(type, id, block, id, image, 
            component, detailsData);
    }

    public void addActiveJobElementsNode(String type, String id, String block,
        String nodeText, ImageIcon image, JComponent component,
        Vector detailsData)
    {
        STAXMonitorTreeNode newNode = 
            new STAXMonitorTreeNode(nodeText, type, image, component);
            
        STAXMonitorTreeNode parentNode = (STAXMonitorTreeNode) 
                                          fMonitorTreeBlocks.get(block);
        fMonitorTreeModel.insertNodeInto(newNode, parentNode,
                                         parentNode.getChildCount());
        TreeNode[] parentNodes = 
            fMonitorTreeModel.getPathToRoot(parentNode);        
        
        fMonitorTree.expandPath(new TreePath(parentNodes));            
                
        fPluginNodeHashtable.put(id, newNode);
        fPluginNodeToIDHashtable.put(newNode, id);
        fPluginNodeToNodeTextHashtable.put(newNode, nodeText);
        fPluginNodeDetailsHashtable.put(newNode, detailsData);
    }
    
    public void removeActiveJobElementsNode(String id, String block)
    {
        fMonitorTreeModel.removeNodeFromParent((STAXMonitorTreeNode)
            fPluginNodeHashtable.get(id));
        fPluginNodeHashtable.remove(id);
    }
    
    public void setActiveJobElementsNodeText(String id, String text)
    {
        STAXMonitorTreeNode node = 
            (STAXMonitorTreeNode)fPluginNodeHashtable.get(id);
        node.setPluginText(text);
        
        fMonitorTree.updateUI();
        
        if (fCurrentSelectedNode != null)
        {
            synchronized(fMonitorTree)
            {
                fMonitorTree.setSelectionPath(new 
                    TreePath(fCurrentSelectedNode.getPath()));
            }
        }
    }
    
    public void showErrorDialog(String message)
    {
        JTextPane messagePane = new JTextPane();                                                
        messagePane.setFont(new Font("Dialog", Font.BOLD, 12));
        messagePane.setEditable(false);                        
        messagePane.setText(message);
        messagePane.select(0,0);
        JScrollPane scrollPane = new JScrollPane(messagePane);
        scrollPane.setPreferredSize(new Dimension(400, 100));
        JOptionPane.showMessageDialog(this, scrollPane,
                                      "STAX Error",
                                      JOptionPane.ERROR_MESSAGE);
    }
    
    public void keyPressed(KeyEvent e) 
    {
            }
    
    public void keyReleased(KeyEvent e) {}
    public void keyTyped(KeyEvent e) {}
    
    public void mouseClicked(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    
    public void mousePressed(MouseEvent e) 
    {
        displayPopup(e);
    }
    
    public void mouseReleased(MouseEvent e) 
    {
        displayPopup(e);
    }
    
    public void displayPopup(MouseEvent e)
    {
        if (e.isPopupTrigger())
        {
            TreePath treePath = 
                fMonitorTree.getPathForLocation(e.getX(), e.getY());
        
            if(treePath != null)
            {
                STAXMonitorTreeNode treeNode = 
                    (STAXMonitorTreeNode)treePath.getLastPathComponent();
                
                if (treeNode.fNodeType == STAXMonitorTreeNode.blockNodeType)
                {
                    if (treeNode.fBlockStatus == 
                        STAXMonitorTreeNode.blockRunning)
                    {
                        fBlockHoldMenuItem.setEnabled(true);
                        fBlockReleaseMenuItem.setEnabled(false);
                        fBlockTerminateMenuItem.setEnabled(true);
                    }
                    else if (treeNode.fBlockStatus ==
                             STAXMonitorTreeNode.blockHeld)
                    {
                        fBlockHoldMenuItem.setEnabled(false);
                        fBlockReleaseMenuItem.setEnabled(true);
                        fBlockTerminateMenuItem.setEnabled(true);
                    }
                    else if (treeNode.fBlockStatus ==
                             STAXMonitorTreeNode.blockParentHeld)
                    {
                        fBlockHoldMenuItem.setEnabled(false);
                        fBlockReleaseMenuItem.setEnabled(false);
                        fBlockTerminateMenuItem.setEnabled(true);
                    }
                    
                    fMonitorTree.setSelectionPath(treePath);
                    fBlockPopupMenu.show(e.getComponent(), e.getX(), e.getY());
                }
                else if (treeNode.fNodeType == 
                         STAXMonitorTreeNode.subjobNodeType)
                {
                    fMonitorTree.setSelectionPath(treePath);
                    fSubjobPopupMenu.show(e.getComponent(), 
                        e.getX(), e.getY());
                }
            }
        }
    }
    
    public void valueChanged(TreeSelectionEvent e)
    {
        TreePath selectedPath = e.getPath();

        if ((e.getNewLeadSelectionPath() != null) &&
            (e.getNewLeadSelectionPath().getPath() != null) &&
            (e.getNewLeadSelectionPath().getPath()[0] != null))
        {
            String pathRoot = 
                e.getNewLeadSelectionPath().getPath()[0].toString();
            
            if ((e.getNewLeadSelectionPath().getPath().length == 1) &&
                (pathRoot != null))
            {
              
                if (!(pathRoot.equals("main")))
                
                {
                        // selection has been removed   
                    fCurrentSelection.setText("");
                        // next line is required due to Sun BugIDs 4398286, 4402032
                    fDetailsTable.setRowHeight(30);
                    fDetailsTableModel =
                        new STAXMonitorTableModel(fDataColumns, 0);
                    fDetailsModelSorter.setModel(fDetailsTableModel);
                           
                    updateDetailsTableRenderers();            
                    STAXMonitorUtil.sizeColumnsToFitText(fDetailsTable);
                    fDetailsTable.setVisible(false);
                    fDetailsTable.getTableHeader().setVisible(false);
                    
                    fBlockPopupMenu.setVisible(false);
                }
            }
        }
        
        if (fCurrentSelectedNode == 
            (STAXMonitorTreeNode)selectedPath.getLastPathComponent())
        {
            return;
        }
                
        fCurrentSelectedNode = 
            (STAXMonitorTreeNode)selectedPath.getLastPathComponent();
        String id = fCurrentSelectedNode.toString();       
 
        if (fCurrentSelectedNode == null)
        {
            return;
        }
       
        if (e.isAddedPath())
        {     
            fCurrentSelection.setText(id);
            fDetailsTable.setVisible(true);
            fDetailsTable.getTableHeader().setVisible(true);
            
            if (fMonitorTreeProcessNodes.containsKey(fCurrentSelectedNode))
            {
                id = (String)fMonitorTreeProcessNodes.
                    get(fCurrentSelectedNode);
            }
            
            if (fMonitorTreeCommandNodes.containsKey(fCurrentSelectedNode))
            {
                id = (String)fMonitorTreeCommandNodes.
                    get(fCurrentSelectedNode);
            }
            
            if (id == null)
            {
                return;
            }
            
            if (fDataHashtable.containsKey(id))
            {
                int idIndex = id.indexOf(";");
                
                if (idIndex != -1)
                {
                    updateCurrentSelection(id.substring(0, idIndex), 
                        (Vector)fDataHashtable.get(id));
                }
                else
                {
                    updateCurrentSelection(id, (Vector)fDataHashtable.get(id));
                }
            }
            else if (fPluginNodeDetailsHashtable.
                containsKey(fCurrentSelectedNode))
            {
                String pluginNodeText = (String)
                    fPluginNodeToNodeTextHashtable.get(fCurrentSelectedNode);
                updateCurrentSelection(pluginNodeText, 
                    (Vector)fPluginNodeDetailsHashtable.get(
                    fCurrentSelectedNode));
            }
            else
            {
                // next line is required due to Sun BugIDs 4398286, 4402032                
                fDetailsTable.setRowHeight(30);
                fDetailsTableModel =
                    new STAXMonitorTableModel(fDataColumns, 0);
                fDetailsModelSorter.setModel(fDetailsTableModel);
                   
                updateDetailsTableRenderers();
                STAXMonitorUtil.sizeColumnsToFitText(fDetailsTable);
                fDetailsTable.setVisible(false);
                fDetailsTable.getTableHeader().setVisible(false);
            }
            
            previousTreePath = selectedPath;
        }
    }
    
    public void updateCurrentSelection(String title, Vector tableData)
    {
        if (fViewableComponents.get("Current Selection") == null)
        {
            return;
        }

        if (fInfoPane.getTabCount() > 0)
        {   
            fInfoPane.setSelectedIndex(fInfoPane.
                indexOfTab("Current Selection"));
        }
        
        fCurrentSelection.setText(title);

        // next line is required due to Sun BugIDs 4398286, 4402032
        fDetailsTable.setRowHeight(30);
        fDetailsTableModel = new STAXMonitorTableModel(tableData,
            fDataColumns);                
        fDetailsModelSorter.setModel(fDetailsTableModel);                        
        updateDetailsTableRenderers();
        STAXMonitorUtil.updateRowHeights(fDetailsTable, 1);
        STAXMonitorUtil.sizeColumnsToFitText(fDetailsTable);
        fDetailsTable.setVisible(true);
        fDetailsTable.getTableHeader().setVisible(true);
    }
    
    public void updateDetailsTableRenderers()
    {   
        fDetailsTable.getColumnModel().getColumn(0).setCellRenderer(
            new STAXMonitorTableCellRenderer(Color.black));
    	  
        fDetailsTable.getColumnModel().getColumn(0).
            setHeaderRenderer(new STAXMonitorTableCellRenderer(Color.black,
                true, new Font("Dialog", Font.BOLD, 12)));
                                                                                                 
        fDetailsTable.getColumnModel().getColumn(1).setCellRenderer(
            new STAXMonitorTableCellRenderer(Color.black));
    	  
        fDetailsTable.getColumnModel().getColumn(1).
            setHeaderRenderer(new STAXMonitorTableCellRenderer(Color.black,
                true, new Font("Dialog", Font.BOLD, 12)));
    }   
    
    public String getJobNumber()
    {
        return fJobNumber;
    }
    
    public void addRow(Vector vector, String name, String value)
    {
        Vector newRow = new Vector(2);
        newRow.add(name);
        newRow.add(value);
        vector.add(newRow);
    }

    public void run()
    {
        STAFResult queueGetResult;
        boolean continueRunning = true;

        while (continueRunning)
        {
            queueGetResult = fHandle.submit2("local", "queue", "get wait");
            
            if (queueGetResult.rc != STAFResult.Ok)
            {
                // It is possible to receive a HandleDoesNotExist if
                // the MonitorFrame is being closed
                return;
            }
            
            String message = 
                STAXMonitorUtil.parseEventToMessage(queueGetResult.result);
            
            if (message.startsWith("STAXMonitorJob/End"))
            {
                continueRunning = false;                
            }            
            else
            {
                String type = STAXMonitorUtil.parseEventGetType(message);
                
                if (type.equalsIgnoreCase(fStaxServiceName + "/" +                                 
                    fStaxMachine + "/" + fJobNumber))
                {
                    String messageServiceName = 
                        message.substring(0, message.indexOf(";")-1);      
                    if (messageServiceName.equalsIgnoreCase
                           ("STAF/SERVICE/" + fEventServiceName))
                    {                    
                        String eventID = 
                            STAXMonitorUtil.parseEventGetEventID(message);

                        STAFResult ackResult =
                            fHandle.submit2(STAFHandle.ReqFireAndForget,
                                            fEventMachine, fEventServiceName,
                                            "acknowledge eventid " 
                                            + eventID);                                                
                    }               

                    message = 
                        STAXMonitorUtil.parseEventToMonitorKeyword(message);
                    
                    final String runnableMessage = message;
                    final Map runnableMap = STAXMonitorUtil.getMap(message);
                    
                    if (message.startsWith("Message"))
                    {
                        Runnable runnable = new Runnable()
                        {
                            public void run()
                            {
                                handleJobMessage(runnableMessage);
                            }
                        };
                        
                        try
                        {
                            SwingUtilities.invokeAndWait(runnable);
                        }
                        catch (InterruptedException e)
                        {
                        }
                        catch (InvocationTargetException e)
                        {
                        }
                    }
                    else if (message.startsWith("Job") ||
                             message.startsWith("Block") ||
                             message.startsWith("Process") ||
                             message.startsWith("STAFCommand") ||
                             message.startsWith("SubJob"))
                    {
                        Runnable runnable = new Runnable()
                        {
                            public void run()
                            {
                                handleJobEvent(runnableMessage);
                            }
                        };
                        
                        try
                        {
                            SwingUtilities.invokeAndWait(runnable);
                        }
                        catch (InterruptedException e)
                        {
                        }
                        catch (InvocationTargetException e)
                        {
                        }
                    }
                    
                    String eventType = message.substring(0, 
                        message.indexOf(";"));
                    
                    // process the extensions
                    
                    Enumeration plugins = fRegisteredPlugins.keys();
                    while (plugins.hasMoreElements())
                    {
                        final STAXMonitorExtension plugin = 
                            (STAXMonitorExtension)plugins.nextElement();
                        
                        Vector pluginRegisteredTypes = (Vector)
                            fRegisteredPlugins.get(plugin);
                       
                        if (pluginRegisteredTypes.contains(
                            eventType.toLowerCase()))
                        {
                            Runnable runnable = new Runnable()
                            {
                                public void run()
                                {
                                    plugin.handleEvent(runnableMap);
                                }
                            };
                        
                            try
                            {
                                SwingUtilities.invokeAndWait(runnable);
                            }
                            catch (InterruptedException e)
                            {
                            }
                            catch (InvocationTargetException e)
                            {
                                e.printStackTrace();
                            }                            
                        }
                    }
                }
            }
        }
    }

    void startMonitor()
    {
        if (fStaxMachine.equals("") || fJobNumber.equals(""))
        {
            JOptionPane.showMessageDialog(this,
                                      "You must specify both a STAX Machine" +
                                      " and a Job Number",
                                      "Error starting Job Monitor",
                                      JOptionPane.ERROR_MESSAGE);
        }
        else
        {
            String externalSubTypes = "";

            for (int i = 0; i < fExternalPluginNotificationTypes.size(); i++)
            {
                externalSubTypes += "SUBTYPE " + 
                    fExternalPluginNotificationTypes.elementAt(i) + " ";
            }
            
            STAFResult registerResult = fHandle.submit2(fEventMachine,
                              fEventServiceName, "REGISTER TYPE "
                              + fStaxServiceName.toUpperCase() + "/" + 
                              fStaxMachine + "/" + fJobNumber
                              + " SUBTYPE Job SUBTYPE Block SUBTYPE Process "
                              + "SUBTYPE STAFCommand SUBTYPE Message " 
                              + "SUBTYPE Testcase " 
                              + "SUBTYPE TestCaseStatus "
                              + "SUBTYPE subjob "
                              + externalSubTypes
                              + "MAXATTEMPTS 1 "
                              + "ACKNOWLEDGETIMEOUT 1000 BYHANDLE");

            if (registerResult.rc != 0)
            {
                JOptionPane.showMessageDialog(this,
                                   "Could not register for Job events\n" + 
                                   "RC = " + registerResult.rc, 
                                   "Error registering Job Monitor",
                                   JOptionPane.ERROR_MESSAGE);            
            }
            else
            {
                setTitle(kMonitorFrameTitle + "  " + 
    	                 fStaxMachine + " Job#" + fJobNumber + " " + fJobName);              
            }
        }
    }

    void stopMonitor()
    {   
        fMonitorStopped = true;
        
        String externalSubTypes = "";

        for (int i = 0; i < fExternalPluginNotificationTypes.size(); i++)
        {
            externalSubTypes += "SUBTYPE " + 
                fExternalPluginNotificationTypes.elementAt(i) + " ";
        }

        STAFResult unRegisterResult = fHandle.submit2(fEventMachine,
                              fEventServiceName, "UNREGISTER TYPE "
                              + fStaxServiceName.toUpperCase() + "/" + 
                              fStaxMachine + "/" + fJobNumber
                              + " SUBTYPE Job SUBTYPE Block SUBTYPE Process "
                              + "SUBTYPE STAFCommand SUBTYPE Testcase "
                              + "SUBTYPE TestcaseStatus SUBTYPE Message "
                              + "SUBTYPE subjob "
                              + externalSubTypes);

        if (unRegisterResult.rc != 0)
        {
            JOptionPane.showMessageDialog(this,
                                      "Could not unregister for Job events\n" 
                                      + "RC = " + unRegisterResult.rc,
                                      "Error unregistering Job Monitor",
                                      JOptionPane.ERROR_MESSAGE);
        }

        fHandle.submit2(STAFHandle.ReqFireAndForget,
                        "local", "queue",
                        "queue message STAXMonitorJob/End");
    }

    void closeMonitor()
    {
        if (fParentMonitor != null)
        {
            fParentMonitor.monitorExiting(fJobNumber);
        }
        fContinueProcessMonitor = false;
        fContinueElapsedTime = false;
        
        STAXMonitorUtil.freeHandle(fHandle.getHandle());
        
        Enumeration plugins = fRegisteredPlugins.keys();
        
        while (plugins.hasMoreElements())
        {
            STAXMonitorExtension plugin = 
               (STAXMonitorExtension)plugins.nextElement();
       
            plugin.term();
        }

        this.dispose();        
    }
    
    public int getProcessMonitorInterval()
    {
        return fParentMonitor.getProcessMonitorInterval();
    }
    
    public int getElapsedTimeInterval()
    {
        return fParentMonitor.getElapsedTimeInterval();
    }
    
    class STAFProcessMonitor extends Thread
    {
        public void run()
        {
            final int frequency = getProcessMonitorInterval();
            
            if (frequency == 0)
                return;
                       
            while (fContinueProcessMonitor)
            {
                int waitTime = frequency;
                
                final Enumeration processNodeKeys = 
                    fMonitorTreeProcessNodes.keys();
                
                int numProcesses = fMonitorTreeProcessNodes.size();
    
                Runnable runnable = new Runnable()
                {
                    public void run()
                    {
                        while (fContinueProcessMonitor && 
                               processNodeKeys.hasMoreElements())
                        {                    
                            STAXMonitorTreeNode processNode = null;
                            String processID = null;                        

                            synchronized (processNodeKeys)
                            {                            
                                if (processNodeKeys.hasMoreElements())
                                {               
                                    processNode = (STAXMonitorTreeNode)
                                        processNodeKeys.nextElement();
                                }
                            }
                            
                            if (processNode != null)
                            {
                                synchronized(fMonitorTreeProcessNodes)
                                {
                                    processID = (String)
                                        fMonitorTreeProcessNodes.get(
                                        processNode);
                                }
                            }
                    
                            if (processID != null)
                            {
                                int handleIndex = processID.lastIndexOf(";");
                                String handle = 
                                    processID.substring(handleIndex + 1);
                                processID = 
                                    processID.substring(0, handleIndex);
                    
                                int locationIndex = processID.lastIndexOf(";");
                                String location = 
                                    processID.substring(locationIndex + 1);
                                if (location.equals("local"))
                                {
                                    location = fStaxMachine;
                                }
                    
                                String processName = 
                                    processID.substring(0, locationIndex);
     
                                STAFResult monitorQueryResult = 
                                    fHandle.submit2(location, "monitor",
                                                    "query machine " +
                                                    location + " handle " + 
                                                    handle);
                                       
                                if (monitorQueryResult.rc == 0)
                                {
                                    String message = monitorQueryResult.result;
                                    message = message.substring(
                                                  message.indexOf(" "));
                                
                                    processNode.setProcessMonitorText(message);
                                    
                                    fMonitorTreeModel.nodeChanged(processNode);
                                    
                                    if (fCurrentSelectedNode != null)
                                    {
                                        synchronized(fMonitorTree)
                                        {
                                            fMonitorTree.setSelectionPath(new 
                                                TreePath(fCurrentSelectedNode.
                                                    getPath()));
                                        }
                                    }
                                }                                
                            }                                                                                   
                        }
                    }
                };                    
                    
                try
                {
                    SwingUtilities.invokeAndWait(runnable);
                }
                catch (InterruptedException ex)
                {
                    ex.printStackTrace();
                }
                catch (InvocationTargetException ex)
                {
                    ex.printStackTrace();
                }
                
                waitTime = frequency;
                try
                {                        
                    Thread.sleep(waitTime);
                }
                catch (InterruptedException ex)
                {
                }                              
            }
        }
    }
    
    class MonitorElapsedTime extends Thread
    {
        public void run()
        {
            final int waitTime = getElapsedTimeInterval();
            
            if (waitTime == 0)
                return;
            
            while (fContinueElapsedTime)
            {
                // handle Processes
                final Enumeration processElapsedTimeKeys = 
                    fMonitorTreeProcessStartTimes.keys();                                   
                    
                Runnable processRunnable = new Runnable()
                {
                    public void run()
                    {
                        while (fContinueElapsedTime && 
                               processElapsedTimeKeys.hasMoreElements())
                        {
                            STAXMonitorTreeNode processNode = 
                                (STAXMonitorTreeNode)
                                processElapsedTimeKeys.nextElement();
                                
                            Calendar processStarted = (Calendar)
                                fMonitorTreeProcessStartTimes.get(processNode);

                            processNode.setElapsedTime(
                                STAXMonitorUtil.getElapsedTime(processStarted));
                                
                            fMonitorTreeModel.nodeChanged(processNode);
                        }
                              
                        if (fCurrentSelectedNode != null)
                        {
                            synchronized(fMonitorTree)
                            {
                                fMonitorTree.setSelectionPath(new 
                                    TreePath(fCurrentSelectedNode.
                                        getPath()));
                            }
                        }
                    }
                };
                    
                try
                {
                    SwingUtilities.invokeAndWait(processRunnable);
                }
                catch (InterruptedException ex)
                {
                     ex.printStackTrace();
                }
                catch (InvocationTargetException ex)
                {
                     ex.printStackTrace();
                }
                
                // handle Commands
                final Enumeration commandElapsedTimeKeys = 
                    fMonitorTreeCommandStartTimes.keys();                
                    
                Runnable commandRunnable = new Runnable()
                {
                    public void run()
                    {
                        while (fContinueElapsedTime && 
                               commandElapsedTimeKeys.hasMoreElements())
                        {                                                              
                            STAXMonitorTreeNode commandNode = 
                                (STAXMonitorTreeNode)
                                commandElapsedTimeKeys.nextElement();
                            Calendar commandStarted = (Calendar)
                                fMonitorTreeCommandStartTimes.get(commandNode);

                            commandNode.setElapsedTime(
                               STAXMonitorUtil.getElapsedTime(commandStarted));
                               
                            fMonitorTreeModel.nodeChanged(commandNode);
                        }
                            
                        if (fCurrentSelectedNode != null)
                        {
                            synchronized(fMonitorTree)
                            {
                                fMonitorTree.setSelectionPath(new 
                                    TreePath(fCurrentSelectedNode.
                                        getPath()));
                            }
                        }
                    }
                };
                    
                try
                {
                    SwingUtilities.invokeAndWait(commandRunnable);
                }
                catch (InterruptedException ex)
                {
                     ex.printStackTrace();
                }
                catch (InvocationTargetException ex)
                {
                     ex.printStackTrace();
                }
                
                // handle Subjobs
                final Enumeration subjobElapsedTimeKeys = 
                    fMonitorTreeSubjobStartTimes.keys();                
                    
                Runnable subjobRunnable = new Runnable()
                {
                    public void run()
                    {
                        while (fContinueElapsedTime && 
                               subjobElapsedTimeKeys.hasMoreElements())
                        {                                                              
                            STAXMonitorTreeNode subjobNode = 
                                (STAXMonitorTreeNode)
                                subjobElapsedTimeKeys.nextElement();
                            Calendar subjobStarted = (Calendar)
                                fMonitorTreeSubjobStartTimes.get(subjobNode);

                            subjobNode.setElapsedTime(
                                STAXMonitorUtil.getElapsedTime(subjobStarted));
                                                        
                            fMonitorTreeModel.nodeChanged(subjobNode);                                              
                        }
                        
                        if (fCurrentSelectedNode != null)
                        {
                            synchronized(fMonitorTree)
                            {
                                fMonitorTree.setSelectionPath(new 
                                    TreePath(fCurrentSelectedNode.
                                        getPath()));
                            }
                        }
                    }
                };
                    
                try
                {
                    SwingUtilities.invokeAndWait(subjobRunnable);
                }
                catch (InterruptedException ex)
                {
                     ex.printStackTrace();
                }
                catch (InvocationTargetException ex)
                {
                     ex.printStackTrace();
                }
                
                try
                {                    
                    Thread.sleep(waitTime);
                }
                catch (InterruptedException ex)
                {
                }
            }
        }
    }
    
}