/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2002                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service.stax;

import com.ibm.staf.*;
import com.ibm.staf.service.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.table.*;

class STAXMonitorLogViewer extends JFrame
{
    static String lineSep;

    static String helpText = "\nSTAXMonitorLogViewer Help\n\n" +
        "-name <logName>\n" +
        "-machine <machineName>\n" + 
        "-help\n" + "-version";

    static String kVersion = "1.2.0";

    public static void main(String argv[])
    {
        new STAXMonitorLogViewer(new JFrame(), null, argv);
    }
    
    public STAXMonitorLogViewer(Component parent, STAFHandle handle, 
        String argv[])
    {
        this.parent = parent;

        // If a handle was specified, don't do a system exit
        if (handle != null)
            fSystemExit = false;
        
        if ((argv.length < 1) || (argv.length > 6))
        {
            System.out.println("Invalid parameters");
            System.out.println(helpText);
            System.exit(1);
        }
        else
        {
            if (argv[0].equalsIgnoreCase("-HELP"))
            {
                System.out.println(helpText);
                System.exit(0);
            }
            else if (argv[0].equalsIgnoreCase("-VERSION"))
            {
                System.out.println(kVersion);
                System.exit(0);
            }
            else
            {
                for (int i = 0; i < argv.length; i++)
                {
                    if (argv[i].equals("-machine"))
                    {
                        fMachine = argv[i+1];
                    }
                    else if (argv[i].equals("-name"))
                    {
                        fLogName = argv[i+1];
                    }
                }
            }
        }
                
        if (fLogName == null)
        {
            System.out.println(helpText);
            System.exit(1);
        }
        
        try
        {
            if (handle == null)
            {
                fHandle = new STAFHandle("STAFLogViewer");
            }
            else
            {
                STAFResult createHandleResult = handle.submit2("local",
                    "handle", "create handle name STAXMonitorLogViewer");
                
                if (createHandleResult.rc == 0)
                {                
                    fHandle = new STAFHandle(new 
                        Integer(createHandleResult.result).intValue());
                }
                else
                {
                    fHandle = handle;
                }
            }
        }
        catch(STAFException e)
        {
            System.out.println("Error registering with STAF, RC: " + e.rc);
            //e.printStackTrace();

            if (! fSystemExit)
                return;
            else
                System.exit(0);
        }
       
        STAFResult stafResult = null;
        
        if (fMachine.equalsIgnoreCase("local"))
        {
            fMachine = (fHandle.submit2("local", "var", 
                "resolve {STAF/Config/EffectiveMachine}")).result;
        }

        lineSep = (fHandle.submit2("local", "var", 
            "resolve {STAF/Config/Sep/Line}")).result;
            
        if (fLogName.indexOf("_Service") > -1)
        {
            stafResult = fHandle.submit2(
                fMachine, "LOG", "QUERY ALL MACHINE " + fMachine +
                " LOGNAME " + fLogName);
                
            if (stafResult.rc == 7)
            {
                // Submit query without the ALL option which was added
                // in STAF V2.6.6 (in case the STAX service machine is
                // running on a machine whose STAF version < STAF V2.6.6
                stafResult = fHandle.submit2(
                    fMachine, "LOG", "QUERY MACHINE " + fMachine +
                    " LOGNAME " + fLogName);
            }

            if ((stafResult.rc != 0) && (stafResult.rc != 17))
            {
                if (stafResult.rc == 48)
                {
                    JOptionPane.showMessageDialog(parent,
                        "STAX Service Log has no entries", 
                        "No STAX Service Log Entries",
                        JOptionPane.INFORMATION_MESSAGE);
                }
                else
                {
                    System.out.println("Error querying log.  RC=" + 
                        stafResult.rc);
                }

                if (! fSystemExit)
                    return;
                else
                    System.exit(0);
            }

            String queryResult = stafResult.result;
                
            int levelIndex = queryResult.lastIndexOf("|Start|JobID: 1,");

            if (levelIndex < 1)
            {
                JOptionPane.showMessageDialog(parent,
                    "STAX Service Log has no entries", 
                    "No STAX Service Log Entries",
                    JOptionPane.INFORMATION_MESSAGE);


                if (! fSystemExit)
                    return;
                else
                    System.exit(0);
            }
                
            int logNameIndex = queryResult.substring(0, 
                levelIndex - 1).lastIndexOf("|");
           
            int handleIndex = queryResult.substring(0, 
                logNameIndex - 1).lastIndexOf("|");
        
            int machineIndex = queryResult.substring(0, 
                handleIndex - 1).lastIndexOf("|");
            
            fStartDateTime = queryResult.substring(machineIndex - 17,
                machineIndex);
           
            fStartDateTime = fStartDateTime.replace('-', '@');
        }
        else
        {
            if (fLogName.endsWith("_User"))
            {
                fJobLogName = fLogName.substring(0, fLogName.length() - 5);
            }
            else
            {
                fJobLogName = fLogName;
            }
            
            stafResult = fHandle.submit2(
                fMachine, "LOG", "QUERY ALL MACHINE " + fMachine +
                " LOGNAME " + fJobLogName);
                
            if (stafResult.rc == 7)
            {
                // Submit query without the ALL option which was added
                // in STAF V2.6.6 (in case the STAX service machine is
                // running on a machine whose STAF version < STAF V2.6.6
                stafResult = fHandle.submit2(
                    fMachine, "LOG", "QUERY MACHINE " + fMachine +
                    " LOGNAME " + fJobLogName);
            }

            if ((stafResult.rc != 0) && (stafResult.rc != 17))
            {
                if (stafResult.rc == 48)
                {
                    JOptionPane.showMessageDialog(parent,
                        "Job Log has no entries", "No Job Log Entries",
                        JOptionPane.INFORMATION_MESSAGE);
                }
                else
                {
                    System.out.println("Error querying log.  RC=" + 
                        stafResult.rc);
                }

                if (! fSystemExit)
                    return;
                else
                    System.exit(0);
            }

            String queryResult = stafResult.result;
                
            int levelIndex = queryResult.lastIndexOf("|Start|JobID: ");
                
            int logNameIndex = queryResult.substring(0, 
                levelIndex - 1).lastIndexOf("|");
           
            int handleIndex = queryResult.substring(0, 
                logNameIndex - 1).lastIndexOf("|");
        
            int machineIndex = queryResult.substring(0, 
                handleIndex - 1).lastIndexOf("|");
            
            fStartDateTime = queryResult.substring(machineIndex - 17,
                machineIndex);
           
            fStartDateTime = fStartDateTime.replace('-', '@');
        }
        
        STAFLogFrame logFrame = 
            new STAFLogFrame(fMachine, fLogName);
                            
        Vector logLines = refreshTable();
        if (logLines == null)
        {
            if (! fSystemExit)
                return;
            else
                System.exit(0);
        }
        
        if (logLines.size() == 0)
        {
            JOptionPane.showMessageDialog(parent,
                "Job Log has no entries", "No Job Log Entries",
                JOptionPane.INFORMATION_MESSAGE);

            if (! fSystemExit)
                return;
            else
                System.exit(0);
        }
        
        logFrame.setSize(800, 400);
        logFrame.show();
            
        String osName = System.getProperties().getProperty("os.name");

        if (osName.equals("Windows 2000")) 
        {
            logFrame.setState(JFrame.ICONIFIED);
            logFrame.setState(JFrame.NORMAL);
        }
        else 
        {
            logFrame.toFront();
        }
    }
    
    public Vector getLogLines()
    {
        STAFResult stafResult = null;    
        Vector logLines;
        
        String startDateTime = "";
        
        if (fStartDateTime != null)
        {
            startDateTime = " from " + fStartDateTime;
        }
        
        stafResult = fHandle.submit2(
            fMachine, "LOG", "QUERY ALL MACHINE " + fMachine +
            " LOGNAME " + fLogName + startDateTime + getLogMask());
        
        if (stafResult.rc == 7)
        {
            // Submit query without the ALL option which was added
            // in STAF V2.6.6 (in case the STAX service machine is
            // running on a machine whose STAF version < STAF V2.6.6
            stafResult = fHandle.submit2(
                fMachine, "LOG", "QUERY MACHINE " + fMachine +
                " LOGNAME " + fLogName + startDateTime + getLogMask());
        }

        if ((stafResult.rc != 0) && (stafResult.rc != 17))
        {
            if (stafResult.rc == 48)
            {
                JOptionPane.showMessageDialog(parent,
                    "Job Log has no entries", "No Job Log Entries",
                    JOptionPane.INFORMATION_MESSAGE);
            }
            else
            {
                System.out.println("Error querying log.  RC=" + stafResult.rc);
            }

            return null;
        }                    

        BufferedReader logFile = new BufferedReader(
            new StringReader(stafResult.result));

        try
        {
            logLines = new Vector();
             
            for (String theLogLine = logFile.readLine();
                theLogLine != null;
                theLogLine = logFile.readLine())
            {
                Vector thisLogData = new Vector();

                int index = theLogLine.indexOf("|");

                // A new log line has a format of:
                // yyyymmdd-hh:mm:ss|machine|handle#|handleName|level|msg text

                if ((index == 17) && (occurrences(theLogLine, "|") >= 5) &&
                    (theLogLine.indexOf('-') == 8) &&
                    (theLogLine.indexOf(':') == 11) &&
                    (theLogLine.indexOf(':', 12) == 14))
                {
                    // This is a new log line
                    thisLogData.add(theLogLine.substring(0, index));

                    index = theLogLine.indexOf('|', index + 1);
                    index = theLogLine.indexOf('|', index + 1);
                    index = theLogLine.indexOf('|', index + 1);

                    int msgIndex = theLogLine.indexOf('|', index + 1);

                    thisLogData.add(theLogLine.substring(index + 1,
                                                             msgIndex));
                    thisLogData.add(theLogLine.substring(msgIndex + 1));
                    logLines.add(thisLogData);
                }
                else if (logLines.size() > 0)
                {
                    // This is a continuation of the previous log line
                    Vector originalVector = (Vector)
                        logLines.get(logLines.size() - 1);
                    String appendedLine = 
                        (String)(originalVector.lastElement()) + lineSep +
                        theLogLine;
                    originalVector.setElementAt(appendedLine, 2);
                    logLines.setElementAt(originalVector, logLines.size() - 1);
                }
                else
                {
                    // This is an unexpected log record format
                    thisLogData.add("yyyymmdd-hh:mm:ss");
                    thisLogData.add("?????");
                    thisLogData.add("Unexpected log record format. " +
                                    "Report to STAF developers. " +
                                    "Log record=\n" + theLogLine);
                    logLines.add(thisLogData);
                }
            }
            
            return logLines;
        }
        catch(IOException e)
        {
            e.printStackTrace();
            return null;
        }
        finally
        {
            try
            {
                logFile.close();
            }
            catch (IOException e)
            {
                e.printStackTrace();
                return null;
            }
        }
    }
    
    public String getLogMask()
    {
        String logMask = "";
        
        if (fLevelFatal.isSelected())
        {
            logMask += " FATAL";
        }
        
        if (fLevelError.isSelected())
        {
            logMask += " ERROR";
        }
        
        if (fLevelWarning.isSelected())
        {
            logMask += " WARNING";
        }
        
        if (fLevelInfo.isSelected())
        {
            logMask += " INFO";
        }
        
        if (fLevelTrace.isSelected())
        {
            logMask += " TRACE";
        }
        
        if (fLevelTrace2.isSelected())
        {
            logMask += " TRACE2";
        }
        
        if (fLevelTrace3.isSelected())
        {
            logMask += " TRACE3";
        }
        
        if (fLevelDebug.isSelected())
        {
            logMask += " DEBUG";
        }
        
        if (fLevelDebug2.isSelected())
        {
            logMask += " DEBUG2";
        }
        
        if (fLevelDebug3.isSelected())
        {
            logMask += " DEBUG3";
        }
        
        if (fLevelStart.isSelected())
        {
            logMask += " START";
        }
        
        if (fLevelStop.isSelected())
        {
            logMask += " STOP";
        }
        
        if (fLevelPass.isSelected())
        {
            logMask += " PASS";
        }
        
        if (fLevelFail.isSelected())
        {
            logMask += " FAIL";
        }
        
        if (fLevelStatus.isSelected())
        {
            logMask += " STATUS";
        }
        
        if (fLevelUser1.isSelected())
        {
            logMask += " USER1";
        }
        
        if (fLevelUser2.isSelected())
        {
            logMask += " USER2";
        }
        
        if (fLevelUser3.isSelected())
        {
            logMask += " USER3";
        }
        
        if (fLevelUser4.isSelected())
        {
            logMask += " USER4";
        }
        
        if (fLevelUser5.isSelected())
        {
            logMask += " USER5";
        }
        
        if (fLevelUser6.isSelected())
        {
            logMask += " USER6";
        }
        
        if (fLevelUser7.isSelected())
        {
            logMask += " USER7";
        }
        
        if (fLevelUser8.isSelected())
        {
            logMask += " USER8";
        }
        
        if (!(logMask.equals("")))
        {
            logMask = " LEVELMASK" + logMask;
        }
        else
        {
            logMask = " LEVELMASK" + " 00000000000000000000000000000000";
        }
        
        return logMask;
    }
    
    public int occurrences(String str, String match)
    {
        int count = 0;
        int index =  0;
        
        while (index != -1)
        {
            index = str.indexOf(match, index + 1);
            if (index > -1) 
                count++;
        }
        
        return count;
    }
    
    public class STAFLogFrame extends JFrame implements ActionListener
    {
        public STAFLogFrame(String machine, String logName)
        {
            super("Machine: " + machine + "  " + logName);

            JMenuBar mainMenuBar = new JMenuBar();
            setJMenuBar(mainMenuBar);
            fFileMenu = new JMenu("File");
            mainMenuBar.add(fFileMenu);
            fViewMenu = new JMenu("View");
            mainMenuBar.add(fViewMenu);
            fFileExit = new JMenuItem("Exit");
            fFileExit.addActionListener(this);
            fFileMenu.add(fFileExit);
            fViewRefresh = new JMenuItem("Refresh");
            fViewRefresh.addActionListener(this);
            fViewMenu.add(fViewRefresh);
            fLevels = new JMenu("Levels");
            mainMenuBar.add(fLevels);
            fLevelAll = new JCheckBoxMenuItem("All", true);
            fLevelAll.addActionListener(this);
            fLevels.add(fLevelAll);
            fLevels.addSeparator();

            fLevelFatal = new JCheckBoxMenuItem("Fatal", true);
            fLevelFatal.addActionListener(this);
            fLevels.add(fLevelFatal);
            fLevelError = new JCheckBoxMenuItem("Error", true);
            fLevelError.addActionListener(this);
            fLevels.add(fLevelError);
            fLevelWarning = new JCheckBoxMenuItem("Warning", true);
            fLevelWarning.addActionListener(this);            
            fLevels.add(fLevelWarning);
            fLevelInfo = new JCheckBoxMenuItem("Info", true);
            fLevelInfo.addActionListener(this);            
            fLevels.add(fLevelInfo);
            fLevelTrace = new JCheckBoxMenuItem("Trace", true);
            fLevelTrace.addActionListener(this);            
            fLevels.add(fLevelTrace);
            fLevelTrace2 = new JCheckBoxMenuItem("Trace2", true);
            fLevelTrace2.addActionListener(this);            
            fLevels.add(fLevelTrace2);
            fLevelTrace3 = new JCheckBoxMenuItem("Trace3", true);
            fLevelTrace3.addActionListener(this);            
            fLevels.add(fLevelTrace3);
            fLevelDebug = new JCheckBoxMenuItem("Debug", true);
            fLevelDebug.addActionListener(this);            
            fLevels.add(fLevelDebug);
            fLevelDebug2 = new JCheckBoxMenuItem("Debug2", true);
            fLevelDebug2.addActionListener(this);            
            fLevels.add(fLevelDebug2);
            fLevelDebug3 = new JCheckBoxMenuItem("Debug3", true);
            fLevelDebug3.addActionListener(this);            
            fLevels.add(fLevelDebug3);
            fLevelStart = new JCheckBoxMenuItem("Start", true);
            fLevelStart.addActionListener(this);            
            fLevels.add(fLevelStart);
            fLevelStop = new JCheckBoxMenuItem("Stop", true);
            fLevelStop.addActionListener(this);            
            fLevels.add(fLevelStop);
            fLevelPass = new JCheckBoxMenuItem("Pass", true);
            fLevelPass.addActionListener(this);            
            fLevels.add(fLevelPass);
            fLevelFail = new JCheckBoxMenuItem("Fail", true);
            fLevelFail.addActionListener(this);            
            fLevels.add(fLevelFail);
            fLevelStatus = new JCheckBoxMenuItem("Status", true);
            fLevelStatus.addActionListener(this);            
            fLevels.add(fLevelStatus);
            fLevelUser1 = new JCheckBoxMenuItem("User1", true);
            fLevelUser1.addActionListener(this);            
            fLevels.add(fLevelUser1);
            fLevelUser2 = new JCheckBoxMenuItem("User2", true);
            fLevelUser2.addActionListener(this);            
            fLevels.add(fLevelUser2);
            fLevelUser3 = new JCheckBoxMenuItem("User3", true);
            fLevelUser3.addActionListener(this);            
            fLevels.add(fLevelUser3);
            fLevelUser4 = new JCheckBoxMenuItem("User4", true);
            fLevelUser4.addActionListener(this);            
            fLevels.add(fLevelUser4);
            fLevelUser5 = new JCheckBoxMenuItem("User5", true);
            fLevelUser5.addActionListener(this);            
            fLevels.add(fLevelUser5);
            fLevelUser6 = new JCheckBoxMenuItem("User6", true);
            fLevelUser6.addActionListener(this);            
            fLevels.add(fLevelUser6);
            fLevelUser7 = new JCheckBoxMenuItem("User7", true);
            fLevelUser7.addActionListener(this);            
            fLevels.add(fLevelUser7);
            fLevelUser8 = new JCheckBoxMenuItem("User8", true);
            fLevelUser8.addActionListener(this);            
            fLevels.add(fLevelUser8);
            
            columnNames = new Vector();
            columnNames.add(new String("Timestamp"));
            columnNames.add(new String("Level"));
            columnNames.add(new String("Message"));

            fLogTable = new JTable();
            
            fLogTable.setRowSelectionAllowed(true);
            fLogTable.setColumnSelectionAllowed(false);
            fLogTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
            
            STAXMonitorUtil.updateRowHeights(fLogTable, 2);
            STAXMonitorUtil.sizeColumnsToFitText(fLogTable);
            
            JScrollPane logScroll = new JScrollPane(fLogTable);
            getContentPane().add(logScroll);

            addWindowListener(new WindowAdapter()
            {
                public void windowClosing(WindowEvent event)
	        {
                    if (! fSystemExit)
                        dispose();
                    else
                    {
                        System.exit(0);
                    }
	        }
	    });
        }

        public void actionPerformed(ActionEvent e) 
        {
            if (e.getSource() == fFileExit)
            {
                if (! fSystemExit)
                    dispose();
                else
                {
                    System.exit(0);
                }
            }
            else if (e.getSource() == fViewRefresh)
            {
                refreshTable();
            }
            else if ((e.getSource() == fLevelAll) &&
                fLevelAll.isSelected())
            {                
                fLevelFatal.setSelected(true);
                fLevelError.setSelected(true);
                fLevelWarning.setSelected(true);
                fLevelInfo.setSelected(true);
                fLevelTrace.setSelected(true);
                fLevelTrace2.setSelected(true);
                fLevelTrace3.setSelected(true);
                fLevelDebug.setSelected(true);
                fLevelDebug2.setSelected(true);
                fLevelDebug3.setSelected(true);
                fLevelStart.setSelected(true);
                fLevelStop.setSelected(true);
                fLevelPass.setSelected(true);
                fLevelFail.setSelected(true);
                fLevelStatus.setSelected(true);
                fLevelUser1.setSelected(true);
                fLevelUser2.setSelected(true);
                fLevelUser3.setSelected(true);
                fLevelUser4.setSelected(true);
                fLevelUser5.setSelected(true);
                fLevelUser6.setSelected(true);
                fLevelUser7.setSelected(true);
                fLevelUser8.setSelected(true);
                
                refreshTable();
            }
            else if ((e.getSource() == fLevelAll) &&
                !(fLevelAll.isSelected()))
            {                
                fLevelFatal.setSelected(false);
                fLevelError.setSelected(false);
                fLevelWarning.setSelected(false);
                fLevelInfo.setSelected(false);
                fLevelTrace.setSelected(false);
                fLevelTrace2.setSelected(false);
                fLevelTrace3.setSelected(false);
                fLevelDebug.setSelected(false);
                fLevelDebug2.setSelected(false);
                fLevelDebug3.setSelected(false);
                fLevelStart.setSelected(false);
                fLevelStop.setSelected(false);
                fLevelPass.setSelected(false);
                fLevelFail.setSelected(false);
                fLevelStatus.setSelected(false);
                fLevelUser1.setSelected(false);
                fLevelUser2.setSelected(false);
                fLevelUser3.setSelected(false);
                fLevelUser4.setSelected(false);
                fLevelUser5.setSelected(false);
                fLevelUser6.setSelected(false);
                fLevelUser7.setSelected(false);
                fLevelUser8.setSelected(false);
                
                refreshTable();
            }
            else if ((e.getSource() == fLevelFatal))
            {
                refreshTable();
                
                if (!(fLevelFatal.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelError))
            {
                refreshTable();
                
                if (!(fLevelError.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelWarning))
            {
                refreshTable();
                
                if (!(fLevelWarning.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelInfo))
            {
                refreshTable();
                
                if (!(fLevelInfo.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelTrace))
            {
                refreshTable();
                
                if (!(fLevelTrace.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelTrace2))
            {
                refreshTable();
                
                if (!(fLevelTrace2.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelTrace3))
            {
                refreshTable();
                
                if (!(fLevelTrace3.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelDebug))
            {
                refreshTable();
                
                if (!(fLevelDebug.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelDebug2))
            {
                refreshTable();
                
                if (!(fLevelDebug2.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelDebug3))
            {
                refreshTable();
                
                if (!(fLevelDebug3.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelStart))
            {
                refreshTable();
                
                if (!(fLevelStart.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelStop))
            {
                refreshTable();
                
                if (!(fLevelStop.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelPass))
            {
                refreshTable();
                
                if (!(fLevelPass.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelFail))
            {
                refreshTable();
                
                if (!(fLevelFail.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelStatus))
            {
                refreshTable();
                
                if (!(fLevelStatus.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelUser1))
            {
                refreshTable();
                
                if (!(fLevelUser1.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelUser2))
            {
                refreshTable();
                
                if (!(fLevelUser2.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelUser3))
            {
                refreshTable();
                
                if (!(fLevelUser3.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelUser4))
            {
                refreshTable();
                
                if (!(fLevelUser4.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelUser5))
            {
                refreshTable();
                
                if (!(fLevelUser5.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelUser6))
            {
                refreshTable();
                
                if (!(fLevelUser6.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelUser7))
            {
                refreshTable();
                
                if (!(fLevelUser7.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
            else if ((e.getSource() == fLevelUser8))
            {
                refreshTable();
                
                if (!(fLevelUser8.isSelected()))
                {
                    fLevelAll.setSelected(false);
                }
                else if (allLevelsSelected())
                {
                    fLevelAll.setSelected(true);
                }
            }
        }   

    }
    
    public boolean allLevelsSelected()
    {
        if (fLevelFatal.isSelected() &&
            fLevelError.isSelected() &&
            fLevelWarning.isSelected() &&
            fLevelInfo.isSelected() &&
            fLevelTrace.isSelected() &&
            fLevelTrace2.isSelected() &&
            fLevelTrace3.isSelected() &&
            fLevelDebug.isSelected() &&
            fLevelDebug2.isSelected() &&
            fLevelDebug3.isSelected() &&
            fLevelStart.isSelected() &&
            fLevelStop.isSelected() &&
            fLevelPass.isSelected() &&
            fLevelFail.isSelected() && 
            fLevelStatus.isSelected() &&
            fLevelUser1.isSelected() &&
            fLevelUser2.isSelected() &&
            fLevelUser3.isSelected() &&
            fLevelUser4.isSelected() &&
            fLevelUser5.isSelected() &&
            fLevelUser6.isSelected() &&
            fLevelUser7.isSelected() &&
            fLevelUser8.isSelected())
        {
            return true;
        }
        else
        {
            return false;
        }
    }
    
    public Vector refreshTable()
    {
        Vector logLines = getLogLines();
        if (logLines == null) return null;
                
        fLogTable.setRowHeight(30);
        fLogTable.setModel(new 
            STAXMonitorTableModel(logLines, columnNames));
                                      
        updateLogTableRenderers();
        STAXMonitorUtil.updateRowHeights(fLogTable, 2);
        STAXMonitorUtil.sizeColumnsToFitText(fLogTable);
        
        return logLines;
    }
    
    public void updateLogTableRenderers()
    {
        fLogTable.getColumnModel().getColumn(0).setCellRenderer(
            new STAXLogTableCellRenderer());
    	  
        fLogTable.getColumnModel().getColumn(0).
            setHeaderRenderer(new STAXLogTableCellRenderer(
                new Font("Dialog", Font.BOLD, 12), Color.lightGray));
                
        fLogTable.getColumnModel().getColumn(1).setCellRenderer(
            new STAXLogTableCellRenderer());
    	  
        fLogTable.getColumnModel().getColumn(1).
            setHeaderRenderer(new STAXLogTableCellRenderer(
                new Font("Dialog", Font.BOLD, 12), Color.lightGray));
                
        fLogTable.getColumnModel().getColumn(2).setCellRenderer(
            new STAXLogTableCellRenderer());
    	  
        fLogTable.getColumnModel().getColumn(2).
            setHeaderRenderer(new STAXLogTableCellRenderer(
                new Font("Dialog", Font.BOLD, 12), Color.lightGray));
    }

    
    public class STAXLogTableCellRenderer
             extends JTextArea implements TableCellRenderer
    {        
        public Hashtable rowHeights = new Hashtable();
        private boolean isHeader = true;
        
        public STAXLogTableCellRenderer()
        {
            this(new Font("Dialog", Font.PLAIN, 12), Color.white);
            isHeader = false;        
        }
        
        public STAXLogTableCellRenderer(Font font, Color background)
        {            
            setFont(font);
            setOpaque(true);
            setBackground(background);
            setForeground(Color.black);
            //setHorizontalAlignment(SwingConstants.LEFT);
            
            setBorder(BorderFactory.createRaisedBevelBorder());         
        }
        
        public void clearRowHeights()
        {            
            rowHeights.clear();
        }
    
        public Component getTableCellRendererComponent(JTable table, 
                                                       Object value,
                                                       boolean isSelected, 
                                                       boolean hasFocus,
                                                       int row, int col)
        {
            if (isHeader)
            {
                setBackground(Color.lightGray);
            }
            else if (isSelected)
            {                
                setBackground(UIManager.getColor("Table.selectionBackground"));
            }
            else
            {
                setBackground(Color.white);                
            }
            
            setText((value == null) ? "" : String.valueOf(value));
                                            
            return this;
        }       
    }
        
    private STAFHandle fHandle;
    private String fMachine = "local";
    private String fLogName = null;
    private String fJobLogName;
    private String fStartTimestamp;
    private JTable fLogTable;

    boolean fSystemExit = true;

    STAXMonitorTableModel fLogTableModel;
    STAXMonitorTableSorter fLogModelSorter;
    
    String fStartDateTime = null;
    Vector columnNames;

    Component parent;
    JMenu fFileMenu;
    JMenu fViewMenu;
    JMenuItem fFileExit;
    JMenuItem fViewRefresh;
    JMenu fLevels;
    JCheckBoxMenuItem fLevelAll;
    JCheckBoxMenuItem fLevelFatal;
    JCheckBoxMenuItem fLevelError;
    JCheckBoxMenuItem fLevelWarning;
    JCheckBoxMenuItem fLevelInfo;
    JCheckBoxMenuItem fLevelTrace;
    JCheckBoxMenuItem fLevelTrace2;
    JCheckBoxMenuItem fLevelTrace3;
    JCheckBoxMenuItem fLevelDebug;
    JCheckBoxMenuItem fLevelDebug2;
    JCheckBoxMenuItem fLevelDebug3;
    JCheckBoxMenuItem fLevelStart;
    JCheckBoxMenuItem fLevelStop;
    JCheckBoxMenuItem fLevelPass;
    JCheckBoxMenuItem fLevelFail;
    JCheckBoxMenuItem fLevelStatus;
    JCheckBoxMenuItem fLevelUser1;
    JCheckBoxMenuItem fLevelUser2;
    JCheckBoxMenuItem fLevelUser3;
    JCheckBoxMenuItem fLevelUser4;
    JCheckBoxMenuItem fLevelUser5;
    JCheckBoxMenuItem fLevelUser6;
    JCheckBoxMenuItem fLevelUser7;
    JCheckBoxMenuItem fLevelUser8;
}