/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2002                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service.stax;
import com.ibm.staf.*;
import java.util.*;
import java.text.*;

public class STAXLogAction implements STAXAction
{
    public STAXLogAction()
    { /* Do Nothing */ }

    public STAXLogAction(String message, String level, int logfile)
    { 
        fUnevalMessage = message;
        fMessage = message;
        fUnevalLevel = level;
        fLevel = level;
        fUnevalIf = "1";
        fIf = true;
        fLogfile = logfile;
    }

    public String getMessage() { return fMessage; } 
    public void setMessage(String message)
    {
        fUnevalMessage = message;
        fMessage = message;
    }

    public String getLevel() { return fLevel; }
    public void setLevel(String level)
    { 
        fUnevalLevel = level;
        fLevel = level;
    }

    public boolean getIf() { return fIf; }
    public void setIf(String ifValue)
    { 
        fUnevalIf = ifValue;
    }

    public int getLogfile() { return fLogfile; } 
    public void setLogfile(int logfile) { fLogfile = logfile; }

    public String getXMLInfo()
    {
        return "<log level=\"" + fUnevalLevel + "\" if=\"" + fUnevalIf +
               "\">" + fUnevalMessage + "</log>";
    } 

    public String getInfo()
    {
        int msgLength = fMessage.length();
        if (msgLength > 40)
            return fMessage.substring(0, 40) + "...";
        else
            return fMessage;
    } 

    public String getDetails()
    {
        return "Level:" + fLevel +
               ";If:" + fIf +
               ";Message:" + fMessage +
               ";Logfile:" + fLogfile;
    } 

    public void execute(STAXThread thread)
    {         
        fThread = thread;
        
        try
        {
            fIf = thread.pyBoolEval(fUnevalIf);
            
            if (!fIf)
            {   // Ignore log element if "if" attribute evaluates to FALSE
                fThread.popAction();
                return;
            }

            fMessage = thread.pyStringEval(fUnevalMessage);
            fLevel = thread.pyStringEval(fUnevalLevel);
        }
        catch (STAXPythonEvaluationException e)
        {
            fThread.popAction();

            fThread.pySetVar("STAXPythonEvalMsg", 
                             "\n" + getXMLInfo() + "\n\n" + e);
            fThread.raiseSignal("STAXPythonEvaluationError");
            return;
        }
        
        STAFResult result = fThread.getJob().log(fLogfile, fLevel, fMessage);

        if (result.rc != 0 && result.rc != 2)
        { 
            fThread.popAction();

            fThread.pySetVar("STAXLogMsg", 
                             "  " + getXMLInfo() + 
                             "  RC=" + result.rc +
                             ", Result=" + result.result);
            fThread.raiseSignal("STAXLogError");
            return;
        } 
         
        fThread.popAction();
    }

    public void handleCondition(STAXThread thread, STAXCondition cond)
    {
        thread.popAction();
    }

    public STAXAction cloneAction()
    {
        STAXLogAction clone = new STAXLogAction();

        clone.fUnevalMessage = fUnevalMessage;
        clone.fMessage = fMessage;
        clone.fUnevalLevel = fUnevalLevel;
        clone.fLevel = fLevel;
        clone.fUnevalIf = fUnevalIf;
        clone.fIf = fIf;
        clone.fLogfile = fLogfile;

        return clone;
    }   

    STAXThread fThread = null;    
    private String fUnevalMessage = new String();
    private String fMessage = new String();
    private String fLevel = new String();
    private String fUnevalLevel = new String();
    private boolean fIf = true;
    private String fUnevalIf = new String();
    private int fLogfile;
}
