/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2002                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service.stax;
import com.ibm.staf.*;
import java.util.HashMap;

public class STAXMessageAction implements STAXAction
{
    public STAXMessageAction()
    { /* Do Nothing */ }

    public STAXMessageAction(String messageValue)
    { 
        fUnevalMessageValue = messageValue;
        fMessageValue = messageValue;
        fUnevalIf = "1";
        fIf = true;
    }

    public String getMessageValue() { return fMessageValue; } 

    public void setMessageValue(String messageValue) 
    {
        fUnevalMessageValue = messageValue; 
        fMessageValue = messageValue; 
    }

    public boolean getIf() { return fIf; }
    public void setIf(String ifValue)
    { 
        fUnevalIf = ifValue;
    }

    public String getXMLInfo()
    {
        return "<message if=\"" + fIf + "\">" + fMessageValue + "</message>";
    }

    public String getInfo()
    {
        int msgLength = fMessageValue.length();
        if (msgLength > 40)
            return fMessageValue.substring(0, 40) + "...";
        else
            return fMessageValue;
    }

    public String getDetails()
    {
        return "If:" + fIf + "MessageValue:" + fMessageValue;
    }

    public void execute(STAXThread thread)
    {         
        thread.popAction();

        try
        {
            fIf = thread.pyBoolEval(fUnevalIf);

            if (!fIf)
            {   // Ignore message element if "if" attribute evaluates to FALSE
                return;
            }

            fMessageValue = thread.pyStringEval(fUnevalMessageValue);
        }
        catch (STAXPythonEvaluationException e)
        {
            thread.pySetVar("STAXPythonEvalMsg", 
                             "\n" + getXMLInfo() + "\n\n" + e);
            thread.raiseSignal("STAXPythonEvaluationError");
            return;
        }        

        String currentBlockName;
        try
        {
            currentBlockName = thread.pyStringEval("STAXCurrentBlock");
        }
        catch (STAXPythonEvaluationException e)
        {
            currentBlockName = "";  //Shouldn't happen
        }
                
        STAXTimestamp timestamp = new STAXTimestamp();

        HashMap messageMap = new HashMap();
        messageMap.put("messagetext", timestamp.getTimestampString() + 
            " " + fMessageValue);
        messageMap.put("block", currentBlockName);
        
        thread.getJob().generateEvent(STAXMessageActionFactory.STAX_MESSAGE,
            messageMap);
    }

    public void handleCondition(STAXThread thread, STAXCondition cond)
    {
        thread.popAction();
    }

    public STAXAction cloneAction()
    {
        STAXMessageAction clone = new STAXMessageAction();

        clone.fUnevalMessageValue = fUnevalMessageValue;
        clone.fMessageValue = fMessageValue;
        clone.fUnevalIf = fUnevalIf;
        clone.fIf = fIf;

        return clone;
    }   

    private String fUnevalMessageValue = new String();
    private String fMessageValue = new String();
    private boolean fIf = true;
    private String fUnevalIf = new String();
}
