/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2002                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service.stax;
import com.ibm.staf.STAFResult;

public class STAXSTAFMessage
{
    public STAXSTAFMessage(String result)
    {
        // Break up the STAFResult

        int priorityPos = result.indexOf(';');
        int timestampPos = result.indexOf(';', priorityPos + 1);
        int machinePos = result.indexOf(';', timestampPos + 1);
        int processPos = result.indexOf(';', machinePos + 1);
        int handlePos = result.indexOf(';', processPos + 1);

        try
        {
            fHandle = Integer.parseInt(result.substring(processPos + 1, 
                                                        handlePos));
        }
        catch (NumberFormatException e)
        {
            // XXX: Do what?
        }

        fMachine = result.substring(timestampPos + 1, machinePos);
        fMessage = result.substring(handlePos + 1);

        if (false)
        {
            System.out.println("PriorityPos : " + priorityPos);
            System.out.println("TimestampPos: " + timestampPos);
            System.out.println("MachinePos  : " + machinePos);
            System.out.println("ProcessPos  : " + processPos);
            System.out.println("HandlePos   : " + handlePos);
            System.out.println("Message     : " + fMessage);
        }

        if (fMessage.startsWith("STAF/RequestComplete"))
        {
            int requestPos = fMessage.indexOf(';');
            int rcPos = fMessage.indexOf(';', requestPos + 1);

            fRequestResult = fMessage.substring(rcPos + 1);

            try
            {
                // Note: 21 = length of "STAF/RequestComplete"
                fRequestNumber = Integer.parseInt(fMessage.substring(21,
                                                  requestPos));
                fRequestRC = Integer.parseInt(fMessage.substring(
                             requestPos + 1, rcPos));
            }
            catch (NumberFormatException e)
            {
                System.out.println("STAXSTAFMessage: " +
                                   "fRequestNumber/fRequestRC - " +
                                   e.toString());
            }

            if (false)
            {
                System.out.println("Request number: " + fRequestNumber);
                System.out.println("Request rc    : " + fRequestRC);
                System.out.println("Request result: " + fRequestResult);
            }
        } 

        else if (fMessage.startsWith("STAF/PROCESS/END"))
        {
            int processHandlePos = fMessage.indexOf(';');
            int processTimestampPos = fMessage.indexOf(';', processHandlePos + 1);
            fProcessTimestamp = fMessage.substring(processHandlePos + 1,
                                                  processTimestampPos);

            try
            {
                // Note: 17 = length of "STAF/PROCESS/END"
                fProcessHandle = Integer.parseInt(fMessage.substring(17,
                                                  processHandlePos));
                                             
                String processRCAndResult = 
                    fMessage.substring(processTimestampPos + 1);
                    
                int processReturnFilesIndex = processRCAndResult.indexOf(";");
                
                if (processReturnFilesIndex == -1)
                {
                    fProcessRC = Long.parseLong(processRCAndResult);
                }
                else
                {
                    fProcessRC = Long.parseLong(processRCAndResult.
                        substring(0, processReturnFilesIndex));
                        
                    fProcessResult = processRCAndResult.substring(
                        processReturnFilesIndex + 1);
                    
                    if (fProcessResult.startsWith("KEY="))
                    {
                        int keyIndex = fProcessResult.indexOf(";");
                        
                        if (keyIndex == -1)
                        {
                            // Note: 4 = length of "KEY="
                            fProcessKey = fProcessResult.substring(4);
                            fProcessResult = "";
                        }
                        else
                        {
                            fProcessKey = fProcessResult.substring(4, keyIndex);
                            
                            fProcessResult = fProcessResult.
                                substring(keyIndex + 1);
                        }
                    }
                }
            }
            catch (NumberFormatException e)
            {
                System.out.println("STAXSTAFMessage: fProcessRC - " +
                                   e.toString());
            }

            if (false)
            {
                System.out.println("Process handle   : " + fProcessHandle);
                System.out.println("Process rc       : " + fProcessRC);
                System.out.println("Process timestamp: " + fProcessTimestamp);
            }
        }
    }

    public String toString()
    {
        return ("Handle: " + fHandle + ", Machine: " + fMachine +
                ", Message: " + fMessage);
    }

    public int getHandle() { return fHandle; }
    
    public String getMachine() { return fMachine; }
    
    public String getMessage() { return fMessage; }

    public int getRequestNumber() { return fRequestNumber; }
    public int getRequestRC() { return fRequestRC; }
    public String getRequestResult() { return fRequestResult; }

    public int getProcessHandle() { return fProcessHandle; }
    public long getProcessRC() { return fProcessRC; }
    public String getProcessKey() { return fProcessKey; }
    public String getProcessTimestamp() { return fProcessTimestamp; }
    public String getProcessResult() { return fProcessResult; }

    // Common STAF Message Result fields

    int fHandle = 0;
    String fMachine;
    String fMessage;

    // STAF/RequestComplete message prefix fields

    int fRequestNumber = 0;
    int fRequestRC = 0;
    String fRequestResult;

    // STAF/PROCESS/END message prefix fields

    int fProcessHandle = 0;
    long fProcessRC = 0;
    String fProcessKey = "";
    String fProcessTimestamp;
    String fProcessResult = "";
}