/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service.whoami;

import com.ibm.staf.*;
import com.ibm.staf.service.*;

public class WhoAmI implements STAFServiceInterfaceLevel1
{
    public WhoAmI() { /* Do Nothing */ }

    public int init(String name, String parms)
    {
        int rc = STAFResult.Ok;

        try
        {
            fHandle = new STAFHandle("STAF/Service/" + name);
        }
        catch (STAFException e)
        {
            rc = STAFResult.STAFRegistrationError;
        }

        return rc;
    }

    public STAFResult acceptRequest(String machine, String process, int handle,
                                    String request)
    {
        STAFResult effectiveMachine = fHandle.submit2("LOCAL", "MISC",
                                                      "MACHINE :" +
                                                      machine.length() + ":" +
                                                      machine);

        if (effectiveMachine.rc != 0) return effectiveMachine;

        String result = "Machine     : " + machine + "\r\n" +
                        "Eff. Machine: " + effectiveMachine.result + "\r\n" +
                        "Process     : " + process + "\r\n" +
                        "Handle      : " + handle + "\r\n" +
                        "Request     : " + request + "\r\n";

        return new STAFResult(STAFResult.Ok, result);
    }

    public int term()
    {
        // Un-register the service handle

        try
        {
            fHandle.unRegister();
        }
        catch (STAFException ex)
        {
            return STAFResult.STAFRegistrationError;
        }

        return STAFResult.Ok;
    }

    private STAFHandle fHandle;
}
