/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

#include "STAF.h"
#include "STAFString.h"
#include "STAF_iostream.h"
#include <stdlib.h>

STAFString massageArgument(char *theArg)
{
    STAFString arg(theArg);

    if (getenv("STAF_OLDCLI") != 0) return arg;

    if (arg.find(kUTF8_SPACE) != STAFString::kNPos)
    {
        if ((arg.find(kUTF8_DQUOTE) != 0) ||
            (arg.findLastOf(kUTF8_DQUOTE, STAFString::kNPos,
                               STAFString::kChar) !=
             arg.length(STAFString::kChar)))
        {
            arg = kUTF8_COLON + STAFString(arg.length(STAFString::kChar)) +
                  kUTF8_COLON + arg;
        }
    }

    return arg;
}


int main(int argc, char **argv)
{
    if (argc < 4)
    {
        cout << "Usage: STAF <TCPIP HostName | LOCAL> <Service> "
             << "<Request>" << endl;
        return 1;
    }

    unsigned int verboseOutput = (getenv("STAF_QUIET_MODE") == 0) ? 1 : 0;
    unsigned int staticHandle = 0;

    if (getenv("STAF_STATIC_HANDLE") != 0)
        staticHandle = strtoul(getenv("STAF_STATIC_HANDLE"), 0, 10);
    
    STAFHandlePtr handlePtr;
    unsigned int rc = 0;

    if (staticHandle != 0)
        rc = STAFHandle::create(staticHandle, handlePtr);
    else
        rc = STAFHandle::create("STAF/Client", handlePtr);

    if (rc != 0)
    {
        cout << "Error registering with STAF, RC: " << rc << endl;
        return rc;
    }

    STAFString request;

    for (int i = 3; i < (argc - 1); i++)
    {
        request += massageArgument(argv[i]) + STAFString(" ");
    }

    request += massageArgument(argv[argc - 1]);
    STAFResultPtr result = handlePtr->submit(argv[1], argv[2], request,
                                             kSTAFReqSync);

    if (getenv("STAF_REPLACE_NULLS") != 0)
    {
        STAFString replaceNullStr = getenv("STAF_REPLACE_NULLS");
        result->result = result->result.replace(kUTF8_NULL, replaceNullStr);
        result->result = result->result.replace(kUTF8_NULL2, replaceNullStr);
    }

    if (result->rc != 0)
    {
        if (verboseOutput)
        {
            cout << "Error submitting request, RC: " << result->rc << endl;

            if (result->result.length() != 0)
                cout << "Additional info: " << result->result << endl;
        }
        else cout << result->result;

        return result->rc;
    }

    if (verboseOutput)
    {
        cout << "Response" << endl << "--------" << endl;
        cout << result->result << endl;
    }
    else cout << result->result;

    return 0;
}
