/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

#ifndef STAF_ConnectionFactory
#define STAF_ConnectionFactory

#include "STAFConnection.h"
#include "STAFRefPtr.h"


// STAFConnectionFactory - An interface for classes that generate connections
//
// This class is meant to be a generic front from which users can receive
// STAFConnections.  This class is meant to be derived from and work in
// conjunction with a particular STAFConnection derived class.  The derived
// factory class then allows users to obtain STAFConnections through this
// interface.
//
// Facilities are provided to obtain an STAFConnection.
//
// A derived class must override the acceptConnection() function.

class STAFConnectionFactory;  // Forward declaration needed for typdef

typedef STAFRefPtr<STAFConnectionFactory> STAFConnFactoryPtr;

class STAFConnectionFactory
{
public:

    STAFConnectionFactory() { /* Do Nothing */ }

    virtual STAFConnAllocInfo acceptConnection() = 0;

    virtual ~STAFConnectionFactory() { /* Do Nothing */ }

private:

    // Don't allow copy construction or assignment
    STAFConnectionFactory(const STAFConnectionFactory &);
    STAFConnectionFactory &operator=(const STAFConnectionFactory &);
};

#endif
