/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

#ifndef STAF_SocketConnection
#define STAF_SocketConnection

#include "STAF.h"
#include "STAFString.h"
#include "STAFSocketFuncs.h"
#include "STAFConnection.h"

// STAFSocketConnection - Derives from STAFConnection to provide support for
//                        sockets connections.

class STAFSocketConnection : public STAFConnection
{
public:

    static STAFConnAllocInfo create(STAFString host, unsigned short port,
                                    unsigned int timeout = 0);
    static STAFConnAllocInfo create(int socket);

    // Reads size bytes into buffer
    virtual void read(void *buffer, unsigned int size);

    // Writes size bytes from buffer
    virtual void write(void *buffer, unsigned int size);

    static void freeSocketConnection(STAFConnection *pConn);

    virtual ~STAFSocketConnection();

private:

    // Don't allow copy construction or assignment
    STAFSocketConnection(const STAFSocketConnection &);
    STAFSocketConnection &operator=(const STAFSocketConnection &);

    // User's must use the create functions
    STAFSocketConnection(STAFString host, unsigned short port,
                         unsigned int timeout = 0);
    STAFSocketConnection(int socket);

    STAFSocket_t fSocket;

    // This buffer size should probably be specified at constructor time
    // Better yet, it should be allocated in the read and write calls so that
    // this class becomes thread safe.
    char fBuffer[4096];
};

#endif
