/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

#include "STAF.h"
#include <fcntl.h>
#include "STAFSocketFuncs.h"
#include "STAFSocketConnectionFactory.h"
#include "STAFUtil.h"
#include "STAFString.h"

STAFSocketConnectionFactory::STAFSocketConnectionFactory(unsigned short port)
{
    fSocket = STAFSocketFuncs::socket(PF_INET, SOCK_STREAM, 0);

    if (!STAFUtilIsValidSocket(fSocket))
    {
        STAFString mesg = STAFString("No socket available") +
                          STAFString(STAFSocketFuncs::sock_errno());
        STAFException noSocketAvailable(mesg.toCurrentCodePage()->buffer());
        THROW_STAF_EXCEPTION(noSocketAvailable);
    }

    // This call ensures that the file descriptor is closed when we fork() and
    // exec().
    // XXX: Need to move to STAFOSUtil
    // XXX: fcntl(fSocket, F_SETFD, 1);  no fcntl call on win32

    struct sockaddr_in serverAddress = { 0 };

    serverAddress.sin_family = AF_INET;
    serverAddress.sin_port = STAFSocketFuncs::bswap(port);
    serverAddress.sin_addr.s_addr = INADDR_ANY;

    int bindRC = STAFSocketFuncs::bind(fSocket,
                 (struct sockaddr *)&serverAddress, sizeof(serverAddress));

    if (bindRC != 0)
    {
        STAFString mesg = STAFString("Error binding server socket, rc = ") +
                          STAFString(STAFSocketFuncs::sock_errno());
        STAFException bindError(mesg.toCurrentCodePage()->buffer());
        THROW_STAF_EXCEPTION(bindError);
    }

    int listenRC = STAFSocketFuncs::listen(fSocket, SOMAXCONN);

    if (listenRC != 0)
    {
        STAFString mesg = STAFString("Error listening on server socket, rc = ") +
                          STAFString(STAFSocketFuncs::sock_errno());
        STAFException listenError(mesg.toCurrentCodePage()->buffer());
        THROW_STAF_EXCEPTION(listenError);
    }
}

STAFConnAllocInfo STAFSocketConnectionFactory::acceptConnection()
{
    struct sockaddr_in clientAddress = { 0 };
    int clientAddressLength = sizeof(clientAddress);
    STAFSocket_t clientSocket = 0;

    do
    {
        clientSocket = STAFSocketFuncs::accept(fSocket,
                           (struct sockaddr *)&clientAddress,
                           &clientAddressLength);
    } while (!STAFUtilIsValidSocket(clientSocket) &&
             (STAFSocketFuncs::sock_errno() == SOCEINTR));

    if (!STAFUtilIsValidSocket(clientSocket))
    {
        STAFString mesg = STAFString("Error accepting on server socket, rc = ") +
                          STAFString(STAFSocketFuncs::sock_errno());
        STAFException acceptError(mesg.toCurrentCodePage()->buffer());
        THROW_STAF_EXCEPTION(acceptError);
    }

    return STAFSocketConnection::create(clientSocket);
}

STAFSocketConnectionFactory::~STAFSocketConnectionFactory()
{
    STAFSocketFuncs::soclose(fSocket);
}
