/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

#ifndef STAF_LocalConnection
#define STAF_LocalConnection

#include "STAFOSTypes.h"
#include "STAFSocketFuncs.h"
#include "STAFConnection.h"

// STAFLocalConnection - Derives from STAFConnection to provide support for
//                       local connections via UNIX domain sockets.

class STAFLocalConnection : public STAFConnection
{
public:

    static STAFConnAllocInfo create(STAFString localName,
                                    unsigned int timeout = 0);
    static STAFConnAllocInfo create(STAFSocket_t socket);

    // Reads size bytes into buffer
    virtual void read(void *buffer, unsigned int size);

    // Writes size bytes from buffer
    virtual void write(void *buffer, unsigned int size);

    static void freeLocalConnection(STAFConnection *pConn);

    virtual ~STAFLocalConnection();

private:

    // Don't allow copy construction or assignment
    STAFLocalConnection(const STAFLocalConnection &);
    STAFLocalConnection &operator=(const STAFLocalConnection &);

    // User's must use the create functions
    STAFLocalConnection(STAFString localName, unsigned int timeout = 0);
    STAFLocalConnection(STAFSocket_t socket);

    STAFSocket_t fSocket;

    // XXX: This buffer size should probably be specified at constructor time.
    // Better yet, it should be allocated in the read and write calls so that
    // this class becomes thread safe.
    char fBuffer[4096];
};

#endif
