/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

#include "STAFOSTypes.h"
#include "STAFException.h"
#include "STAFSocketFuncs.h"
#include "STAFString.h"
#include "STAFLocalConnectionFactory.h"
#include "STAFUtil.h"
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/un.h>

STAFLocalConnectionFactory::STAFLocalConnectionFactory(STAFString localName)
{
    // XXX: Major Hack to make sure socket pointers are available on the
    //      server side.  Yuck!
    if (STAFSocketFuncs::bswap == 0) STAFSocketFuncs::initSocketFuncs();

    fSocket = STAFSocketFuncs::socket(PF_UNIX, SOCK_STREAM, 0);

    if (!STAFUtilIsValidSocket(fSocket))
    {
        STAFString mesg = STAFString("No socket available") +
                          STAFString(STAFSocketFuncs::sock_errno());
        STAFException noSocketAvailable(mesg.toCurrentCodePage()->buffer());
        THROW_STAF_EXCEPTION(noSocketAvailable);
    }

    // This call ensures that the file descriptor is closed when we
    // fork() and exec()
    fcntl(fSocket, F_SETFD, 1);

    struct sockaddr_un serverAddress = { 0 };

    serverAddress.sun_family = AF_UNIX;
    strcpy(serverAddress.sun_path, "/tmp/STAFIPC_");
    strcat(serverAddress.sun_path, localName.toCurrentCodePage()->buffer());
    // XXX: sun_len is not available on Linux, what to do?
    // serverAddress.sun_len = strlen(serverAddress.sun_path) + 1;

    // We attempt to remove the socket path in case it was left around
    // from a previous invocation
    unlink(serverAddress.sun_path);

    int bindRC = STAFSocketFuncs::bind(fSocket,
                 (struct sockaddr *)&serverAddress, sizeof(serverAddress));

    if (bindRC != 0)
    {
        STAFString mesg = STAFString("Error binding server socket, rc = ") 
                        + STAFString(STAFSocketFuncs::sock_errno()); 
        STAFException bindError(mesg.toCurrentCodePage()->buffer());
        THROW_STAF_EXCEPTION(bindError);
    }

    int listenRC = STAFSocketFuncs::listen(fSocket, SOMAXCONN);

    if (listenRC != 0)
    {
        STAFString mesg = STAFString("Error listening on server socket, ") +
                          STAFString("rc = ") +
                          STAFString(STAFSocketFuncs::sock_errno()); 
        STAFException listenError(mesg.toCurrentCodePage()->buffer());
        THROW_STAF_EXCEPTION(listenError);
    }
}

STAFConnAllocInfo STAFLocalConnectionFactory::acceptConnection()
{
  //    struct sockaddr_in clientAddress = { 0 };
    struct sockaddr clientAddress;
    int clientAddressLength = sizeof(clientAddress);
    STAFSocket_t clientSocket = 0;

    do
    {
        clientSocket = STAFSocketFuncs::accept(fSocket,
                          (struct sockaddr *)&clientAddress,
                          &clientAddressLength);
    } while (!STAFUtilIsValidSocket(clientSocket) &&
             (STAFSocketFuncs::sock_errno() == SOCEINTR));

    if (!STAFUtilIsValidSocket(clientSocket))
    {
        STAFString mesg = STAFString("Error accepting on server socket, ") +
                          STAFString("rc = ") +
                          STAFString(STAFSocketFuncs::sock_errno()); 
        STAFException acceptError(mesg.toCurrentCodePage()->buffer());
        THROW_STAF_EXCEPTION(acceptError);
    }

    return STAFLocalConnection::create(clientSocket);
}

STAFLocalConnectionFactory::~STAFLocalConnectionFactory()
{
    STAFSocketFuncs::soclose(fSocket);
}
