/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

#include "STAFSocketFuncs.h"
#include "STAF_iostream.h"

// From wsock32.dll

SOCKET (SYSLINK * STAFSocketFuncs::accept)(SOCKET, struct sockaddr *, int *) = 0;
int (SYSLINK * STAFSocketFuncs::bind)(SOCKET, struct sockaddr *, int) = 0;
int (SYSLINK * STAFSocketFuncs::connect)(SOCKET, struct sockaddr *, int) = 0;
int (SYSLINK * STAFSocketFuncs::gethostname)(char *, int);
int (SYSLINK * STAFSocketFuncs::listen)(SOCKET, int) = 0;
int (SYSLINK * STAFSocketFuncs::recv)(SOCKET, char *, int, int) = 0;
int (SYSLINK * STAFSocketFuncs::send)(SOCKET, char *, int, int) = 0;
int (SYSLINK * STAFSocketFuncs::select)(int, fd_set *, fd_set *, fd_set *, struct timeval *) = 0;
int (SYSLINK * STAFSocketFuncs::sock_errno)(void) = 0;
SOCKET (SYSLINK * STAFSocketFuncs::socket)(int, int, int) = 0;
int (SYSLINK * STAFSocketFuncs::soclose)(SOCKET) = 0;
int (SYSLINK * STAFSocketFuncs::soshutdown)(SOCKET, int) = 0;

unsigned short   (SYSLINK * STAFSocketFuncs::bswap)(unsigned int short) = 0;
struct hostent * (SYSLINK * STAFSocketFuncs::gethostbyaddr)(const char *, int, int) = 0;
struct hostent * (SYSLINK * STAFSocketFuncs::gethostbyname)(const char *) = 0;
unsigned long    (SYSLINK * STAFSocketFuncs::inet_addr)(const char *) = 0;
unsigned long    (SYSLINK * STAFSocketFuncs::lswap)(unsigned long) = 0;

typedef int (SYSLINK * StartupFunc)(WORD, LPWSADATA);
static int (SYSLINK * winioctlsocket)(SOCKET, long, u_long *);

int STAFSocketFuncs::initSocketFuncs()
{
    int rc = 1;

    try
    {
        HINSTANCE theLibrary = LoadLibrary("WSOCK32");

        if (theLibrary == 0)
            return rc;

        accept         = (SOCKET (SYSLINK *)(SOCKET, struct sockaddr *, int *))
                         GetProcAddress(theLibrary, "accept");
        bind           = (int (SYSLINK *)(SOCKET, struct sockaddr *, int))
                         GetProcAddress(theLibrary, "bind");
        connect        = (int (SYSLINK *)(SOCKET, struct sockaddr *, int))
                         GetProcAddress(theLibrary, "connect");
        gethostname    = (int (SYSLINK *)(char *, int))
                         GetProcAddress(theLibrary, "gethostname");
        winioctlsocket = (int (SYSLINK *)(SOCKET, long, u_long *))
                         GetProcAddress(theLibrary, "ioctlsocket");
        listen         = (int (SYSLINK *)(SOCKET, int))
                         GetProcAddress(theLibrary, "listen");
        recv           = (int (SYSLINK *)(SOCKET, char *, int, int))
                         GetProcAddress(theLibrary, "recv");
        select         = (int (SYSLINK *)(int, fd_set *, fd_set *, fd_set *, struct timeval *))
                         GetProcAddress(theLibrary, "select");
        send           = (int (SYSLINK *)(SOCKET, char *, int, int))
                         GetProcAddress(theLibrary, "send");
        sock_errno     = (int (SYSLINK *)(void))
                         GetProcAddress(theLibrary, "WSAGetLastError");
        socket         = (SOCKET (SYSLINK *)(int, int, int))
                         GetProcAddress(theLibrary, "socket");
        soclose        = (int (SYSLINK *)(SOCKET))
                         GetProcAddress(theLibrary, "closesocket");
        soshutdown     = (int (SYSLINK *)(SOCKET, int))
                         GetProcAddress(theLibrary, "shutdown");
        bswap          = (unsigned short   (SYSLINK *)(unsigned int short))
                         GetProcAddress(theLibrary, "htons");
        gethostbyaddr  = (struct hostent * (SYSLINK *)(const char *, int, int))
                         GetProcAddress(theLibrary, "gethostbyaddr");
        gethostbyname  = (struct hostent * (SYSLINK *)(const char *))
                         GetProcAddress(theLibrary, "gethostbyname");
        inet_addr      = (unsigned long    (SYSLINK *)(const char *))
                         GetProcAddress(theLibrary, "inet_addr");
        lswap          = (unsigned long    (SYSLINK *)(unsigned long))
                         GetProcAddress(theLibrary, "htonl");

        // Now, initialize the socket functions

        StartupFunc WinSockStartup = (StartupFunc)GetProcAddress(theLibrary,
                                     "WSAStartup");

        WORD version = MAKEWORD(1, 1);
        WSADATA wsaData = { 0 };

        rc = WinSockStartup(version, &wsaData);

        if ((rc == 0) && (wsaData.wVersion != version))
            rc = 1;
    }
    catch (STAFException &se)
    { se.write("STAFSocketFuncs::initSocketFuncs()"); }
    catch (...)
    {
        cout << "Caught unknown exception in "
             << "STAFSocketFuncs::initSocketFuncs()" << endl;
    }

    return rc;
}


int STAFSocketFuncs::getRealHostName(STAFString &hostname)
{
    char buffer[256];

    if (STAFSocketFuncs::gethostname(buffer, sizeof(buffer)) == SOCKET_ERROR)
    {
        cout << "Error on gethostname()" << endl;
        return 1;
    }

    struct hostent *host = STAFSocketFuncs::gethostbyname(buffer);

    if (host == 0)
    {
        cout << "Error on gethostbyname()" << endl;
        return 1;
    }

    STAFString hostname1(host->h_name);

    if (hostname1.count(kUTF8_PERIOD) > 2)
    {
        hostname = hostname1;
        return 0;
    }

    int addr = *(int *)host->h_addr;

    host = STAFSocketFuncs::gethostbyaddr((char *)&addr, sizeof(addr), AF_INET);

    if (host == 0)
    {
        cout << "Error on gethostbyaddr()" << endl;
        return 1;
    }

    STAFString hostname2(host->h_name);

    if (hostname2.count(kUTF8_PERIOD) > 2)
    {
        hostname = hostname1;
        return 0;
    }

    if (hostname1.length() > hostname2.length())
        hostname = hostname1;
    else
        hostname = hostname2;

    return 0;
}


int STAFSocketFuncs::setBlockingMode(SOCKET theSocket,
                                   STAFSocketFuncs::BlockingMode blockingMode)
{
    u_long nonBlocking = (blockingMode == kBlocking) ? 0 : 1;
    int ioctlRC = winioctlsocket(theSocket, FIONBIO, (u_long *)&nonBlocking);

    if (ioctlRC != 0)
        return STAFSocketFuncs::sock_errno();

    return 0;
}

STAFRC_t STAFSocketFuncs::getInAddrByName(const char *name, in_addr *addr,
                                          unsigned int *osRC)
{
    struct hostent *hostname = gethostbyname(name);

    if (hostname == 0)
    {
        if (osRC) *osRC = STAFSocketFuncs::sock_errno();
        return kSTAFBaseOSError;
    }

    addr->s_addr = *reinterpret_cast<unsigned long *>(hostname->h_addr);

    return kSTAFOk;
}
