/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

#include "STAF.h"
#include "STAFProc.h"
#include "STAFDelegatedService.h"
#include "STAFConnectionManager.h"

STAFDelegatedService::STAFDelegatedService(STAFString name, 
                                           STAFString machine,
                                           STAFString toName)
    : STAFService(name), fMachine(machine), fToName(toName.toUpperCase())
{
    /* Do Nothing */
}


STAFDelegatedService::~STAFDelegatedService()
{
    /* Do Nothing */
}


STAFString STAFDelegatedService::info(unsigned int) const
{
    return name() + ": Delegated to " + fToName + " on " + fMachine;
}


STAFServiceResult STAFDelegatedService::acceptRequest(
    const STAFServiceRequest &requestInfo)
{
    STAFConnPtr connection;
    STAFString result;
    STAFRC_t rc = gConnectionManagerPtr->makeConnection(fMachine,
                      connection, result);

    if (rc == kSTAFOk)
    {
        connection->writeUInt(1);      // API Number
        connection->writeUInt(0);      // API Level

        STAFRC_t ack = static_cast<STAFRC_t>(connection->readUInt());

        if (ack != kSTAFOk) return ack;

        connection->writeString(requestInfo.fMachine);
        connection->writeUInt(requestInfo.fHandle);
        connection->writeString(requestInfo.fHandleName);
        connection->writeString(fToName);
        connection->writeString(requestInfo.fRequest);

        rc = static_cast<STAFRC_t>(connection->readUInt());
        result = connection->readString();
    }

    return STAFServiceResult(rc, result);
}
