/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

#ifndef STAF_ServiceManager
#define STAF_ServiceManager

#include <map>
#include <list>
#include "STAF.h"
#include "STAFString.h"
#include "STAFService.h"
#include "STAFRefPtr.h"
#include "STAFMutexSem.h"

// STAFServiceManager - This class manages the set of STAF Services.  It allows
//                      you to add, delete, and replace services, retrieve
//                      services by name, and retrieve ordered or sequential
//                      lists of services.

class STAFServiceManager
{
public:

    STAFServiceManager() { /* Do Nothing */ }

    // Service management methods

    STAFRC_t add(STAFServicePtr theService);
    STAFRC_t addSLS(STAFServicePtr theService);
    STAFRC_t remove(STAFString name);
    STAFRC_t removeSLS(STAFServicePtr theService);
    STAFRC_t replace(STAFServicePtr theService, STAFString newName,
                     STAFString &errorBuffer);
    STAFRC_t rename(STAFString name, STAFString newName,
                    STAFString &errorBuffer);
    STAFRC_t get(STAFString name, STAFServicePtr &theService, 
                 bool addingService = false);

    // Service list methods

    typedef std::map<STAFString, STAFServicePtr> OrderedServiceList;
    typedef std::list<STAFServicePtr> ServiceList;

    OrderedServiceList getOrderedServiceListCopy();
    ServiceList getServiceListCopy();
    ServiceList getSLSListCopy();

    // Service tracing methods

    typedef std::set<STAFString> ServiceStringList;

    static void traceServices(const STAFString &services);
    static void traceAllServices();
    static ServiceStringList getTraceServices();
    static unsigned int doTraceService(const STAFString &service);

    ~STAFServiceManager() { /* Do Nothing */ }

private:

    // Don't allow copy construction or assignment
    STAFServiceManager(const STAFServiceManager &);
    STAFServiceManager &operator=(const STAFServiceManager &);

    STAFMutexSem fServiceListSem;
    OrderedServiceList fOrderedServiceList;
    ServiceList fServiceList;

    static STAFMutexSem sServiceStringListSem;
    static ServiceStringList sServiceStringList;
    
    STAFMutexSem fSLSListSem;
    ServiceList fSLSList;
};

#endif
