/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

#include "STAF.h"
#include "STAFString.h"
#include "STAFUtil.h"
#include "STAFProc.h"
#include "STAFSimpleServices.h"
#include "STAFTrustManager.h"
#include "STAFThreadManager.h"


STAFString STAFPingService::sPing("ping");
STAFString STAFPingService::sPong("PONG");

STAFServiceResult STAFPingService::acceptRequest(
    const STAFServiceRequest &requestInfo)
{
    if (requestInfo.fTrustLevel < 1) return kSTAFAccessDenied;

    if (requestInfo.fRequest.isEqualTo(sPing, STAFString::kCaseInsensitive))
        return STAFServiceResult(kSTAFOk, sPong);
    else
        return STAFServiceResult(kSTAFInvalidRequestString);
}

STAFString STAFPingService::info(unsigned int) const
{
    return name() + STAFString(": Internal");
}

STAFPingService::~STAFPingService()
{
    /* Do Nothing */
}


STAFServiceResult STAFEchoService::acceptRequest(
    const STAFServiceRequest &requestInfo)
{
    if (requestInfo.fTrustLevel < 2) return kSTAFAccessDenied;

    if (requestInfo.fRequest.subWord(0, 1).lowerCase() == "echo")
        return STAFServiceResult(kSTAFOk, requestInfo.fRequest.subWord(1));
    else
        return STAFServiceResult(kSTAFInvalidRequestString);
}

STAFString STAFEchoService::info(unsigned int) const
{
    return name() + STAFString(": Internal");
}

STAFEchoService::~STAFEchoService()
{
    /* Do Nothing */
}


STAFServiceResult STAFDelayService::acceptRequest(
    const STAFServiceRequest &requestInfo)
{
    if (requestInfo.fTrustLevel < 2) return kSTAFAccessDenied;

    if (requestInfo.fRequest.subWord(0, 1).lowerCase() != "delay")
        return STAFServiceResult(kSTAFInvalidRequestString);

    STAFString delayString = requestInfo.fRequest.subWord(1);

    if (!delayString.isDigits())
        return STAFServiceResult(kSTAFInvalidValue, delayString);

    int delay = delayString.asUInt();

    gThreadManagerPtr->sleepCurrentThread(delay);

    return STAFServiceResult(kSTAFOk);
}

STAFString STAFDelayService::info(unsigned int) const
{
    return name() + STAFString(": Internal");
}

STAFDelayService::~STAFDelayService()
{
    /* Do Nothing */
}
