/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

#include "STAF.h"
#include "STAFProc.h"
#include "STAFTrustManager.h"

unsigned int STAFTrustManager::trustLevel(STAFString machine)
{
    unsigned int pos = STAFString::kNPos;

    if ((gUseLongNames == 0) &&
        ((pos = machine.find(kUTF8_PERIOD)) != STAFString::kNPos))
    {
        machine = machine.subString(0, pos);
    }

    machine.lowerCase();

    if (machine == gMachineNamePtr->toLowerCase())
        return fMaxTrustLevel;

    STAFMutexSemLock trustLock(fTrustListSem);

    if (fTrustList.find(machine) != fTrustList.end())
        return fTrustList[machine].trustLevel;
    else
        return fDefaultTrustLevel;
}


STAFRC_t STAFTrustManager::setTrusteeLevel(STAFString machine,
                                                unsigned int trustLevel)
{
    if (trustLevel > fMaxTrustLevel)
        return kSTAFInvalidTrustLevel;

    machine.lowerCase();

    if (machine == *gMachineNamePtr)
        return kSTAFOk;

    STAFMutexSemLock trustLock(fTrustListSem);

    if (fTrustList.find(machine) == fTrustList.end())
        fTrustList[machine] = TrustData(machine, trustLevel);
    else
        fTrustList[machine].trustLevel = trustLevel;

    return kSTAFOk;
}


STAFRC_t STAFTrustManager::deleteTrustee(STAFString machine)
{
    machine.lowerCase();

    STAFMutexSemLock trustLock(fTrustListSem);

    if (fTrustList.find(machine) == fTrustList.end())
        return kSTAFTrusteeDoesNotExist;

    fTrustList.erase(machine);

    return kSTAFOk;
}


unsigned int STAFTrustManager::getDefaultTrusteeLevel()
{
    return fDefaultTrustLevel;
}


STAFRC_t STAFTrustManager::setDefaultTrusteeLevel(unsigned int trustLevel)
{
    if (trustLevel > fMaxTrustLevel)
        return kSTAFInvalidTrustLevel;

    STAFMutexSemLock trustLock(fTrustListSem);

    if (trustLevel > fMaxTrustLevel)
        return kSTAFInvalidTrustLevel;

    fDefaultTrustLevel = trustLevel;

    return kSTAFOk;
}


STAFTrustManager::TrustList STAFTrustManager::getTrustListCopy()
{
    STAFMutexSemLock trustLock(fTrustListSem);

    return fTrustList;
}
