/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

#ifndef STAF_TrustManager
#define STAF_TrustManager

#include <map>
#include "STAF.h"
#include "STAFString.h"
#include "STAFUtil.h"
#include "STAFRefPtr.h"
#include "STAFMutexSem.h"


class STAFTrustManager
{
public:

    // Public types

    struct TrustData
    {
        TrustData() { /* Do Nothing */ }

        TrustData(STAFString aMachine, unsigned int aTrustLevel)
            : lowerCaseMachine(aMachine.toLowerCase()), machine(aMachine),
              trustLevel(aTrustLevel)
        { /* Do Nothing */ }

        STAFString lowerCaseMachine;
        STAFString machine;
        unsigned int trustLevel;
    };

    typedef std::map<STAFString, TrustData> TrustList;

    STAFTrustManager(unsigned int maxTrustLevel,
                     unsigned int defaultTrustLevel = 3)
        : fMaxTrustLevel(maxTrustLevel), fDefaultTrustLevel(defaultTrustLevel)
    { /* Do Nothing */ }

    unsigned int trustLevel(STAFString machine);

    STAFRC_t setTrusteeLevel(STAFString machine,
                                  unsigned int trustLevel);
    STAFRC_t deleteTrustee(STAFString machine);

    unsigned int getDefaultTrusteeLevel();
    STAFRC_t setDefaultTrusteeLevel(unsigned int trustLevel);

    TrustList getTrustListCopy();

private:

    // Don't allow copy construction or assignment
    STAFTrustManager(const STAFTrustManager &);
    STAFTrustManager &operator=(const STAFTrustManager &);

    unsigned int fMaxTrustLevel;
    unsigned int fDefaultTrustLevel;
    STAFMutexSem fTrustListSem;
    TrustList fTrustList;
};


#endif
