/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

#ifndef STAF_VariablePool
#define STAF_VariablePool

#include <map>
#include "STAF.h"
#include "STAFString.h"
#include "STAFRefPtr.h"
#include "STAFMutexSem.h"
#include "STAFUtil.h"

// STAFVariablePoolPtr is also defined in STAFProc.h
class STAFVariablePool;
typedef STAFRefPtr<STAFVariablePool> STAFVariablePoolPtr;

class STAFVariablePool
{
public:

    struct Variable
    {
        Variable(const STAFString &aName = STAFString(),
                 const STAFString &aValue = STAFString())
            : name(aName), lowerCaseName(name.toLowerCase()),
              value(aValue)
        { /* Do Nothing */ }

        STAFString name;
        STAFString lowerCaseName;
        STAFString value;
    };

    typedef std::map<STAFString, Variable> VariableList;

    STAFVariablePool() { /* Do Nothing */ }

    STAFRC_t set(const STAFString &name, const STAFString &value);
    STAFRC_t get(const STAFString &name, STAFString &value) const;
    STAFRC_t del(const STAFString &name);
    STAFRC_t resolve(const STAFString &aString,
                          const STAFVariablePool *secondary,
                          STAFString &result) const;
    VariableList getVariableListCopy();

private:

    // Don't allow copy construction or assignment
    STAFVariablePool(const STAFVariablePool &);
    STAFVariablePool &operator=(const STAFVariablePool &);

    VariableList fVarList;
    STAFMutexSem fVarListSem;
};

#endif
